/*
 * Copyright 2022 BytePlus Pte. Ltd.
 * SPDX-License-Identifier: MIT
 */

package com.ss.bytertc.engine.flutter.base;

import android.annotation.SuppressLint;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.annotation.RestrictTo;
import androidx.annotation.UiThread;

import com.ss.bytertc.engine.RTCRoom;
import com.ss.bytertc.engine.RTCVideo;
import com.ss.bytertc.engine.handler.IRTCRoomEventHandler;
import com.ss.bytertc.engine.handler.IRTCVideoEventHandler;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public final class RTCVideoManager {
    @SuppressLint("StaticFieldLeak")
    private static Context mAppCxt;

    public static void setAppContext(Context cxt) {
        mAppCxt = cxt.getApplicationContext();
    }

    /**
     * The RTCVideo object of the global singleton.
     */
    private static RTCVideo sRtcVideo;

    /**
     * Create an engine instance.
     *
     * @param appId      A unique identifier for each App, randomly generated by the RTC console.
     * @param handler    Delegate sent from SDK to App.
     * @param parameters Reserved parameter, in JSON string format.
     */
    public static void create(String appId, IRTCVideoEventHandler handler, JSONObject parameters) {
        try {
            sRtcVideo = RTCVideo.createRTCVideo(mAppCxt, appId, handler, null, parameters);
        } catch (Exception e) {
            throw new RuntimeException("create engine error!", e);
        }
    }

    @UiThread
    public static void destroy() {
        Iterator<Map.Entry<Integer, RTCRoom>> iterator = sRTCRoomMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<Integer, RTCRoom> next = iterator.next();
            next.getValue().destroy();

            iterator.remove();
        }

        RTCVideo.destroy();
        sRtcVideo = null;
    }

    public static boolean hasRTCVideo() {
        return sRtcVideo != null;
    }

    @NonNull
    public static RTCVideo getRTCVideo() {
        if (sRtcVideo == null) {
            throw new IllegalStateException("RTCVideo not created YET.");
        }
        return sRtcVideo;
    }

    private static final HashMap<Integer, RTCRoom> sRTCRoomMap = new HashMap<>();

    private RTCVideoManager() {

    }

    /**
     * Create a Room and place it in the cache.
     *
     * @param roomInsId Room's object ID, cached KEY
     * @param roomId    Room ID
     * @return A RTCRoom instance.
     * @see #getRoom(Integer)
     */
    public static RTCRoom createRoom(@NonNull Integer roomInsId, @NonNull String roomId) {
        RTCVideo rtcVideo = getRTCVideo();
        RTCRoom room = rtcVideo.createRTCRoom(roomId);
        if (room != null) {
            sRTCRoomMap.put(roomInsId, room);
        }
        return room;
    }

    /**
     * Gets the room for the specified [insId].
     *
     * @param insId Room's object ID, cached KEY.
     * @return The cached RTCRoom object.
     * @see #createRoom(Integer, String)
     */
    public static RTCRoom getRoom(@NonNull Integer insId) {
        return sRTCRoomMap.get(insId);
    }

    /**
     * Destroys the room for the specified [insId] and removes it from the cache.
     *
     * @param roomIns Room's object ID, cached KEY.
     */
    public static void destroyRoom(@NonNull Integer roomIns) {
        RTCRoom room = sRTCRoomMap.remove(roomIns);
        if (room != null) {
            room.destroy();
        }
    }
}