// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'package:flutter/material.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:byteplus_rtc_example/room_page.dart';
import 'package:byteplus_rtc/byteplus_rtc.dart';

import 'constants.dart';

/// The Login UI of BytePlusRTC
///
/// Features:
/// - Login to start an RTC call;
/// - Verify the roomID and userID;
/// - Show the SDK version. [RTCVideo.getSdkVersion]
///
/// Note：
/// 1. The permissions of Android SDK are necessary: external memory IO, camera and microphone.For more information, refer to {@link android/app/src/main/AndroidManifest.xml}.
/// 2. The REGEX applies to roomId and userId: "^[a-zA-Z0-9@._|-]{1,128}$"
class LoginPage extends StatefulWidget {
  const LoginPage({Key? key}) : super(key: key);

  @override
  State<LoginPage> createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  final TextEditingController _roomIdTextController = TextEditingController();
  final TextEditingController _userIDTextController = TextEditingController();
  final FocusNode _roomIDFocusNode = FocusNode();
  final FocusNode _userIDFocusNode = FocusNode();

  String _version = "BytePlusRTC v";

  @override
  void initState() {
    super.initState();
    _requestPermission();

    /// Gets the current version number of the SDK.
    RTCVideo.getSdkVersion().then((value) {
      setState(() {
        _version = "BytePlusRTC v${value ?? ""}";
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: SafeArea(
            child: Stack(children: [
      Align(
          alignment: const Alignment(0, -0.25),
          child: Container(
            padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: [
                TextField(
                  focusNode: _roomIDFocusNode,
                  controller: _roomIdTextController,
                  decoration: const InputDecoration(labelText: "RoomId"),
                ),
                TextField(
                  focusNode: _userIDFocusNode,
                  controller: _userIDTextController,
                  decoration: const InputDecoration(labelText: "UserId"),
                ),
                ElevatedButton(
                  child: const Text(
                    "Enter Room",
                    textAlign: TextAlign.center,
                  ),
                  onPressed: () {
                    _joinRoom();
                  },
                )
              ],
            ),
          )),
      Positioned(
          left: 20,
          right: 20,
          bottom: 0,
          child: Text(
            _version,
            textAlign: TextAlign.center,
            style: const TextStyle(color: Colors.black, fontSize: 18),
          ))
    ])));
  }

  void _requestPermission() async {
    await [Permission.camera, Permission.microphone].request();
  }

  void _joinRoom() {
    String roomId = _roomIdTextController.text;
    String userId = _userIDTextController.text;
    if (roomId.isEmpty) {
      _showAlert('Room ID can not be empty!');
      return;
    }
    if (userId.isEmpty) {
      _showAlert('User ID can not be empty!');
      return;
    }
    if (_checkValidity(roomId) == false) {
      _showAlert('Room ID format error!');
      return;
    }
    if (_checkValidity(userId) == false) {
      _showAlert('User ID format error!');
      return;
    }
    _roomIDFocusNode.unfocus();
    _userIDFocusNode.unfocus();
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      return RoomPage(roomId: roomId, userId: userId);
    }));
  }

  bool _checkValidity(String string) => RegExp(inputRegexp).hasMatch(string);

  void _showAlert(String warning) => showDialog<String>(
        context: context,
        builder: (BuildContext context) => AlertDialog(
          content: Text(warning),
          actions: [
            TextButton(
              onPressed: () => Navigator.pop(context, 'OK'),
              child: const Text('OK'),
            ),
          ],
        ),
      );
}
