// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'package:flutter/material.dart';
import 'package:byteplus_rtc_example/user_live_view.dart';
import 'package:byteplus_rtc/byteplus_rtc.dart';

import 'constants.dart';

/// The UI of video call
/// The number of users in the same room is not limited in the demo.
/// No more than 4 views are shown on the page:
/// - local video view;
/// - views of no more than 3 remote users.
///
/// Features:
/// - create / destroy RTC engine
/// - set video publishing parameters
/// - render local / remote video
/// - join / leave the RTC room
/// - switch between front / rear cameras
/// - enable / disable microphone
/// - enable / disable camera
/// - switch between earpiece and speakerphone
///
/// Follow the following process to implement an RTC call:
/// 1. create the RTC engine: [RTCVideo.createRTCVideo]
/// 2. set encoder parameters: [RTCVideo.setMaxVideoEncoderConfig]
/// 3. start capturing local video: [RTCVideo.startVideoCapture]
/// 4. set local video canvas for rendering: [RTCViewContext.localContext]
/// 5. create an RTC room: [RTCVideo.createRTCRoom]
/// 6. join an RTC room: [RTCRoom.joinRoom]
/// 7. after receiving the first video frame of the remote user, set the canvas for video rendering: [RTCViewContext.remoteContext]
/// 8. leave the RTC room: [RTCRoom.leaveRoom]
/// 9. destroy the RTC room: [RTCRoom.destroy]
/// 10. destroy the RTC engine: [RTCVideo.destroy]
///
/// Notes:
/// 1. In the demo, the canvas for video rendering is set after the callback [RTCVideoEventHandler.onFirstRemoteVideoFrameDecoded] . You can also set the canvas after the callback [RTCRoomEventHandler.onUserJoined] .
/// 2. After the user joins the room, the user receives the callback [RTCRoomEventHandler.onUserJoined] for the users in the room.
/// 3. You can publish video streams with different configs with [RTCVideo.enableSimulcastMode] & [RTCVideo.setVideoEncoderConfig] .
/// 4. Join the room with valid token. If not, you will receive the callback [RTCVideoEventHandler.onError] with the corresponding code.
/// 5. Use the right roomProfile for your scene when joining room. In this demo, [RoomProfile.communication] is used for video calls.
/// 6. You do not need to destroy RTC engine after leaving the room. The demo does that to show the whole process.
///
/// For detailed API reference, refer to https://pub.dev/documentation/byteplus_rtc/latest/
class RoomPage extends StatefulWidget {
  final String roomId;
  final String userId;

  const RoomPage({Key? key, required this.roomId, required this.userId})
      : super(key: key);

  @override
  State<RoomPage> createState() => _RoomPageState();
}

class _RoomPageState extends State<RoomPage> {
  RTCVideo? _rtcVideo;
  RTCRoom? _rtcRoom;
  final RTCVideoEventHandler _videoHandler = RTCVideoEventHandler();
  final RTCRoomEventHandler _roomHandler = RTCRoomEventHandler();

  RTCViewContext? _localRenderContext;
  RTCViewContext? _firstRemoteRenderContext;
  RTCViewContext? _secondRemoteRenderContext;
  RTCViewContext? _thirdRemoteRenderContext;

  bool _isCameraFront = true;
  bool _isSpeakerphone = true;
  bool _enableAudio = true;
  bool _enableVideo = true;

  @override
  void initState() {
    super.initState();
    _initVideoEventHandler();
    _initRoomEventHandler();
    _initVideoAndJoinRoom();
  }

  @override
  void dispose() {
    super.dispose();

    /// Destroy the RTC room.
    _rtcRoom?.destroy();

    /// Destroy the RTC engine.
    _rtcVideo?.destroy();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            Container(
                padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
                height: 49,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    ImageButton(
                        onPressed: () {
                          _switchCamera();
                        },
                        imageName: 'switch_camera',
                        size: 26),
                    Expanded(
                        child: Text('RoomID:${widget.roomId}',
                            maxLines: 1,
                            overflow: TextOverflow.ellipsis,
                            textAlign: TextAlign.center,
                            style: const TextStyle(
                                color: Colors.black, fontSize: 18))),
                    ImageButton(
                        onPressed: () {
                          _switchAudioRoute();
                        },
                        imageName: _isSpeakerphone ? 'speaker' : 'ear_on',
                        size: 26)
                  ],
                )),
            Expanded(
                child: Container(
                    decoration: const BoxDecoration(color: Colors.grey),
                    child: LayoutBuilder(builder:
                        (BuildContext context, BoxConstraints constraints) {
                      return Wrap(
                          children: _buildRenderView(context, constraints));
                    }))),
            SizedBox(
                height: 49,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: [
                    ImageButton(
                        onPressed: () {
                          _enableLocalAudio();
                        },
                        imageName: _enableAudio ? 'normal_audio' : 'mute_audio',
                        size: 28),
                    ImageButton(
                        onPressed: () {
                          _hangUp();
                        },
                        imageName: 'hang_up',
                        size: 36),
                    ImageButton(
                        onPressed: () {
                          _enableLocalVideo();
                        },
                        imageName: _enableVideo ? 'normal_video' : 'mute_video',
                        size: 28)
                  ],
                ))
          ],
        ),
      ),
    );
  }

  List<Widget> _buildRenderView(
      BuildContext context, BoxConstraints constraints) {
    double height = constraints.maxHeight * 0.5;
    double width = constraints.maxWidth * 0.5;
    List<Widget> renderViewList = [];
    if (_localRenderContext != null) {
      renderViewList.add(SizedBox(
          height: height,
          width: width,
          child: UserLiveView(viewContext: _localRenderContext!)));
    }
    if (_firstRemoteRenderContext != null) {
      renderViewList.add(SizedBox(
          height: height,
          width: width,
          child: UserLiveView(viewContext: _firstRemoteRenderContext!)));
    }
    if (_secondRemoteRenderContext != null) {
      renderViewList.add(SizedBox(
          height: height,
          width: width,
          child: UserLiveView(viewContext: _secondRemoteRenderContext!)));
    }
    if (_thirdRemoteRenderContext != null) {
      renderViewList.add(SizedBox(
          height: height,
          width: width,
          child: UserLiveView(viewContext: _thirdRemoteRenderContext!)));
    }
    if (renderViewList.isEmpty) {
      renderViewList.add(const SizedBox());
    }
    return renderViewList;
  }

  void _initVideoAndJoinRoom() async {
    /// Create engine objects.
    _rtcVideo = await RTCVideo.createRTCVideo(
        RTCVideoContext(appId, eventHandler: _videoHandler));

    if (_rtcVideo == null) {
      _showAlert(
          'Engine creation failure\nPlease check whether the correct AppId is set');
      return;
    }

    /// Start pushing multiple video streams and set the video parameters when pushing multiple streams,
    /// Including resolution, frame rate, bit rate, zoom mode, fallback strategy when the network is poor, etc.
    VideoEncoderConfig solution = VideoEncoderConfig(
      width: 360,
      height: 640,
      frameRate: 15,
      maxBitrate: 800,
      encoderPreference: EncoderPreference.maintainFrameRate,
    );
    _rtcVideo?.setMaxVideoEncoderConfig(solution);

    /// Sets the view used when rendering local videos.
    setState(() {
      _localRenderContext = RTCViewContext.localContext(uid: widget.userId);
    });

    /// Enable internal video capture immediately. The default is off.
    _rtcVideo?.startVideoCapture();

    /// Enables internal audio capture. The default is off.
    _rtcVideo?.startAudioCapture();

    /// Create a room.
    _rtcRoom = await _rtcVideo?.createRTCRoom(widget.roomId);

    /// Set room event callback handler
    _rtcRoom?.setRTCRoomEventHandler(_roomHandler);

    /// Join the room.
    UserInfo userInfo = UserInfo(uid: widget.userId);
    RoomConfig roomConfig = RoomConfig(
        isAutoPublish: true,
        isAutoSubscribeAudio: true,
        isAutoSubscribeVideo: true);
    _rtcRoom?.joinRoom(
        token: token, userInfo: userInfo, roomConfig: roomConfig);
  }

  void _initVideoEventHandler() {
    /// The user receives this callback after the SDK receives the first frame of remote video decoding data.
    _videoHandler.onFirstRemoteVideoFrameDecoded =
        (RemoteStreamKey streamKey, VideoFrameInfo videoFrameInfo) {
      debugPrint('onFirstRemoteVideoFrameDecoded: ${streamKey.uid}');
      if (_firstRemoteRenderContext?.uid == streamKey.uid ||
          _secondRemoteRenderContext?.uid == streamKey.uid ||
          _thirdRemoteRenderContext?.uid == streamKey.uid) {
        return;
      }

      /// Sets the view to use when rendering a video stream from a specified remote user uid.
      if (_firstRemoteRenderContext == null) {
        setState(() {
          _firstRemoteRenderContext = RTCViewContext.remoteContext(
              roomId: widget.roomId, uid: streamKey.uid);
        });
      } else if (_secondRemoteRenderContext == null) {
        setState(() {
          _secondRemoteRenderContext = RTCViewContext.remoteContext(
              roomId: widget.roomId, uid: streamKey.uid);
        });
      } else if (_thirdRemoteRenderContext == null) {
        setState(() {
          _thirdRemoteRenderContext = RTCViewContext.remoteContext(
              roomId: widget.roomId, uid: streamKey.uid);
        });
      } else {}
    };

    /// Callback of warnings, see {https://pub.dev/documentation/byteplus_rtc/latest/api_bytertc_common_defines/WarningCode.html}.
    _videoHandler.onWarning = (WarningCode code) {
      debugPrint('warningCode: $code');
    };

    /// Callback of errors, see {https://pub.dev/documentation/byteplus_rtc/latest/api_bytertc_common_defines/ErrorCode.html}.
    _videoHandler.onError = (ErrorCode code) {
      debugPrint('errorCode: $code');
      _showAlert('errorCode: $code');
    };
  }

  void _initRoomEventHandler() {
    /// Callback for remote visible user joining the room.
    _roomHandler.onUserJoined = (UserInfo userInfo, int elapsed) {
      debugPrint('onUserJoined: ${userInfo.uid}');
    };

    /// Callback for remote visible user leaving the room.
    _roomHandler.onUserLeave = (String uid, UserOfflineReason reason) {
      debugPrint('onUserLeave: $uid reason: $reason');
      if (_firstRemoteRenderContext?.uid == uid) {
        setState(() {
          _firstRemoteRenderContext = null;
        });
        _rtcVideo?.removeRemoteVideo(uid: uid, roomId: widget.roomId);
      } else if (_secondRemoteRenderContext?.uid == uid) {
        setState(() {
          _secondRemoteRenderContext = null;
        });
        _rtcVideo?.removeRemoteVideo(uid: uid, roomId: widget.roomId);
      } else if (_thirdRemoteRenderContext?.uid == uid) {
        setState(() {
          _thirdRemoteRenderContext = null;
        });
        _rtcVideo?.removeRemoteVideo(uid: uid, roomId: widget.roomId);
      }
    };
  }

  void _showAlert(String warning) => showDialog<String>(
        context: context,
        builder: (BuildContext context) => AlertDialog(
          content: Text(warning),
          actions: [
            TextButton(
              onPressed: () => Navigator.pop(context, 'OK'),
              child: const Text('OK'),
            ),
          ],
        ),
      );

  void _switchCamera() {
    _isCameraFront = !_isCameraFront;

    /// Toggle the front/postcondition camera used when video internal capture.
    /// Default uses front-facing camera.
    CameraId cameraId = _isCameraFront ? CameraId.front : CameraId.back;
    _rtcVideo?.switchCamera(cameraId);
  }

  void _switchAudioRoute() {
    setState(() {
      _isSpeakerphone = !_isSpeakerphone;
    });

    /// Set the audio playback device as a speaker or earpiece, and use speakers by default.
    AudioRoute audioRoute =
        _isSpeakerphone ? AudioRoute.speakerphone : AudioRoute.earpiece;
    _rtcVideo?.setDefaultAudioRoute(audioRoute);
  }

  void _enableLocalAudio() {
    setState(() {
      _enableAudio = !_enableAudio;
    });

    /// Controls the send status of the local audio stream: send/not send.
    if (_enableAudio) {
      _rtcRoom?.publishStream(MediaStreamType.audio);
    } else {
      _rtcRoom?.unpublishStream(MediaStreamType.audio);
    }
  }

  void _enableLocalVideo() {
    setState(() {
      _enableVideo = !_enableVideo;
    });

    /// Enable internal video capture immediately. The default is off.
    if (_enableVideo) {
      _rtcVideo?.startVideoCapture();
    } else {
      _rtcVideo?.stopVideoCapture();
    }
  }

  void _hangUp() {
    /// Leave the room.
    _rtcRoom?.leaveRoom();
    Navigator.pop(context, true);
  }
}

/// Icon button.
class ImageButton extends StatelessWidget {
  final String imageName;

  final double? size;

  final VoidCallback? onPressed;

  const ImageButton(
      {Key? key, required this.imageName, this.size, required this.onPressed})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final double? iconSize = size ?? IconTheme.of(context).size;
    return GestureDetector(
        onTap: onPressed,
        child: SizedBox(
          height: iconSize,
          width: iconSize,
          child: Image.asset('assets/$imageName.png'),
        ));
  }
}
