// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_common_defines.dart';

/// Type of audio device
enum AudioDeviceType {
  /// Unknown device
  unknown,

  /// Speaker or headphone
  renderDevice,

  /// Microphone
  captureDevice,

  /// Screen capturing audio device
  screenCaptureDevice,
}

/// Audio playback device
enum AudioRoute {
  /// Unknown device
  unknown,

  /// Wired earphones
  headset,

  /// Earpiece. Built-in device for calling
  earpiece,

  /// Speaker. Built-in device for hands-free audio playback
  speakerphone,

  /// Bluetooth earphones
  headsetBluetooth,

  /// USB device
  headsetUSB,
}

/// Audio scene type
///
/// After selecting an audio scene, RTC will automatically apply call volume/media volume based on the client's audio capture playback device and status. <br>
/// If the preset audio scene type cannot meet your business needs, please contact technical support for customization.
enum AudioScenario {
  /// Music scene. Default setting.
  ///
  /// This option is suitable for scenes that require musical expression. Such as live music. <br>
  /// Audio capture playback device and capture playback status, the mapping to the volume type is as follows:
  /// <table border>
  /// <tr><th></th><th>Mic off</th><th>Mic on</th><th>Remarks</th></tr>
  /// <tr><td>Build-in microphone and speaker/earpiece</td><td>Media volume</td><td>Call volume</td><td>/</td></tr>
  /// <tr><td>Wired headset</td><td>Media volume</td><td>Media volume</td><td></td></tr>
  /// <tr><td>Bluetooth headset</td><td>Media volume</td><td>Media volume</td><td>Media volume</td><td>Only the device's build-in microphone can be used for local audio capture, even if the Bluetooth headset has a microphone.</td></tr>
  /// </table>
  music,

  /// High-quality call scene.
  ///
  /// This option is suitable for scenarios where musical expression is required but you also want to be able to use the microphone that comes with the Bluetooth headset for audio capture. <br>
  /// This scenario allows for a balanced audio experience when playing out/using Bluetooth headphones and avoids the sudden change in listening experience caused by switching between volume types when using Bluetooth headphones.<br>
  /// Audio capture playback device and capture playback status, the mapping to the volume type is as follows: <br>
  /// <table border>
  /// <tr><th></th><th>Mic off</th><th>Mic on</th><th>Remarks</th></tr>
  /// <tr><td>Build-in microphone and speaker/earpiece</td><td>Media volume</td><td>Call volume</td><td>/</td></tr>
  /// <tr><td>Wired headset</td><td>Media volume</td><td>Media volume</td><td>/</td></tr>
  /// <tr><td>Bluetooth headset</td><td>Call volume</td><td>Call volume</td><td>Call volume</td><td>The microphone on the Bluetooth headset can be used for audio capture.</td></tr>
  /// </table>
  highQualityCommunication,

  /// Call volume-only scene.
  ///
  /// This option is suitable for calls or conference scenarios where frequent speaker switching is required.<br>
  /// In this scenario, the call volume is used throughout, regardless of the client audio capture/playback device and capture/playback status.<br>
  /// This scene maintains a uniform audio pattern without sudden volume changes in listening; echoes are eliminated to the greatest extent possible for optimum call clarity.<br>
  /// When using a Bluetooth headset, you can use the microphone that comes with the Bluetooth headset for audio capture. <br>
  /// However, this scene will depress the volume of other audio being played back using the media volume and the sound quality will deteriorate.
  communication,

  /// Media volume-only scene. Not generally recommended.
  ///
  /// In this scenario, the media volume is used throughout, regardless of the client audio capture/playback device and capture/playback status.
  media,

  /// Game media scene
  ///
  /// In this scenario, the Bluetooth headset uses the call volume and the other devices use the media volume.<br>
  /// Echoes and whistles are very likely to occur when talking on speakerphone and without game sound cancellation optimisation.
  gameStreaming
}

/// Types of voice change effects
enum VoiceChangerType {
  /// Original sound, no effects
  original,

  giant,

  chipmunk,

  minions,

  vibrato,

  robot,
}

/// Types of reverb effects
enum VoiceReverbType {
  /// Original sound, no effects
  original,

  echo,

  concert,

  ethereal,

  ktv,

  studio,
}

/// Mixing audio playback status
enum AudioMixingState {
  /// Mixing file is loaded
  preloaded,

  /// Mixing file is playing
  playing,

  /// Mixing is paused
  paused,

  /// Mixing is stopped
  stopped,

  /// Mixing file playback failed
  failed,

  /// End of mixing file playback
  finished,

  /// PCM mixing enabled
  pcmEnabled,

  /// PCM mixing disabled
  pcmDisabled,
}

/// Audio mixing related error code
enum AudioMixingError {
  /// Normal.
  ok,

  /// Preload failed. Invalid path or the length exceeds 20s.
  preloadFailed,

  /// Mixing failed. Invalid path or fail to open the file.
  startFailed,

  /// Invalid mixint task ID.
  idNotFound,

  /// Invalid start position.
  setPositionFailed,

  /// Invalid volume. The range is [0, 400].
  invalidVolume,

  /// Another file was preloaded for mixing. Call [RTCAudioMixingManager.unloadAudioMixing] first.
  loadConflict,

  /// Invalid mixing type.
  idTypeNotMatch,

  /// Invalid pitch value.
  idTypeInvalidPitch,

  /// Invalid audio track.
  invalidAudioTrack,

  /// Mixing starting.
  isStarting,

  /// Invalid playback speed.
  invalidPlaybackSpeed,
}

/// Enable/disable earphone monitoring.
enum EarMonitorMode {
  /// Disable(default setting)
  off,

  /// Enable
  on,
}

/// Authentication method for speech recognition services. Please consult technical support for details.
enum ASRAuthorizationType {
  token,

  signature,
}

/// Verification information required when using ASR services
class RTCASRConfig {
  /// User ID
  String uid;

  /// Access token
  String accessToken;

  /// Private key
  ///
  /// This value cannot be empty in Signature authentication mode and should be empty in token authentication mode.
  String secretKey;

  /// Authentication method
  ASRAuthorizationType authorizationType;

  /// Scenario information
  String cluster;

  /// APP ID
  String appId;

  RTCASRConfig(
      {required this.uid,
      required this.accessToken,
      this.secretKey = '',
      this.authorizationType = ASRAuthorizationType.token,
      required this.cluster,
      required this.appId});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'accessToken': accessToken,
      'secretKey': secretKey,
      'authorizationType': authorizationType.value,
      'cluster': cluster,
      'appId': appId,
    };
  }
}

/// Audio mixing type
enum AudioMixingType {
  /// Played at the local device only.
  playout,

  /// Sent to the remote devices only.
  publish,

  /// Played at the local device and sent to the remote devices.
  playoutAndPublish,
}

/// Audio mixing configuration
class AudioMixingConfig {
  /// Audio mixing playback type
  AudioMixingType type;

  /// Playback times
  ///
  /// + playCount <= 0: Infinite loop
  /// + playCount == 1:Play once(default)
  /// + playCount > 1: Play `playCount` times
  int playCount;

  /// The position of the audio file playback progress bar during audio mixing, the parameter should be an integer in milliseconds.
  int position;

  /// Frequency (ms) of audio file playback progress callback. [RTCVideoEventHandler.onAudioMixingPlayingProgress] will be triggered then according to the set value, no callback by default.
  ///
  /// + The value is a multiple of 10 greater than 0. When the value set is not divisible by 10, it will be automatically rounded up by 10. For example, if the value is set to 52ms, it will be automatically adjusted to 60ms.
  /// + If the value is less than or equals to 0, no callback will be triggered.
  int callbackOnProgressInterval;

  AudioMixingConfig(
      {this.type = AudioMixingType.playout,
      this.playCount = 0,
      this.position = 0,
      this.callbackOnProgressInterval = 0});
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'type': type.value,
      'playCount': playCount,
      'position': position,
      'callbackOnProgressInterval': callbackOnProgressInterval,
    };
  }
}

/// Audio mixing playback channel type
enum AudioMixingDualMonoMode {
  /// Consistent with audio files.
  auto,

  /// Only the left channel audio in the file can be heard
  left,

  /// Only the right channel audio in the file can be heard
  right,

  /// Both the left and right channel audio in the file can be heard at the same time
  mix,
}

/// Configuration related to audio property information prompt
class AudioPropertiesConfig {
  /// Information prompt interval in milliseconds
  ///
  /// + `<= 0`: Turn off the prompt.
  /// + `(0,100]`: Invalid interval value, and will be automatically reset to 100ms.
  /// + `> 100`: The actual value of interval.
  int interval;

  /// Whether to enable audio spectrum detection
  bool enableSpectrum;

  /// Whether to enable Voice Activity Detection(VAD)
  bool enableVad;

  AudioPropertiesConfig(
      {this.interval = 100,
      this.enableSpectrum = false,
      this.enableVad = false});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'interval': interval,
      'enableSpectrum': enableSpectrum,
      'enableVad': enableVad,
    };
  }
}

/// Audio properties
class AudioPropertiesInfo {
  /// Linear volume. The value is in linear relation to the original volume. The higher the value, the higher the volume. The range is [0,255].
  ///
  /// - [0, 25]: Silence
  /// - [26, 75]: Low volume
  /// - [76, 204]: Medium volume
  /// - [205, 255]: High volume
  int linearVolume;

  /// Non-linear volume in dB. The value is in proportion to the log value of the original volume. You can use the value to recognize the Active Speaker in the room. The range is [-127, 0].
  ///
  /// - [-127, -60]: Silence
  /// - [-59, -40]: Low volume
  /// - [-39, -20]: Medium volume
  /// - [-19, 0]: High volume
  int nonlinearVolume;

  /// Voice Activity Detection(VAD)result
  ///
  /// - 1: Voice activity detected.
  /// - 0: No voice activity detected.
  /// - -1: VAD not activated.
  int vad;

  /// Spectrum array. The default length is 257.
  List<double> spectrum;

  AudioPropertiesInfo(
      {required this.linearVolume,
      required this.nonlinearVolume,
      required this.vad,
      required this.spectrum});
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'linearVolume': linearVolume,
      'nonlinearVolume': nonlinearVolume,
      'vad': vad,
      'spectrum': spectrum,
    };
  }

  factory AudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return AudioPropertiesInfo(
      linearVolume: map['linearVolume'],
      nonlinearVolume: map['nonlinearVolume'],
      vad: map['vad'],
      spectrum: List<double>.from(map['spectrum']),
    );
  }
}

/// Remote audio properties
class RemoteAudioPropertiesInfo {
  /// Information about the remote stream
  RemoteStreamKey streamKey;

  /// Information about the audio properties
  AudioPropertiesInfo audioPropertiesInfo;
  RemoteAudioPropertiesInfo(
      {required this.streamKey, required this.audioPropertiesInfo});
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'audioPropertiesInfo': audioPropertiesInfo.toMap(),
      'streamKey': streamKey.toMap(),
    };
  }

  factory RemoteAudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return RemoteAudioPropertiesInfo(
      streamKey: RemoteStreamKey.fromMap(map['streamKey']),
      audioPropertiesInfo:
          AudioPropertiesInfo.fromMap(map['audioPropertiesInfo']),
    );
  }
}

/// Local audio properties
class LocalAudioPropertiesInfo {
  /// Type of the stream
  StreamIndex streamIndex;

  /// Information about the audio properties
  AudioPropertiesInfo audioPropertiesInfo;
  LocalAudioPropertiesInfo(
      {required this.streamIndex, required this.audioPropertiesInfo});
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'type': streamIndex.value,
      'audioPropertiesInfo': audioPropertiesInfo.toMap(),
    };
  }

  factory LocalAudioPropertiesInfo.fromMap(Map<dynamic, dynamic> map) {
    return LocalAudioPropertiesInfo(
        streamIndex: (map['type'] as int).streamIndex,
        audioPropertiesInfo:
            AudioPropertiesInfo.fromMap(map['audioPropertiesInfo']));
  }
}

/// Sound quality
enum AudioProfileType {
  /// Default sound quality
  ///
  /// The sound quality is configured via the [ChannelProfile] sent from the server or set by the client.
  def,

  /// Fluent
  ///
  /// + Sample rate: 16 KHz
  /// + Mono-channel
  /// + Encoding bitrate: 32 Kpbs
  ///
  /// Low resource consumption and small network packets guarantee a smooth call. It is suitable for most game scenarios, such as team-wide voice chat, group-wide voice chat, nation-wide voice chat.
  fluent,

  /// Mono-channel standard
  ///
  /// + Sample rate: 24 KHz
  /// + Encoding bitrate: 48 Kpbs
  ///
  /// For scenarios where distinct voice quality is required, you can choose this mode, which achieves balanced latency, consumption, and network packets. It is suitable for educational Apps and the online Mafia Games.
  standard,

  /// Dual-channel music
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 128 Kpbs
  ///
  /// This mode provides high-resolution audio at a cost of high latency, consumption, and large network packets. It is suitable for music Apps such as co-hosting and online talent contests. <br>
  /// Not recommended for game Apps.
  hd,

  /// Dual-channel standard
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 80 Kpbs
  standardStereo,

  /// Mono-channel music
  ///
  /// + Sample rate: 48 KHz
  /// + Encoding bitrate: 64 Kpbs
  hdMono,
}

/// Coordinate value of the local user's position in the rectangular coordinate system in the RTC room.
class Position {
  /// X-coordinate
  int x = 0;

  /// Y-coordinate
  int y = 0;

  /// Z-coordinate
  int z = 0;

  Position({
    this.x = 0,
    this.y = 0,
    this.z = 0,
  });
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'z': z,
    };
  }
}

/// Direction orientation information
class RTCOrientation {
  /// X direction vector
  double x = 0;

  /// Y direction vector
  double y = 0;

  /// Z direction vector
  double z = 0;

  RTCOrientation({
    this.x = 0,
    this.y = 0,
    this.z = 0,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'z': z,
    };
  }
}

/// Three-dimensional orientation information, each pair of vectors need to be perpendicular.
class HumanOrientation {
  /// Forward orientation, the default value is {1,0,0}, i.e., the forward orientation is in the positive direction of x-axis.
  RTCOrientation forward = RTCOrientation();

  /// Rightward orientation, the default value is {0,1,0}, i.e., the rightward orientation is in the positive direction of y-axis.
  RTCOrientation right = RTCOrientation();

  /// Upward orientation, the default value is {0,0,1}, i.e., the upward orientation is in the positive direction of z-axis.
  RTCOrientation up = RTCOrientation();

  HumanOrientation.origin()
      : forward = RTCOrientation(x: 1, y: 0, z: 0),
        right = RTCOrientation(x: 0, y: 1, z: 0),
        up = RTCOrientation(x: 0, y: 0, z: 1);

  HumanOrientation({
    required this.forward,
    required this.right,
    required this.up,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'forward': forward.toMap(),
      'right': right.toMap(),
      'up': up.toMap(),
    };
  }
}

/// Mode of volume attenuation with distance from the sound source in spatial audio
enum AttenuationType {
  /// Disable volume attenuation
  none,

  /// Linear attenuation
  linear,

  /// Exponential attenuation
  exponential,
}

/// The range in which local user can hear audio with attenuation effect.
class ReceiveRange {
  /// The minimum distance at which the local user can hear the attenuated audio from remote users.
  ///
  /// The value must be ≥ 0, but ≤ max. <br>
  /// No attenuation effect for audio from distances less than this value you set.
  int min;

  /// The maximum distance at which the local user can hear audio from remote users.
  ///
  /// The value must be ≥ min, and > 0.  <br>
  /// Audio from distances larger than the value you set cannot be heard.
  int max;

  ReceiveRange({
    this.min = 0,
    this.max = 1,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'min': min,
      'max': max,
    };
  }
}

/// Stream information in range audio
class RangeAudioInfo {
  /// ID of remote stream publisher
  String uid;

  /// Attenuation coefficient of the remote user. The value range is [0,100].
  ///
  /// A value of 0 means that the remote user's audio volume is 0.
  int factor;

  RangeAudioInfo({required this.uid, required this.factor});

  factory RangeAudioInfo.fromMap(Map<dynamic, dynamic> map) {
    return RangeAudioInfo(uid: map['uid'], factor: map['factor']);
  }
}
