// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import '../src/base/bytertc_enum_convert.dart';

/// The warning code indicates that the SDK has encountered a problem and is trying to recover. It only serves as a notification.
enum WarningCode {
  /// Undefined warning code
  unknown,

  /// Failed to enter the room due to a server error. The SDK will automatically retry.
  joinRoomFailed,

  /// Failed to publish streams due to a server error. The SDK will automatically retry.
  publishStreamFailed,

  /// Failed to subscribe to streams. The target stream cannot be found.
  ///
  /// The SDK will automatically retry the subscription and if it still fails you can quit and retry.
  subscribeStreamFailed404,

  /// Failed to subscribe to streams due to a server error. The SDK will automatically retry.
  subscribeStreamFailed5xx,

  /// Publishing stream after calling [RTCRoom.setUserVisibility] to set user visibility to false will trigger this warning.
  publishStreamForbidden,

  /// Failed to send custom broadcast message, because the local user is not in the room.
  sendCustomMessage,

  /// When the number of people in the room exceeds 500, the SDK will stop sending [RTCRoomEventHandler.onUserJoined] and [RTCRoomEventHandler.onUserLeave] callbacks to existing users in the room, and notify all users in the room via broadcast.
  receiveUserNotifyStop,

  /// The user has already published a stream in another room, or the user is publishing a public stream now.
  userInPublish,

  /// The room with the same roomId already exists.
  roomIdAlreadyExist,

  /// A new room with the same roomId has replaced the old one.
  oldRoomBeenReplaced,

  /// Invalid call as an echo test is currently in progress.
  inEchoTestMode,

  /// The current app does not have access to the camera
  noCameraPermission,

  /// Switching screen audio capture source after calling [RTCRoom.publishScreen] is not allowed.
  setScreenAudioSourceTypeFailed,

  /// Setting up the screen audio sharing type with [RTCVideo.setScreenAudioStreamIndex] after calling [RTCRoom.publishScreen] is not allowed.
  setScreenAudioStreamIndexFailed,

  /// Invalid pitch value.
  invalidVoicePitch,

  /// Mixed use of old and new APIs for external audio sources.
  invalidCallForExtAudio,

  /// Applies to iOS. The specified internal rendering canvas handle is invalid.
  invalidCanvasHandle,
}

/// The error code indicates that the SDK has encountered a non-recoverable error. It will be send to users through [RTCVideoEventHandler.onError].
enum ErrorCode {
  /// Undefined error code
  unknown,

  /// The token used when joining the room is invalid or expired. The user is required to retrieve the token and update it by calling [RTCRoom.updateToken].
  invalidToken,

  /// An unknown error occurs when entering a room causing a failure to join the room, requiring the user to rejoin the room.
  joinRoom,

  /// The user failed to publish streams in the room because they do not have permission.
  noPublishPermission,

  /// The user failed to subscribe to streams in the room because they do not have permission.
  noSubscribePermission,

  /// The current user is removed from the room because another user with the same user ID joins the room.
  duplicateLogin,

  /// Applies to Android.
  ///
  /// The App ID is not passed in when you create the engine.
  appIdNull,

  /// The user has been removed from the room by the administrator via a OpenAPI call.
  kickedOut,

  /// The roomId passed in when you call [RTCVideo.createRTCRoom] is illegal.
  roomIdIllegal,

  /// The Token has expired.
  ///
  /// The user should rejoin the room with a new Token.
  tokenExpired,

  /// The Token provided in [RTCRoom.updateToken] is invalid.
  updateTokenWithInvalidToken,

  /// All the users have been removed from the room because the administrator dismissed the room by calling OpenAPI.
  roomDismiss,

  /// Failed to join the room. The LICENSE billing account does not use the LICENSE_AUTHENTICATE SDK.
  joinRoomWithoutLicenseAuthenticateSDK,

  /// A room with the same roomId already exists during the call echo test.
  roomAlreadyExist,

  /// Failed to subscribe to streams because the total number of subscribed streams has exceeded the upper limit.
  ///
  /// In order to ensure the performance and quality in game scenarios, the server will limit the total number of audio & video streams subscribed by the user. <br>
  /// When the total number of subscribed stream has reached the maximum, continuing to subscribe to more streams will fail, and the user will receive this error notification.
  overStreamSubscribeLimit,

  /// Failed to publish camera-captured video stream because the total number of published streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  overStreamPublishLimit,

  /// Failed to publish screen video stream because the total number of published streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  overScreenPublishLimit,

  /// The total number of published video streams has exceeded the upper limit.
  ///
  /// RTC will limit the total number of streams published in a single room, including video, audio, and screen streams. <br>
  /// Local users will fail to publish streams to the room when the maximum number of published streams in the room has been reached, and will receive this error notification.
  overVideoPublishLimit,

  /// Audio/Video synchronization failed.
  ///
  /// The current audio source has been synchronizd with another video source.<br>
  /// One single audio source cannot be synchronized with multiple video sources at the same time.
  invalidAudioSyncUidRepeated,

  /// The user has been removed from the room due to the abnormal status of server.
  ///
  /// SDK is disconnected with the signaling server and will not reconnect automatically. Please contact technical support.
  abnormalServerStatus,
}

/// The reason why the remote user leaves the room.
enum UserOfflineReason {
  /// The remote user calls [RTCRoom.leaveRoom].
  quit,

  /// The remote user is disconnected.
  dropped,

  /// The remote user switches to invisible.
  switchToInvisible,

  /// The remote user is kicked out of the room due to an OpenAPI call.
  kickedByAdmin,
}

/// The reason for the removal of the remote stream in the room.
enum StreamRemoveReason {
  /// The remote user stops publishing streams.
  unpublish,

  /// The remote user fails to publish streams.
  publishFailed,

  /// Keep alive failed.
  keepLiveFailed,

  /// The remote user is disconnected.
  clientDisconnected,

  /// The remote user republishes streams.
  republish,

  /// Other reasons.
  other,
}

/// Login result
///
/// The result of calling [RTCVideo.login] will be notified to the user through [RTCVideoEventHandler.onLoginResult].
enum LoginErrorCode {
  /// Login successful
  success,

  /// The token is invalid or expired. The user is required to re-acquire the token.
  invalidToken,

  /// An unknown error occurred when calling [RTCVideo.login], which caused the login to fail. The user is required to log in again.
  loginFailed,

  /// Invalid user ID passed in through [RTCVideo.login].
  invalidUserId,

  /// Server error during login
  codeServerError,
}

/// Reliability and orderliness of the message
enum MessageConfig {
  /// Reliable and ordered low-latency message
  reliableOrdered,

  /// Ordered ultra-low latency message
  unreliableOrdered,

  /// Unordered ultra-low latency message
  unreliableUnordered,
}

/// Message delivery result
enum UserMessageSendResult {
  /// The message has been sent successfully.
  success,

  /// Failed, message delivery timeout.
  timeout,

  /// Failed. Channel disconnected.
  broken,

  /// Failed. Recipient not found.
  noReceiver,

  /// Failed to get cascade path.
  noRelayPath,

  /// The message sender is not in the room.
  notJoin,

  /// Connection is not initialized.
  init,

  /// No data transmission channel connection available.
  noConnection,

  /// Message exceeds maximum length (64KB).
  exceedMaxLength,

  /// The ID of the recipient is empty.
  emptyUser,

  /// The message sender did not log in.
  notLogin,

  /// No parameters were set before sending the message to the server.
  serverParamsNotSet,

  /// Undefined error.
  unknown,
}

/// User online status
enum UserOnlineStatus {
  /// The peer user is offline, either because [RTCVideo.login] has not been called or [RTCVideo.logout] has been called.
  offline,

  /// The peer user is online
  online,

  /// Unable to get the peer user's online status.
  ///
  /// Returned when a cascade error occurs and the online state of the peer user is abnormal.
  unreachable,
}

/// Result of broadcasting messages in the room
enum RoomMessageSendResult {
  /// The message has been sent successfully.
  success,

  /// Failed. The message sender is not in the room.
  notJoin,

  /// Failed. Connection is not initialized.
  init,

  /// Failed. No data transmission channel connection available.
  noConnection,

  /// Failed. Message exceeds maximum length (64KB).
  exceedMaxLength,

  /// Failed. Undefined error.
  unknown,
}

/// Status of the SDK connection to the signalling server
enum RTCConnectionState {
  /// Disconnected for 12s, SDK will try to reconnect automatically.
  disconnected,

  /// The first request to connect to the server. Connecting.
  connecting,

  /// The first connection was successful.
  connected,

  /// + Failed to connect within 10 seconds on first connection.
  /// + Disconnected for 10 seconds after a successful connection. Automatic reconnection in progress.
  reconnecting,

  /// Successful reconnection.
  reconnected,

  /// In disconnected status for more than 10s without a successful reconnection. SDK will still continue to try to reconnect.
  lost,

  /// Connection failure caused by abnormal state of the server. SDK will not automatically reconnect, please re-enter the room or contact technical support.
  failed,
}

/// Network connection type.
enum NetworkType {
  /// Unknown type
  unknown,

  /// The network is disconnected.
  disconnected,

  /// The network connection type is LAN.
  lan,

  /// The network connection type is Wi-Fi (including hotspots).
  wifi,

  /// 2G mobile network.
  mobile2G,

  /// 3G mobile network.
  mobile3G,

  /// 4G mobile network.
  mobile4G,

  /// 5G mobile network.
  mobile5G,
}

/// Returned result from the SDK by calling [RTCVideo.startNetworkDetection].
enum NetworkDetectionStartReturn {
  /// Successfully initiated detection.
  success,

  /// Failed to start detection. Both `isTestUplink` and `isTestDownlink` are `false`, or the expected bandwidth exceeds the range [100,10000].
  paramError,

  /// Failed to start detection. Local user has already started to publish and receive streams.
  streaming,

  /// Already in a detection.
  started,

  /// This funtion is not supported.
  notSupport,
}

/// Reasons for stopping network detection before a call
enum NetworkDetectionStopReason {
  /// The user stops the detection.
  user,

  /// Detection has been on for more than 3 minutes and has stopped automatically.
  timeout,

  /// Detection network disconnected.
  ///
  /// When no reply is received for more than 12s, the SDK will disconnect from the network and will not attempt to reconnect.
  connectionLost,

  /// Detection stops because local user has already started to publish and receive streams.
  streaming,

  /// Internal exceptions.
  innerErr,
}

/// Type of network detected before a call.
enum NetworkDetectionLinkType {
  /// Uplink network
  up,

  /// Downlink network
  down,
}

/// Media streaming network quality
enum NetworkQuality {
  /// Unknown quality
  unKnown,

  /// Excellent
  excellent,

  /// Good
  good,

  /// The network quality is poor, but does not affect communication.
  poor,

  /// The network quality is bad, users can barely communicate but not smoothly.
  bad,

  /// The quality of the network is very bad and communication is basically impossible.
  veryBad,
}

/// Tx/Rx network quality related data
class NetworkQualityStats {
  /// User ID
  String uid;

  /// Packet loss rate, and the range is [0.0,1.0].
  /// + For a local user, it is the sent-packet loss ratio.
  /// + For a remote user, it is the loss ratio of all the packets received.
  double fractionLost;

  /// Round-trip time (RTT) in ms from client to server. Effective for the local user.
  int rtt;

  /// Average transmission rate(bps) of the media RTP packages in 2s.
  /// + For a local user, it is the packet-sending speed.
  /// + For a more user, it is the speed of receiving all the subsribed medias.
  int totalBandwidth;

  /// Tx network quality grade in a range of [0,5]. The higher the grade, the worse the network quality.
  NetworkQuality txQuality;

  /// Rx network quality grade in a range of [0,5]. The higher the grade, the worse the network quality.
  NetworkQuality rxQuality;

  NetworkQualityStats(
      {required this.uid,
      required this.fractionLost,
      required this.rtt,
      required this.totalBandwidth,
      required this.txQuality,
      required this.rxQuality});

  factory NetworkQualityStats.fromMap(Map<dynamic, dynamic> map) {
    return NetworkQualityStats(
        uid: map['uid'],
        fractionLost: map['fractionLost'],
        rtt: map['rtt'],
        totalBandwidth: map['totalBandwidth'],
        txQuality: (map['txQuality'] as int).networkQuality,
        rxQuality: (map['rxQuality'] as int).networkQuality);
  }
}

/// Room mode
enum RoomProfile {
  /// Communication mode(default)
  communication,

  @Deprecated('use interactivePodcast instead')
  liveBroadCasting,

  /// Game voice mode.
  ///
  /// Low consumption of computing resources and network data.<br>
  /// Additional performance optimizations have been made for low-end devices:
  /// + Encodes frame length 40/60 for some low-end models.
  /// + Disables software 3A audio processing for some low-end models.
  /// Enhance iOS Compatibility with other screen recordings to avoid audio recordings being interrupted by RTC.
  game,

  /// Cloud game mode.
  ///
  /// Use this mode for game App requiring low latency. <br>
  /// In this mode, the call latency will be significantly reduced, but at the same time, weak network resistance and call sound quality will be affected to a certain extent, so before using this mode, it is highly recommended to consult technical support.
  cloudGame,

  /// Cloud render mode featuring extra-low latency.
  ///
  /// Choose this mode if it is not a game App yet requires low latency. <br>
  /// In this mode, latency will significantly decrease at the cost of lower audio quality as well as the bad performance with weak network signal.
  lowLatency,

  /// For 1 vs 1 video and audio calls
  chat,

  /// For voice calls only with 3 or more people
  chatRoom,

  /// For scenarios such as "Watch together" and "Listen together". Multiple clients are capable of playing the same videos and audios synchronically.
  ///
  /// In this mode, RTC does not involve the transportation of the sharing media but only synchronizes video/music playback across multiple clients via signaling.
  lwTogether,

  /// For game apps demanding high-resolution audio. In this mode, RTC plays audio using the media mode only.
  gameHD,

  /// For the events of co-hosting in the live-streaming.
  ///
  /// During a livestreaming using a CDN network, one host can invite another host to join the co-hosting event using RTC.
  coHost,

  /// For interactive podcasts that the host can have video and audio interactions with the audience.
  ///
  /// Using communication mode, and whether to co-host or not will not have mode switching to avoid sudden volume changes.
  interactivePodcast,

  /// For the online karaoke with high-quality audio and low latency.
  ///
  /// In these scenarios, RTC transports the accompaniment and mixed audio, such as solo and non-realtime chorus.
  ktv,

  /// For the online-chorusing scenarios requiring high-quality audio and low latency.
  ///
  /// Contact our technical specialists before you apply it to your App.
  chorus,

  /// For VR chat.
  //
  /// This mode supports audio sampling rates up to 192 KHz, with the ability to turn on spherical stereo.<br>
  /// Available in 3.46 and later versions.
  vrChat,

  /// For scenarios of streaming live videos to only one client on the LAN. It can be applied to devices on the Internet or LAN.
  gameStreaming,

  /// For 1-to-many live video streaming for LANs.
  ///
  /// Maximum supported resolution is 8K, frame rate is 60fps, and bit rate is 100 Mbps.<br>
  /// A private media server is needed on the LAN.
  lanLiveStreaming,

  /// For meeting Apps installed on personal devices.
  meeting,

  /// For meeting Apps installed on terminals of meeting rooms.
  meetingRoom,

  /// For the online classrooms and lectures that over 10 participants in the room are allowed to join the video chat.
  classroom,
}

/// Subscription status
enum SubscribeState {
  /// Successfully changed the subscription status
  success,

  /// Failed to change the subscription status, because the local user was not in the room.
  failedNotInRoom,

  /// Failed to change the subscription status, because the target audio/video stream was not found.
  failedStreamNotFound,

  /// Failed to change the subscription status, because the number of streams you have subscribed to has exceeded the limit.
  failedOverLimit,
}

/// Priority of the publisher. When a user encounters performance insufficiency of either the network or the device, the media stream  will fall back in the ascending order of `RemoteUserPriority`.
enum RemoteUserPriority {
  /// Low(default)
  low,

  /// No priority
  medium,

  /// High
  high,
}

/// Stream fallback options of publisher
enum PublishFallbackOption {
  /// Default setting. No fallback is allowed under limited network conditions.
  disable,

  /// Under limited network conditions, the video streams that you published will degrade sequentially from the highest-quality stream to the lowest-quality stream until it can match current network conditions.
  simulcastSmallVideoOnly
}

/// Stream fallback options of subscriber
enum SubscribeFallbackOption {
  /// Default setting. No fallback is allowed under limited network conditions.
  disable,

  /// The resolution of currently subscribed video stream will be reduced under limited network conditions.
  ///
  /// You can only select this setting after the stream publisher calls [RTCVideo.enableSimulcastMode] to enable the mode of publishing multiple streams, otherwise no fallback will occur.
  streamLow,

  /// Receive audio only under poor network conditions.
  audioOnly
}

/// Causes of fallback or recovery from fallback in remote subscription streams.
enum FallbackOrRecoverReason {
  /// Undefined reason. Default value.
  ///
  /// Fallback due to an unknown reason that is neither infufficienclt bandwidth of the network nor poor-performance of the device.
  unknown,

  /// Fallback of the subscribed stream due to insufficient bandwidth of the network.
  subscribeFallbackByBandwidth,

  /// Fallback of the subscribed stream for poor-performance of the device.
  subscribeFallbackByPerformance,

  /// Reverting from a Fallback of the subscribed stream due to the recovery of the network bandwidth.
  subscribeRecoverByBandwidth,

  /// Reverting from a Fallback of the subscribed stream due to the amelioration of the device performance.
  subscribeRecoverByPerformance,

  /// Fallback of the publishing stream due to Insufficient bandwidth of the network.
  publishFallbackByBandwidth,

  /// Fallback of the publishing stream due to poor-performance of the device.
  publishFallbackByPerformance,

  /// Reverting from a Fallback of the publishing stream due to the recovery of the network bandwidth.
  publishRecoverByBandwidth,

  /// Reverting from a Fallback of the publishing stream due to the amelioration of the device performance.
  publishRecoverByPerformance,
}

/// Reasons for performance-related alarms
enum PerformanceAlarmReason {
  /// The poor network causes the transmission performance to fall back.
  ///
  /// This reason is only received when sending performance fallback is turned on.
  bandwidthFallbacked,

  /// Network performance recovery, transmission performance rollback recovery.
  ///
  /// This reason is only received when sending performance fallback is turned on.
  bandwidthResumed,

  /// If the send performance fallback is not turned on, when receiving this alarm, it means that the performance is insufficient. <br>
  /// If the send performance fallback is turned on, when receiving this alarm, it means that the performance is insufficient and the send performance fallback has occurred.
  performanceFallbacked,

  /// If the send performance fallback is not turned on, when receiving this alarm, it means that the performance shortage has been restored. <br>
  /// If the send performance fallback is turned on, when receiving this alarm, it means that the performance shortage has been restored and the send performance fallback has occurred. Recovery.
  performanceResumed
}

/// Whether to enable publisher performance fallback
enum PerformanceAlarmMode {
  /// No
  normal,

  /// Yes
  simulcast,
}

/// Local audio stream status.
enum LocalAudioStreamState {
  /// The default initial state of the local audio.
  ///
  /// SDK will return this state when the microphone stops working. The corresponding error code is `ok`.
  stopped,

  /// The local audio recording device starts successfully.
  ///
  /// SDK will return this state after successfully capturing the first audio frame. The corresponding error code is `ok`.
  recording,

  /// The first frame of the local audio was successfully encoded. The corresponding error code is `ok`.
  encoding,

  /// The local audio startup failed, and the status is called back at the following times:
  /// + The local recording device failed to start, the corresponding error code is `recordFailure`.
  /// + No recording device permission was detected, the corresponding error code is `deviceNoPermission`.
  /// + Audio encoding failed, the corresponding error code is `encodeFailure`.
  failed
}

/// Error code returned when local audio stream state changes.
enum LocalAudioStreamError {
  /// Local audio status is normal.
  ok,

  /// Unknown error cause.
  failure,

  /// No permission to start local audio recording device.
  deviceNoPermission,

  /// The local audio recording device is already in use.
  deviceBusy,

  /// Local audio recording failed, it is recommended that you check whether the recording device is working properly.
  recordFailure,

  /// Local audio encoding failed.
  encodeFailure,

  /// No audio recording equipment available.
  noRecordingDevice
}

/// Local video stream status
enum LocalVideoStreamState {
  /// Local video default initial state.
  ///
  /// The corresponding error code is `ok`.
  stopped,

  /// The local video recording device starts successfully.
  ///
  /// The corresponding error code is `ok`.
  recording,

  /// The first frame is encoded successfully.
  ///
  /// The corresponding error code is `ok`.
  encoding,

  /// Local video startup failed,  the status is called back at the following times:
  /// + The local acquisition device failed to start, the corresponding error code is `failure`.
  /// + No camera permission was detected, the corresponding error code is `deviceNoPermission`.
  /// + Video encoding failed, the corresponding error code is `encodeFailure`.
  failed
}

/// Error code returned when the local video state changes.
enum LocalVideoStreamError {
  /// Local video status is normal.
  ok,

  /// Local video stream publishing failed.
  failure,

  /// No permission to start local video capture device.
  deviceNoPermission,

  /// Local video capture device is occupied.
  deviceBusy,

  /// Local video capture device does not exist or has been removed.
  deviceNotFound,

  /// Local video capture failed, it is recommended to check whether the capture device is working properly.
  captureFailure,

  /// Local video encoding failure.
  encodeFailure,

  /// The local video capture device is seized by other programs during the call, causing the device connection to be interrupted.
  deviceDisconnected,
}

/// Status of the subscribed remote audio stream and corresponding reason code in [RemoteAudioStateChangeReason].
enum RemoteAudioState {
  /// Initial status, and will be returned at the following times:
  /// + Local user stops receiving the remote audio stream, the reason code is `localMuted`.
  /// + The remote user stops sending the audio stream, the reason code is `remoteMuted`.
  /// + The remote user has left the room, the reason code is `remoteOffline`.
  stopped,

  /// Start receiving the remote audio stream header package.
  ///
  /// When the remote audio stream header is just received, a [RTCVideoEventHandler.onRemoteAudioStateChanged] callback will be triggered, and the reason code is `localUnmuted`.
  starting,

  /// The remote audio stream is decoding and playing normally. The state will be returned at the following times:
  /// + The first frame of the remote audio is successfully decoded, the reason code is `localUnmuted`.
  /// + The network is back to normal from being blocked, the reason code is `networkRecovery`.
  /// + The local user resumes receiving remote audio streams, the reason code is `localUnmuted`.
  /// + The remote user resumes sending audio streams, the reason code is `remoteUnmuted`.
  decoding,

  /// Remote audio stream stalls.
  ///
  /// This state will be called back when network blocking causes packet loss greater than 40%, the reason code is `networkCongestion`.
  frozen,

  /// Remote audio stream playback failed.
  failed
}

/// Reasons for change in remote audio stream status
enum RemoteAudioStateChangeReason {
  /// Internal reasons.
  internal,

  /// Network blocking.
  networkCongestion,

  /// Network back to normal.
  networkRecovery,

  /// Local user stops receiving remote audio streams.
  localMuted,

  /// Local user resumes receiving remote audio streams.
  localUnmuted,

  /// Remote user stops sending audio streams.
  remoteMuted,

  /// Remote user resumes sending audio streams.
  remoteUnmuted,

  /// Remote user leaves the room.
  remoteOffline,
}

/// Remote video stream status and corresponding reason code in [RemoteVideoStateChangedReason].
///
/// [RTCVideoEventHandler.onRemoteVideoStateChanged] will be triggered when status changes.
enum RemoteVideoState {
  /// Initial status. This state will be called back at the following times:
  /// + Local user stops receiving remote video streams, the reason code is `localMuted`.
  /// + Remote user stops sending video streams, the reason code is `remoteMuted`.
  /// + Remote user leaves the room, the reason code is `remoteOffline`.
  stopped,

  /// Local user has received the remote video header package, the reason code is `localUnmuted`.
  starting,

  /// The remote video stream is decoding and playing normally, callback the status at the following times:
  /// + The first frame of remote video is successfully decoded, the reason code is `localUnmuted`.
  /// + The network is back to normal from being blocked, the reason code is `networkRecorvery`.
  /// + The local user resumes receiving remote video streams, the reason code is `localUnmuted`.
  /// + The remote user resumes sending video stream, the reason code is `remoteUnmuted`.
  decoding,

  /// Remote video stream stalls.
  ///
  /// This state will be called back when network blocking causes packet loss greater than 40%, the reason code is `networkCongestion`.
  frozen,

  /// Remote video stream playback failed.
  ///
  /// This state will be called back when internal processing of remote video streams fails, the reason code is `internel`.
  failed,
}

/// Reasons for remote video stream state change
enum RemoteVideoStateChangedReason {
  /// Internal reasons.
  internal,

  /// Network blocking.
  networkCongestion,

  /// Network back to normal.
  networkRecovery,

  /// Local user stops receiving remote video streams or disables video module.
  localMuted,

  /// Local user resumes receiving remote video streams or enables video module.
  localUnmuted,

  /// Remote user stops publishing video streams or disables video module.
  remoteMuted,

  /// Remote user resumes publishing video streams or enables video module.
  remoteUnmuted,

  /// Remote user leaves the room
  remoteOffline
}

/// State of the black frame video stream
enum SEIStreamUpdateEvent {
  /// A black frame video stream is published from the remote user.
  ///
  /// In a voice call, when the remote user calls [RTCVideo.sendSEIMessage] to send SEI data, SDK will automatically publish a black frame video stream, and trigger this callback.
  streamAdd,

  /// The black frame video stream is removed. The timing this callback will be triggered is as following:
  /// + The remote user turns on their camera, switching the voice call to a video call.
  /// + No SEI data is sent within 1min after the remote user calls [RTCVideo.sendSEIMessage].
  streamRemove,
}

/// Performance fallback-related data
class SourceWantedData {
  /// When publish fallback is disabled, this value indicates the recommended video input width.<br>
  /// When publish fallback is enabled, it indicates the maximum width of the current published stream.
  int width;

  /// When publish fallback is disabled, this value indicates the recommended video input height.<br>
  /// When publish fallback is enabled, it indicates the maximum height of the current published stream.
  int height;

  /// When publish fallback is disabled, this value indicates the recommended video input frame rate(fps).<br>
  /// When publish fallback is enabled, it indicates the maximum frame rate of the current published stream.
  int frameRate;

  SourceWantedData(
      {required this.width, required this.height, required this.frameRate});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'width': width,
      'height': height,
      'frameRate': frameRate,
    };
  }

  factory SourceWantedData.fromMap(Map<dynamic, dynamic> map) {
    return SourceWantedData(
      width: map['width'],
      height: map['height'],
      frameRate: map['frameRate'],
    );
  }
}

/// User information
class UserInfo {
  /// User ID
  ///
  /// The string matches the regular expression: `[a-zA-Z0-9_@\-]{1,128}`.
  final String uid;

  /// Additional information the user passed in with a maximum length of 200 bytes, which will be called back to the remote user in [RTCRoomEventHandler.onUserJoined].
  final String metaData;

  const UserInfo({required this.uid, this.metaData = ''});

  factory UserInfo.fromMap(Map<dynamic, dynamic> map) {
    return UserInfo(uid: map['uid'], metaData: map['metaData']);
  }

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'metaData': metaData,
    };
  }
}

/// Call-related statistics
class RTCRoomStats {
  /// Cumulative call duration in seconds after the user enters the room.
  int duration = 0;

  /// Cumulative number of bytes sent by local user.
  int txBytes = 0;

  /// Cumulative number of bytes received by local user.
  int rxBytes = 0;

  /// Uplink code rate (kbps), instantaneous value when obtaining this data.
  int txKBitrate = 0;

  /// Downlink code rate (kbps), instantaneous value when obtaining this data.
  int rxKBitrate = 0;

  /// The code rate (kbps) at which the audio packet is sent, instantaneous value when obtaining this data.
  int txAudioKBitrate = 0;

  /// The code rate (kbps) at which the audio packet is received, instantaneous value when obtaining this data.
  int rxAudioKBitrate = 0;

  /// The code rate (kbps) at which the video packet is sent, instantaneous value when obtaining this data.
  int txVideoKBitrate = 0;

  /// The code rate (kbps) at which the video packet is received, instantaneous value when obtaining this data.
  int rxVideoKBitrate = 0;

  /// The code rate (kbps) at which the screen stream packet is sent, instantaneous value when obtaining this data.
  int txScreenKBitrate = 0;

  /// The code rate (kbps) at which the screen stream packet is received, instantaneous value when obtaining this data.
  int rxScreenKBitrate = 0;

  /// Number of visible users in the current room.
  int userCount = 0;

  /// The uplink packet loss rate of the current application, in the range of `[0, 1]`.
  double txLostrate = 0;

  /// The downlink packet loss rate of the current application, in the range of `[0, 1]`.
  double rxLostrate = 0;

  /// Round-trip time delay for client-to-server data transfer in ms.
  int rtt = 0;

  /// System uplink network jitter in ms.
  int txJitter = 0;

  /// System downlink network jitter in ms.
  int rxJitter = 0;

  /// The code rate (kbps) sent by the cellular path, instantaneous value when obtaining this data.
  int txCellularKBitrate = 0;

  /// The code rate (kbps) received by the cellular path, instantaneous value when obtaining this data.
  int rxCellularKBitrate = 0;

  RTCRoomStats.empty();

  RTCRoomStats(
      {required this.duration,
      required this.txBytes,
      required this.rxBytes,
      required this.txKBitrate,
      required this.rxKBitrate,
      required this.txAudioKBitrate,
      required this.rxAudioKBitrate,
      required this.txVideoKBitrate,
      required this.rxVideoKBitrate,
      required this.txScreenKBitrate,
      required this.rxScreenKBitrate,
      required this.userCount,
      required this.txLostrate,
      required this.rxLostrate,
      required this.rtt,
      required this.txJitter,
      required this.rxJitter,
      required this.txCellularKBitrate,
      required this.rxCellularKBitrate});

  factory RTCRoomStats.fromMap(Map<dynamic, dynamic> map) {
    return RTCRoomStats(
        duration: map['duration'],
        txBytes: map['txBytes'],
        rxBytes: map['rxBytes'],
        txKBitrate: map['txKBitrate'],
        rxKBitrate: map['rxKBitrate'],
        txAudioKBitrate: map['txAudioKBitrate'],
        rxAudioKBitrate: map['rxAudioKBitrate'],
        txVideoKBitrate: map['txVideoKBitrate'],
        rxVideoKBitrate: map['rxVideoKBitrate'],
        txScreenKBitrate: map['txScreenKBitrate'],
        rxScreenKBitrate: map['rxScreenKBitrate'],
        userCount: map['userCount'],
        txLostrate: map['txLostrate'],
        rxLostrate: map['rxLostrate'],
        rtt: map['rtt'],
        txJitter: map['txJitter'],
        rxJitter: map['rxJitter'],
        txCellularKBitrate: map['txCellularKBitrate'],
        rxCellularKBitrate: map['rxCellularKBitrate']);
  }
}

/// Video encoding type
enum VideoCodecType {
  /// Unknown type
  auto,

  /// H.264 format
  h264,

  /// ByteVC1 format
  bytevc1
}

/// Local audio stream statistics called back every 2 seconds.
///
/// After a local user successfully publishes an audio stream, the SDK periodically notifies the user of the status of the published audio stream during this statistics cycle through [RTCRoomEventHandler.onLocalStreamStats].
class LocalAudioStats {
  /// Audio uplink packet loss rate in % for this statistical period, in the range `[0, 1]`.
  double? audioLossRate;

  /// The audio transmission code rate in kbps for this statistical period.
  double? sentKBitrate;

  /// The audio capture sample rate in Hz for this statistical period.
  int? recordSampleRate;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  int? statsInterval;

  /// Round-trip time delay in ms.
  int? rtt;

  /// Number of audio channels.
  int? numChannels;

  /// The audio send sample rate in Hz for this statistical period.
  int? sentSampleRate;

  LocalAudioStats(
      {this.audioLossRate,
      this.sentKBitrate,
      this.recordSampleRate,
      this.statsInterval,
      this.rtt,
      this.numChannels,
      this.sentSampleRate});

  factory LocalAudioStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalAudioStats(
        audioLossRate: map['audioLossRate'],
        sentKBitrate: map['sentKBitrate'],
        recordSampleRate: map['recordSampleRate'],
        statsInterval: map['statsInterval'],
        rtt: map['rtt'],
        numChannels: map['numChannels'],
        sentSampleRate: map['sentSampleRate']);
  }
}

/// Remote audio stream statistics called back every 2 seconds.
///
/// After a local user successfully subscribed to a remote audio stream, the SDK periodically notifies the user of the status of the received audio stream during this statistics cycle through [RTCRoomEventHandler.onRemoteStreamStats].
class RemoteAudioStats {
  /// Audio downlink packet loss rate in % for this statistical period, in the range `[0, 1]`.
  double? audioLossRate;

  /// The audio reception code rate in kbps for this statistical period.
  double? receivedKBitrate;

  /// The number of stalls in this statistical cycle.
  int? stallCount;

  /// The duration of the stall in this statistical period, in ms.
  int? stallDuration;

  /// User experience level end-to-end latency.
  ///
  /// The delay time in ms from the start of encoding at the sender-side after capturing to the start of rendering at the receiver-side after decoding.
  int? e2eDelay;

  /// The audio playback sample rate in Hz for the statistical period.
  int? playoutSampleRate;

  /// The interval of this statistical period, in ms.
  int? statsInterval;

  /// The round-trip time delay of the data transfer from the client to the server, in ms.
  int? rtt;

  /// Sender-server-receiver data transmission time delay in ms.
  int? totalRtt;

  /// The quality of the audio stream sent by the remote user.
  ///
  /// See [NetworkQuality] for the value meaning.
  int? quality;

  /// The delay in ms due to the introduction of the jitter buffer mechanism.
  int? jitterBufferDelay;

  /// Number of audio channels.
  int? numChannels;

  /// Audio reception sample rate in Hz for this statistical cycle.
  int? receivedSampleRate;

  /// Cumulative duration of audio stall occurring after the remote user joins the room as a percentage of the total effective duration of audio.
  ///
  /// Effective audio duration is the duration after the remote user enters the room and publishes the audio stream, except for stopping sending the audio stream and disabling the audio module.
  int? frozenRate;

  /// Total number of audio packet loss compensation (PLC) sample points.
  int? concealedSamples;

  /// Cumulative times of PLC
  int? concealmentEvent;

  /// The audio decoding sample rate in Hz for this statistical period.
  int? decSampleRate;

  /// The total elapsed time in s to decode the received remote audio streams in this statistical period.
  int? decDuration;

  RemoteAudioStats(
      {this.audioLossRate,
      this.receivedKBitrate,
      this.stallCount,
      this.stallDuration,
      this.e2eDelay,
      this.playoutSampleRate,
      this.statsInterval,
      this.rtt,
      this.totalRtt,
      this.quality,
      this.jitterBufferDelay,
      this.numChannels,
      this.receivedSampleRate,
      this.frozenRate,
      this.concealedSamples,
      this.concealmentEvent,
      this.decSampleRate,
      this.decDuration});

  factory RemoteAudioStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteAudioStats(
      audioLossRate: map['audioLossRate'],
      receivedKBitrate: map['receivedKBitrate'],
      stallCount: map['stallCount'],
      stallDuration: map['stallDuration'],
      e2eDelay: map['e2eDelay'],
      playoutSampleRate: map['playoutSampleRate'],
      statsInterval: map['statsInterval'],
      rtt: map['rtt'],
      totalRtt: map['totalRtt'],
      quality: map['quality'],
      jitterBufferDelay: map['jitterBufferDelay'],
      numChannels: map['numChannels'],
      receivedSampleRate: map['receivedSampleRate'],
      frozenRate: map['frozenRate'],
      concealedSamples: map['concealedSamples'],
      concealmentEvent: map['concealmentEvent'],
      decSampleRate: map['decSampleRate'],
      decDuration: map['decDuration'],
    );
  }
}

/// Local video stream statistics called back every 2 seconds.
///
/// After a local user successfully publishes a video stream, the SDK periodically notifies the user of the status of the published video stream during this statistics cycle through [RTCRoomEventHandler.onLocalStreamStats].
class LocalVideoStats {
  /// The sending code rate in Kbps of the video stream with the largest resolution actually published in this statistical period.
  double? sentKBitrate;

  /// The capture frame rate in Hz for this statistical period.
  int? inputFrameRate;

  /// The video send sample rate in Hz for this statistical period.
  int? sentFrameRate;

  /// Encoder output frame rate in fps for the current statistical period.
  int? encoderOutputFrameRate;

  /// The local video rendering frame rate in fps for this statistical period.
  int? renderOutputFrameRate;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  int? statsInterval;

  /// The video uplink packet loss rate in this statistical period, the value range is `[0, 1]`.
  double? videoLossRate;

  /// Round-trip time delay in ms.
  int? rtt;

  /// The video encoding bit rate in kbps of the largest resolution video stream actually published in this statistical period.
  int? encodedBitrate;

  /// The video encoding width in px of the video stream with the largest resolution actually published.
  int? encodedFrameWidth;

  /// The video encoding height in px of the video stream with the largest resolution actually published.
  int? encodedFrameHeight;

  /// The total number of video frames sent for the video stream with the largest resolution actually published during this statistical period.
  int? encodedFrameCount;

  /// Encoding type of the video.
  int? codecType;

  /// Whether the media stream is from screen sharing.
  bool? isScreen;

  LocalVideoStats(
      {this.sentKBitrate,
      this.inputFrameRate,
      this.sentFrameRate,
      this.encoderOutputFrameRate,
      this.renderOutputFrameRate,
      this.statsInterval,
      this.videoLossRate,
      this.rtt,
      this.encodedBitrate,
      this.encodedFrameWidth,
      this.encodedFrameHeight,
      this.encodedFrameCount,
      this.codecType,
      this.isScreen});

  factory LocalVideoStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalVideoStats(
        sentKBitrate: map['sentKBitrate'],
        inputFrameRate: map['inputFrameRate'],
        sentFrameRate: map['sentFrameRate'],
        encoderOutputFrameRate: map['encoderOutputFrameRate'],
        renderOutputFrameRate: map['renderOutputFrameRate'],
        statsInterval: map['statsInterval'],
        videoLossRate: map['videoLossRate'],
        rtt: map['rtt'],
        encodedBitrate: map['encodedBitrate'],
        encodedFrameWidth: map['encodedFrameWidth'],
        encodedFrameHeight: map['encodedFrameHeight'],
        encodedFrameCount: map['encodedFrameCount'],
        codecType: map['codecType'],
        isScreen: map['isScreen']);
  }
}

/// Remote video stream statistics called back every 2 seconds.
///
/// After a local user successfully subscribed to a remote video stream, the SDK periodically notifies the user of the status of the received video stream during this statistics cycle through [RTCRoomEventHandler.onRemoteStreamStats].
class RemoteVideoStats {
  /// Remote video stream width
  int? width;

  /// Remote video stream height
  int? height;

  /// The video downlink packet loss rate in % in the range of `[0, 1]` for this statistical period.
  double? videoLossRate;

  /// The received bit rate of the video in kbps for this statistical period.
  double? receivedKBitrate;

  /// The frame rate of the video decoder output in fps for this statistical period.
  int? decoderOutputFrameRate;

  /// The frame rate of the video rendering in fps during the statistical period.
  int? renderOutputFrameRate;

  /// Number of stalls in the statistical cycle.
  int? stallCount;

  /// The total duration of video stalls in the statistics period in ms.
  int? stallDuration;

  /// Delay in ms from the completion of encoding at the sender-side to the completion of decoding and rendering at the receiver-side
  int? e2eDelay;

  /// Whether the media stream is from screen sharing.
  bool? isScreen;

  /// The interval of this statistical period, in ms.
  ///
  /// This field is used to set the statistics period of the callback, the default setting is 2000ms.
  int? statsInterval;

  /// Round-trip time delay in ms.
  int? rtt;

  /// Cumulative duration of video stalls occurring after the remote user enters the room as a percentage of the total effective duration of the video.
  ///
  /// Effective video duration is the duration of video after the remote user enters the room and publishes the video stream, except for stopping sending the video stream and disabling the video module.
  int? frozenRate;

  /// Encoding type of the video.
  VideoCodecType? codecType;

  /// Numbering of video streams with different resolution.
  int? videoIndex;

  RemoteVideoStats(
      {this.width,
      this.height,
      this.videoLossRate,
      this.receivedKBitrate,
      this.decoderOutputFrameRate,
      this.renderOutputFrameRate,
      this.stallCount,
      this.stallDuration,
      this.e2eDelay,
      this.isScreen,
      this.statsInterval,
      this.rtt,
      this.frozenRate,
      this.codecType,
      this.videoIndex});

  factory RemoteVideoStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteVideoStats(
        width: map['width'],
        height: map['height'],
        videoLossRate: map['videoLossRate'],
        receivedKBitrate: map['receivedKBitrate'],
        decoderOutputFrameRate: map['decoderOutputFrameRate'],
        renderOutputFrameRate: map['renderOutputFrameRate'],
        stallCount: map['stallCount'],
        stallDuration: map['stallDuration'],
        e2eDelay: map['e2eDelay'],
        isScreen: map['isScreen'],
        statsInterval: map['statsInterval'],
        rtt: map['rtt'],
        frozenRate: map['frozenRate'],
        codecType: (map['codecType'] as int).videoCodecType,
        videoIndex: map['videoIndex']);
  }
}

/// Local audio/video stream statistics and network status called back every 2s.
class LocalStreamStats {
  /// Statistics of audio stream published by local device.
  LocalAudioStats? audioStats;

  /// Statistics of video stream published by local device.
  LocalVideoStats? videoStats;

  /// Whether the media stream is from screen sharing.
  bool? isScreen;

  LocalStreamStats({this.audioStats, this.videoStats, this.isScreen});

  factory LocalStreamStats.fromMap(Map<dynamic, dynamic> map) {
    return LocalStreamStats(
      audioStats: LocalAudioStats.fromMap(map['audioStats']),
      videoStats: LocalVideoStats.fromMap(map['videoStats']),
      isScreen: map['isScreen'],
    );
  }
}

/// Statistical information and network status of subscribed remote audio/video stream called back every 2s.
class RemoteStreamStats {
  /// ID of the user who published the audio
  String? uid;

  /// Statistics of remote audio stream
  RemoteAudioStats? audioStats;

  /// Statistics of remote video stream
  RemoteVideoStats? videoStats;

  /// Whether the media stream is from screen sharing.
  bool? isScreen;

  RemoteStreamStats(
      {this.uid, this.audioStats, this.videoStats, this.isScreen});

  factory RemoteStreamStats.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamStats(
      audioStats: RemoteAudioStats.fromMap(map['audioStats']),
      videoStats: RemoteVideoStats.fromMap(map['videoStats']),
      uid: map['uid'],
      isScreen: map['isScreen'],
    );
  }
}

/// Media device state
enum MediaDeviceState {
  /// Start capturing.
  started,

  /// Stop capturing.
  stopped,

  /// The media device is expected to be working but no data is received.
  runtimeError,

  /// Device is plugged in.
  added,

  /// Device is removed.
  removed,

  /// Phone calls, locking screen or the other Apps interrupted the RTC call. RTC call will resume once the phone call ends or the other Apps release the media devices.
  interruptionBegan,

  /// RTC call resumed from the interruption caused by the phone calls or the other Apps.
  interruptionEnded,
}

/// Media device error
enum MediaDeviceError {
  /// Normal.
  ok,

  /// No permission to start the media device.
  deviceNoPermission,

  /// The media device is already in use.
  deviceBusy,

  /// Media device error.
  deviceFailure,

  /// The specified media device was not found.
  deviceNotFound,

  /// The media device is removed.
  deviceDisconnected,

  /// he media device is expected to be working but no data is received.
  deviceNoCallback,

  /// Device sample rate is not supported.
  unsupportedFormat,

  /// No group id parameter for iOS screen capture.
  notFindGroupId,
}

/// Media device warning
enum MediaDeviceWarning {
  /// No warning.
  ok,

  /// Illegal operation.
  ///
  /// Calls the API for internal device when using the external device.
  operationDenied,

  /// The media device is capturing silent frames.
  captureSilence,

  /// Android-specific silence, system-level silence reporting.
  androidSysSilence,

  /// Android-specific silence disappears.
  androidSysSilenceDisappear,

  /// The volume is too high and exceeds the capture range of the device, it is recommended to reduce the microphone volume or reduce the volume of the sound source.
  ///
  /// Applies to conference scenario.
  detectClipping,

  /// Echo leakage.
  ///
  /// Applies to conference scenario.
  detectLeakEcho,

  /// Low signal-to-noise ratio.
  ///
  /// Applies to conference scenario.
  detectLowSNR,

  /// Silence inserted during capture.
  ///
  /// Applies to conference scenario.
  detectInsertSilence,

  /// Silence during capture.
  ///
  /// Applies to conference scenario.
  captureDetectSilence,

  /// Silence during capture disappears.
  ///
  /// Applies to conference scenario.
  captureDetectSilenceDisappear,

  /// Howling.
  ///
  /// Applies to conference scenario.
  captureDetectHowling,

  /// Setting audio route failed because the audio route can not be changed in the current Audio Scenario.
  setAudioRouteInvalidScenario,

  /// Setting audio route failed because the routing device did not exist.
  ///
  /// Applies to Android.
  setAudioRouteNotExists,

  /// Setting audio route failed because the audio route was occupied by the system process or the other Apps.
  setAudioRouteFailedByPriority,

  /// Setting audio route failed because the audio route can only be changed in the communication audio mode.
  ///
  /// Applies to Android.
  setAudioRouteNotVoipMode,

  ///  Setting audio route failed because the audio device did not initiate.
  setAudioRouteDeviceNotStart,
}

/// Audio/video quality feedback
enum ProblemFeedback {
  /// Good
  none,

  /// Undefined problems
  otherMsg,

  /// Unclear voice
  audioNotClear,

  /// Unclear video
  videoNotClear,

  /// Audio & video out of sync
  sync,

  /// Audio stall
  audioLagging,

  /// Video stall
  videoDelay,

  /// Disconnection
  disconnect,

  /// No sound
  noAudio,

  /// No picture
  noVideo,

  /// Sound too low
  audioStrength,

  /// Echo noise
  echo,

  /// Large delay in in-ear monitoring
  earBackDelay,
}

/// Information on CPU and memory used by App.
class SysStats {
  /// Number of CPU cores on current system.
  int cpuCores;

  /// The CPU usage of the current application, the value range is [0,1].
  double cpuAppUsage;

  /// The current system CPU usage, the value range is [0,1].
  double cpuTotalUsage;

  /// Memory usage of the current app in MB.
  double memoryUsage;

  /// Memory size of the device in MB.
  int fullMemory;

  /// System memory used in MB.
  int totalMemoryUsage;

  /// Current free system memory in MB
  int freeMemory;

  /// Current application memory usage (%)
  double memoryRatio;

  /// System memory usage (%)
  double totalMemoryRatio;

  SysStats(
      {required this.cpuCores,
      required this.cpuAppUsage,
      required this.cpuTotalUsage,
      required this.memoryUsage,
      required this.fullMemory,
      required this.totalMemoryUsage,
      required this.freeMemory,
      required this.memoryRatio,
      required this.totalMemoryRatio});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'cpuCores': cpuCores,
      'cpuAppUsage': cpuAppUsage,
      'cpuTotalUsage': cpuTotalUsage,
      'memoryUsage': memoryUsage,
      'fullMemory': fullMemory,
      'totalMemoryUsage': totalMemoryUsage,
      'freeMemory': freeMemory,
      'memoryRatio': memoryRatio,
      'totalMemoryRatio': totalMemoryRatio,
    };
  }

  factory SysStats.fromMap(Map<dynamic, dynamic> map) {
    return SysStats(
      cpuCores: map['cpuCores'],
      cpuAppUsage: map['cpuAppUsage'],
      cpuTotalUsage: map['cpuTotalUsage'],
      memoryUsage: map['memoryUsage'],
      fullMemory: map['fullMemory'],
      totalMemoryUsage: map['totalMemoryUsage'],
      freeMemory: map['freeMemory'],
      memoryRatio: map['memoryRatio'],
      totalMemoryRatio: map['totalMemoryRatio'],
    );
  }
}

/// Custom encryption type
enum EncryptType {
  /// No built-in encryption used (default)
  customize,

  /// AES-128-CBC encryption algorithm
  aes128CBC,

  /// AES-256-CBC encryption algorithm
  aes256CBC,

  /// AES-128-ECB encryption algorithm
  aes128ECB,

  /// AES-256-ECB encryption algorithm
  aes256ECB,
}

/// First frame sending status
enum FirstFrameSendState {
  /// Sending
  sending,

  /// Sent successfully
  sent,

  /// Send failed
  end,
}

/// First frame playback status
enum FirstFramePlayState {
  /// Playing
  playing,

  /// Played successfully
  played,

  /// Play failed
  end,
}

/// Stream type
enum StreamIndex {
  /// Mainstream.
  ///
  /// Video/audio captured by the the camera/microphone using internal capturing.
  main,

  /// Screen-sharing stream.
  ///
  /// Video/Audio streams from screen sharing.
  screen
}

/// Information about the remote stream
class RemoteStreamKey {
  /// The ID of the room from which the stream was published.
  final String roomId;

  /// The ID of the user who published the stream.
  final String uid;

  /// Stream type.
  final StreamIndex streamIndex;

  const RemoteStreamKey(
      {required this.roomId, required this.uid, required this.streamIndex});

  factory RemoteStreamKey.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamKey(
        roomId: map['roomId'],
        uid: map['uid'],
        streamIndex: (map['streamIndex'] as int).streamIndex);
  }

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'uid': uid,
      'streamIndex': streamIndex.value
    };
  }
}

/// Storage format of locally recorded files
enum RecordingFileType {
  /// aac format
  aac,

  /// mp4 format
  mp4,
}

/// Status of local recording
enum RecordingState {
  /// Recording exception.
  error,

  /// Recording in progress.
  processing,

  /// Recorded file saved successfully.
  ///
  /// Received after calling [RTCVideo.stopFileRecording] to end the recording.
  success
}

/// Local recording error code
enum RecordingErrorCode {
  /// Recording normally.
  ok,

  /// No file write permission.
  noPermission,

  /// The current version of the SDK does not provide local recording function, please contact technical support.
  notSupport,

  /// Other exceptions.
  noOther,
}

/// Local recording progress
class RecordingProgress {
  /// The cumulative recording time of the current file, in milliseconds.
  int duration;

  /// The size of the currently recorded file in bytes.
  int fileSize;

  RecordingProgress({
    required this.duration,
    required this.fileSize,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'fileSize': fileSize,
      'duration': duration,
    };
  }

  factory RecordingProgress.fromMap(Map<dynamic, dynamic> map) {
    return RecordingProgress(
      duration: map['duration'],
      fileSize: map['fileSize'],
    );
  }
}

/// Details of local recording
class RecordingInfo {
  /// The absolute path of the recorded file, including file name and file suffix.
  String filePath;

  /// Video encoding type of recorded files.
  VideoCodecType videoCodecType;

  /// The width of the recorded video, in pixels. Please ignore this field for audio-only recording.
  int width;

  /// The height of the recorded video, in pixels. Please ignore this field for audio-only recording.
  int height;

  RecordingInfo(
      {required this.filePath,
      required this.videoCodecType,
      required this.width,
      required this.height});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'width': width,
      'height': height,
      'filePath': filePath,
      'videoCodecType': videoCodecType.value,
    };
  }

  factory RecordingInfo.fromMap(Map<dynamic, dynamic> map) {
    return RecordingInfo(
      filePath: map['filePath'],
      videoCodecType: (map['videoCodecType'] as int).videoCodecType,
      width: map['width'],
      height: map['height'],
    );
  }
}

/// Types of locally recorded media
enum RecordingType {
  /// Audio only
  audioOnly,

  /// Video only
  videoOnly,

  /// Both audio and video
  videoAndAudio,
}

/// Local recording configuration
class RecordingConfig {
  /// The absolute path to save the recording file. You need to specify a legal path with read and write permissions.
  String dirPath;

  /// Storage Format。
  RecordingFileType recordingFileType;

  RecordingConfig({
    required this.dirPath,
    this.recordingFileType = RecordingFileType.mp4,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'dirPath': dirPath,
      'recordingFileType': recordingFileType.value,
    };
  }
}

/// A/V synchronization states
enum AVSyncState {
  /// A/V synchronization begins.
  streamSyncBegin,

  /// The audio stream is removed during the synchronization which, however, does not affect the sync relationship.
  audioStreamRemove,

  /// The video stream is removed during the synchronization which, however, does not affect the sync relationship.
  videoStreamRemove,

  /// Subscriber settings synchronization.
  ///
  /// Currently not available.
  setAVSyncStreamId,
}

/// Stream type for synchronization
enum SyncInfoStreamType {
  /// Audio stream
  audio,
}

/// Configuration related to media streaming information synchronization
class StreamSyncInfoConfig {
  /// Stream type.
  StreamIndex streamIndex;

  /// The number of repetitions of the message sent, in the range `[0,25]`. It is recommended to be set to `[3,5]`.
  int repeatCount;

  /// Stream type for synchronization.
  SyncInfoStreamType streamType;

  StreamSyncInfoConfig({
    this.streamIndex = StreamIndex.main,
    this.repeatCount = 0,
    this.streamType = SyncInfoStreamType.audio,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'streamIndex': streamIndex.value,
      'repeatCount': repeatCount,
      'streamType': streamType.value,
    };
  }
}

/// Information about the target room for media stream relaying across rooms.
class ForwardStreamInfo {
  /// Target room ID
  String roomId;

  /// Token generated using the target room RoomID and UserID.
  ///
  /// For testing, you can use the console to generate temporary Token, for official launch you need to use the key SDK to generate and issue Token on your server.<br>
  /// Invalid token will cause relaying failure.
  String token;

  ForwardStreamInfo({
    required this.roomId,
    required this.token,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'token': token,
    };
  }
}

/// Media stream relaying status
enum ForwardStreamState {
  /// No stream relaying.
  ///
  /// + After [RTCRoom.stopForwardStreamToRooms] is called, all target rooms are in this status.
  /// + When number of target rooms is reduced by calling [RTCRoom.updateForwardStreamToRooms], the target room for this reduction is in this status.
  idle,

  /// Relaying started.
  ///
  /// + Calling [RTCRoom.startForwardStreamToRooms] to successfully start relaying media stream to all rooms will have this status returned.
  /// + Calling [RTCRoom.updateForwardStreamToRooms] to successfully started relaying media stream to the new target rooms will have this status returned.
  success,

  /// relaying failed, see [ForwardStreamError] for details.
  ///
  /// This status will be called back in case of relaying failure after calling [RTCRoom.startForwardStreamToRooms] or [RTCRoom.updateForwardStreamToRooms].
  failure,
}

/// Error code during cross-room relaying of media streams
enum ForwardStreamError {
  /// Normal.
  ok,

  /// Parameter exception.
  invalidArgument,

  /// Invalid token.
  invalidToken,

  /// Error returned from server.
  response,

  /// relaying is interrupted because a user with the same userID joins the target room.
  remoteKicked,

  /// The server does not support cross-room relaying function.
  notSupport,
}

/// Events occuring in this target room during media stream relaying.
enum ForwardStreamEvent {
  /// The network connection between the local end and the server is disconnected and relaying is suspended.
  disconnected,

  /// Network connection between local and server is restored, and relaying service is successfully connected.
  connected,

  /// relaying is interrupted because a user with the same userID joins the target room.
  interrupt,

  /// The target room is updated by calling [RTCRoom.updateForwardStreamToRooms].
  dstRoomUpdated,

  /// API call timing error.
  ///
  /// For example, call [RTCRoom.updateForwardStreamToRooms] before [RTCRoom.startForwardStreamToRooms].
  unexpectedAPICall,
}

/// Status and error messages for different target rooms during cross-room relaying of media streams
class ForwardStreamStateInfo {
  /// Target room ID.
  ///
  /// Empty string represents all target rooms.
  String roomId;

  /// Status of the target room.
  ForwardStreamState state;

  /// Error that occurs during relaying.
  ForwardStreamError error;

  ForwardStreamStateInfo({
    required this.roomId,
    required this.state,
    required this.error,
  });

  factory ForwardStreamStateInfo.fromMap(Map<dynamic, dynamic> map) {
    return ForwardStreamStateInfo(
      roomId: map['roomId'],
      state: (map['state'] as int).forwardStreamState,
      error: (map['error'] as int).forwardStreamError,
    );
  }
}

/// Information on events occuring in different target room during media stream relaying.
class ForwardStreamEventInfo {
  /// Target room ID.
  ///
  /// Empty string represents all target rooms.
  String roomId;

  /// Events occurring in this target room.
  ForwardStreamEvent event;

  ForwardStreamEventInfo({
    required this.roomId,
    required this.event,
  });

  factory ForwardStreamEventInfo.fromMap(Map<dynamic, dynamic> map) {
    return ForwardStreamEventInfo(
      roomId: map['roomId'],
      event: (map['event'] as int).forwardStreamEvent,
    );
  }
}

/// Cloud proxy information
class CloudProxyInfo {
  /// Server IP
  String cloudProxyIp;

  /// Server port
  int cloudProxyPort;

  CloudProxyInfo({required this.cloudProxyIp, required this.cloudProxyPort});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'cloudProxyIp': cloudProxyIp,
      'cloudProxyPort': cloudProxyPort,
    };
  }
}

/// Video/audio call test result
enum EchoTestResult {
  /// The playback of captured audio/video is received, test succeeded.
  success,

  /// Test is not completed after 60 seconds and has been stopped automatically.
  timeout,

  /// Less than 5s between the end of the last test and the start of the next test.
  intervalShort,

  /// Audio capture error.
  audioDeviceError,

  /// Video capture error.
  videoDeviceError,

  /// Audio reception error.
  audioReceiveError,

  /// Video reception error.
  videoReceiveError,

  /// Unrecoverable internal error.
  internalError,
}

/// Video/audio call test configurations
class EchoTestConfig {
  /// User ID for testing.
  String uid;

  /// ID of the room that the tested user will join.
  String roomId;

  /// Token used for authenticating users when they enter the room.
  String token;

  /// Whether to test audio. The device to be tested is the default audio device.
  ///
  /// + true: Yes, and the device microphone will automatically turn on and [RTCVideoEventHandler.onLocalAudioPropertiesReport] will be triggered when the value of audioReportInterval is set to greater than 0, from which you can judge the working status of the microphone.
  /// + flase: No.
  bool enableAudio;

  /// Whether to test video.
  ///
  /// + true: Yes, and the device camera will automatically turn on.
  /// + flase: No.
  ///
  /// The video is published with fixed parameters: resolution 640px × 360px, frame rate 15fps.
  bool enableVideo;

  /// Volume prompt interval in ms, the default value is 100.
  ///
  /// + `<= 0`: Turn off prompt.
  /// + `(0,100]`: Invalid interval value, and will be automatically reset to 100ms.
  /// + `> 100`: The actual value of interval.
  int audioReportInterval;

  EchoTestConfig(
      {required this.uid,
      required this.roomId,
      required this.token,
      required this.enableAudio,
      required this.enableVideo,
      required this.audioReportInterval});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'roomId': roomId,
      'token': token,
      'enableAudio': enableAudio,
      'enableVideo': enableVideo,
      'audioReportInterval': audioReportInterval,
    };
  }
}
