// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_audio_defines.dart';
import 'bytertc_range_audio_observer.dart';

/// Range audio instance
abstract class RTCRangeAudio {
  /// Enables/disables the range audio function.
  ///
  /// Range audio means that within a certain range in a same RTC room, the audio volume of the remote user received by the local user will be amplified/attenuated as the remote user moves closer/away. The audio coming from out of the range cannot be heard. See [RTCRangeAudio.updateReceiveRange] to set audio receiving range.
  ///
  /// You can call this API anytime before or after entering a room. To ensure a smooth switch to the range audio mode after entering the room, you need to call [RTCRangeAudio.updatePosition] before this API to set your own position coordinates, and then enable the range audio function.
  Future<void> enableRangeAudio(bool enable);

  /// Updates the audio receiving range for the local user.
  ///
  /// [range]: Audio receiving range.
  ///
  /// API call result:
  /// + 0: Success.
  /// + !0: Failure.
  ///
  /// If you have previously called [RTCRangeAudio.registerRangeAudioObserver] to set an observer monitoring attenuation coefficient, you will receive [RTCRangeAudioObserver.onRangeAudioInfo] when any user in the room calls this API to update the audio receiving range.
  Future<int?> updateReceiveRange(ReceiveRange range);

  /// Updates the coordinate of the local user's position in the rectangular coordinate system in the current room.
  ///
  /// [pos]: 3D coordinate values, the default value is [0, 0, 0].
  ///
  /// API call result:
  /// + 0: Success.
  /// + !0: Failure.
  ///
  /// Notes:
  /// + After calling this API, you should call [RTCRangeAudio.enableRangeAudio] to enable range audio function to actually enjoy the range audio effect.
  /// + If you have previously called [RTCRangeAudio.registerRangeAudioObserver] to set an observer monitoring attenuation coefficient, you will receive [RTCRangeAudioObserver.onRangeAudioInfo] when any user in the room calls this API to update their coordinates.
  Future<int?> updatePosition(Position pos);

  /// Sets the attenuation coefficient observer.
  ///
  /// [observer]: Attenuation coefficient observer.<br>
  /// After you call this API, SDK will trigger [RTCRangeAudioObserver.onRangeAudioInfo] when it detects that a user in the room updates their location coordinates or audio receiving.
  ///
  /// Notes: You should only call this API in manual subscription mode.
  Future<void> registerRangeAudioObserver(RTCRangeAudioObserver? observer);

  /// Set the volume attenuation mode of the range audio.
  ///
  /// After the attenuation mode is changed, the [RTCRangeAudioObserver.onRangeAudioInfo] callback will calculate and return the volume attenuation value based on the last attenuation mode you set.
  ///
  /// [type]: Volume attenuation mode, the default is linear attenuation.
  ///
  /// [coefficient]： Volume attenuation factor in exponential attenuation mode. The default value is 1.<br>
  /// The value range is [0.1,100]. It is recommended to set this value to `50`.<br>
  /// The higher the value, the faster the attenuation of the volume.
  ///
  /// API call result:
  /// + `0`: Success.
  /// + `-1`: Failure, because you called this API before calling [RTCRangeAudio.enableRangeAudio] to enable range audio or before entering the room.
  ///
  /// Notes: Set the volume attenuation range with [RTCRangeAudio.updateReceiveRange].
  Future<int?> setAttenuationModel(
      {required AttenuationType type, required double coefficient});
}
