// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/services.dart';

import '../src/bytertc_render_view_impl.dart';
import 'bytertc_common_defines.dart';
import 'bytertc_video_defines.dart';

/// Video canvas type
enum VideoCanvasType {
  /// Canvas for rendering local video.
  local,

  /// Canvas for rendering remote video.
  remote,

  /// Canvas for rendering public video.
  publicStream,

  /// Canvas for local video during a call test
  echoTest,
}

/// Used for initialize [RTCSurfaceView]
class RTCViewContext {
  /// Video canvas type
  final VideoCanvasType canvasType;

  /// ID of the room where the publisher of the stream to be rendered is in.
  final String roomId;

  /// ID of the publisher of the stream to be rendered.
  final String uid;

  /// Stream type.
  final StreamIndex streamType;

  /// Sets the canvas for rendering local video.
  ///
  /// The application binds the canvas to the local video stream by calling this API.<br>
  /// In application development, this method is usually called after initialization but before joining the room. The binding remains in effect after leaving the room.<br>
  /// Call [RTCVideo.removeLocalVideo] to unbind.
  RTCViewContext.localContext(
      {required this.uid, this.streamType = StreamIndex.main})
      : canvasType = VideoCanvasType.local,
        roomId = '';

  /// Sets the canvas for rendering remote video.
  ///
  /// The application binds the canvas to the local video stream by calling this API. The binding remains in effect after leaving the room.<br>
  /// Call [RTCVideo.removeRemoteVideo] to unbind.
  RTCViewContext.remoteContext(
      {required this.roomId,
      required this.uid,
      this.streamType = StreamIndex.main})
      : canvasType = VideoCanvasType.remote;

  /// Sets the canvas for rendering public video streams.
  ///
  /// The application binds the canvas to the public video stream by calling this API. The binding remains in effect after leaving the room.<br>
  /// Call [RTCVideo.removePublicStreamVideo] to unbind.
  RTCViewContext.publicStreamContext({required String publicStreamId})
      : canvasType = VideoCanvasType.publicStream,
        roomId = '',
        uid = publicStreamId,
        streamType = StreamIndex.main;

  /// Sets the canvas for rendering pre-call echo test video streams.
  RTCViewContext.echoTestContext()
      : canvasType = VideoCanvasType.echoTest,
        roomId = '',
        uid = '',
        streamType = StreamIndex.main;
}

/// Video rendering settings
///
/// Different platforms for different objects:
/// + Android: [TextureView](https://developer.android.com/reference/android/view/TextureView).
/// + iOS: [UIView](https://developer.apple.com/documentation/uikit/uiview).
/// + Not applicable for other platforms.
class RTCSurfaceView extends StatefulWidget {
  /// Context for instance initialization.
  final RTCViewContext context;

  /// Video rendering mode.
  final VideoRenderMode renderMode;

  /// The background color used to fill the blank part of the canvas.
  ///
  /// The value range is `[0x0000000, 0xFFFFFFFF]`.<br>
  /// The default value is `0x00000000`.
  final int backgroundColor;

  /// Temporarily unavailable.
  ///
  /// Sets whether the Surface of the `SurfaceView` is placed at the top of the window it is in. <br>
  /// See [setZOrderOnTop](https://developer.android.com/reference/android/view/SurfaceView#setZOrderOnTop(boolean)) for details. Applies to  Android only.
  final bool zOrderOnTop;

  /// Temporarily unavailable.
  ///
  /// Sets whether the Surface of the `SurfaceView` is placed on top of another regular `SurfaceView`.<br>
  /// See [setZOrderMediaOverlay](https://developer.android.com/reference/android/view/SurfaceView#setZOrderMediaOverlay(boolean)) for details. Applies to  Android only.
  final bool zOrderMediaOverlay;

  /// This callback will be triggered when the `PlatformView` is created.
  final PlatformViewCreatedCallback? onPlatformViewCreated;

  /// Which gestures should be passed to the `PlatformView`.
  ///
  /// + iOS: See [gestureRecognizers property](https://api.flutter.dev/flutter/widgets/UiKitView/gestureRecognizers.html) for details.
  /// + Android: See [gestureRecognizers property](https://api.flutter.dev/flutter/widgets/AndroidView/gestureRecognizers.html) for details.
  final Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers;

  const RTCSurfaceView({
    Key? key,
    required this.context,
    this.renderMode = VideoRenderMode.hidden,
    this.backgroundColor = 0,
    this.zOrderOnTop = false,
    this.zOrderMediaOverlay = false,
    this.onPlatformViewCreated,
    this.gestureRecognizers,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => RTCSurfaceViewState();
}
