// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_common_defines.dart';
import 'bytertc_range_audio_api.dart';
import 'bytertc_room_event_handler.dart';
import 'dart:typed_data';

import 'bytertc_spatial_audio_api.dart';
import 'bytertc_video_defines.dart';

/// Room instance
abstract class RTCRoom {
  /// Gets the room ID.
  String get roomId;

  /// Exit and destroy the room created by calling [RTCVideo.createRTCRoom].
  Future<void> destroy();

  /// Set the event handler to monitor callback events in the room.
  void setRTCRoomEventHandler(RTCRoomEventHandler eventHandler);

  /// Join the room.
  ///
  /// In a multi-room scenario, after calling [RTCVideo.createRTCRoom], you can call this APi to join the created room and make audio/video calls with other users in the room.
  ///
  /// [token]: Dynamic key for authentication and verification of users entering the room.  <br>
  /// You can use the console to generate a temporary token for testing, but should use the key SDK to generate and issue a token on your server for official launch.  <br>
  /// Apps with different AppIDs can not communicate with each other. Make sure that the AppID used to generate the Token is the same as the AppID used to create the engine, otherwise it will cause failure of joining room.
  ///
  /// [userInfo]: User information.
  ///
  /// [roomConfig]: Room configuration, setting the room mode and whether to automatically publish or subscribe to streams.
  ///
  /// API call result:
  /// + 0: Success
  /// + -1: Failure, invalid parameter in userInfo.
  /// + -2: Failure. After successfully calling this API, this return value is triggered when [RTCRoom.joinRoom] is called again, regardless of whether the filled room ID and user ID are duplicated, as long as a return value of 0 is received and [RTCRoom.leaveRoom] is not called successfully.
  /// + -3: Failure, no existed room.
  ///
  /// Notes:
  /// + In the same room with the same AppID, each user ID must be unique. If two users have the same user ID, the user who joined the room later will kick the user who joined the room first out of the room, and the user who joined the room first will receive [RTCVideoEventHandler.onError], and the error code is `duplicateLogin`.
  /// + Local user will receive [RTCRoomEventHandler.onRoomStateChanged] after calling this method to join the room successfully.
  /// + After the local user calls [RTCRoom.setUserVisibility] to set himself/herself to be visible and successfully joins the room, all the remote users will receive [RTCRoomEventHandler.onUserJoined].
  /// + After a user joins a room successfully, the SDK may lose connection to the server in case of poor local network conditions, in which case the SDK will automatically reconnect. After a successful reconnection, the local user will receive [RTCRoomEventHandler.onRoomStateChanged], if the user joining the room is an visible user, the remote users will receive [RTCRoomEventHandler.onUserJoined].
  Future<int?> joinRoom(
      {required String token,
      required UserInfo userInfo,
      required RoomConfig roomConfig});

  /// Set user visibility (visible by default)
  ///
  /// Set user visibility and user behavior in the room through [enable]:
  /// + true: The user can publish media streams. The other users in the room can get informed of the behaviors of this user, such as joining room, starting video capture, and leaving room.
  /// + false: The user cannot publish media streams. The other users in the room can not get informed of the behaviors of this user, such as joining room, starting video capture, or leaving room.
  ///
  /// Notes:
  /// + You can call this API whether the user is in a room or not.
  /// + When you call this API, the other users in the room will be notified with the related callback:
  ///   - Switch from `false` to `true`: [RTCRoomEventHandler.onUserJoined].
  ///   - Switch from `true` to `false`: [RTCRoomEventHandler.onUserLeave].
  /// + The invisible user will receive [RTCEngineEventHandler.onWarning] when trying to publish media streams.
  Future<void> setUserVisibility(bool enable);

  /// Synchronizes published audio and video.
  ///
  /// When the same user simultaneously uses separate devices to capture and publish audio and video, there is a possiblity that the streams are out of sync due to the network disparity. <br>
  /// In this case, you can call this API on the video publisher side and the SDK will automatically line the video stream up according to the timestamp of the audio stream, ensuring that the audio the receiver hears corresponds to the video the receiver watches.
  ///
  /// Pass in the user ID of the audio publisher via [audioUserId].<br>
  /// You can stop the current A/V synchronization by setting this parameter to null.
  ///
  /// Notes:
  /// + You can call this API whether the user is in a room or not.
  /// + The source user IDs of the audio and video stream to be synchronized must be in the same RTC room.
  /// + When the A/V synchronization state changes, you will receive [RTCRoomEventHandler.onAVSyncStateChange].
  /// + More than one pair of audio and video can be synchronized simultaneously in the same RTC room, but you should note that one single audio source cannot be synchronized with multiple video sources at the same time.
  /// + If you want to change the audio source, call this API again with a new `audioUid`. If you want to change the video source, you need to stop the current synchronization first, then call this API on the new video publisher side.
  Future<int?> setMultiDeviceAVSync(String audioUid);

  /// Leaves the room.
  ///
  /// The user calls this method to leave the room, end the call process, and release all call-related resources.
  ///
  /// Notes:
  /// + After a visible user leaves room, the other users in the room will receive [RTCRoomEventHandler.onUserLeave].
  /// + After the user leaves the room, they will receive [RTCRoomEventHandler.onLeaveRoom] themselves.
  Future<void> leaveRoom();

  /// Updates Token.
  ///
  /// The Token used when joining the room has a certain expiration date. [RTCRoomEventHandler.onTokenWillExpire] will be triggered 30 seconds before the Token expires, then you need to retrieve the Token and call this method to update the Token, otherwise the user will be removed from the room.<br>
  /// When calling [RTCRoom.joinRoom] to join a room or reconnect to a room, if the Token is expired or invalid, the user will fail to join the room receive a [RTCRoomEventHandler.onRoomStateChanged]. In this case, you need to retrieve the Token and call this method to update the Token.
  ///
  /// [token]:The updated Token.
  Future<void> updateToken(String token);

  /// Sets your expected configuration of the remote video stream that you want to subscribe to or have subscribed to.
  ///
  /// [uid]: ID of the publisher of the target stream.
  ///
  /// [remoteVideoConfig]: Your expected configuration.
  ///
  /// Notes:
  /// + This API only works after the publisher calls [RTCVideo.enableSimulcastMode] to enable publishing multiple video streams, in which case the subscriber will receive the stream from the publisher that is closest to the set configuration;  otherwise the subscriber will only receive one video stream with a resolution of 640px × 360px and a frame rate of 15fps.
  /// + If you don't call this API after the publisher enables the function of publishing multiple streams, you will receive by default the video stream with the largest resolution set by the publisher.
  /// + You should call this API in the room. If you want to call it before entering the room, you should set the `remoteVideoConfig` in the `roomConfig ` when calling [RTCRoom.joinRoom].
  /// + SDK will automatically select the stream to be published or subcribed based on the settings of both sides. See [Simulcasting](https://docs.byteplus.com/byteplus-rtc/docs/70139) for details.
  Future<void> setRemoteVideoConfig(
      {required String uid, required RemoteVideoConfig videoConfig});

  /// Publishes media streams captured by camera/microphone in the current room.
  ///
  /// Notes:
  /// + You don't need to call this API if you have selected Auto-publish when calling [RTCRoom.joinRoom].
  /// + An invisible user cannot publish media streams. Call [RTCRoom.setUserVisibility] to change your visibility.
  /// + Call [RTCRoom.publishScreen] to start screen sharing.
  /// + Call [RTCRoom.startForwardStreamToRooms] to relay the published streams to the other rooms.
  /// + After you call this API, the other users in the room will receive [RTCRoomEventHandler.onUserPublishStream]. Those who successfully received your streams will receive [RTCVideoEventHandler.onFirstRemoteAudioFrame]/[RTCVideoEventHandler.onFirstRemoteVideoFrameDecoded].
  /// + Call [RTCRoom.unpublishStream] to stop publishing streams.
  Future<void> publishStream(MediaStreamType type);

  /// Stops publishing media streams captured by camera/microphone in the current room.
  ///
  /// [type]: Media stream type.
  ///
  /// Notes:
  /// + After calling [RTCRoom.publishStream], call this API to stop publishing streams。
  /// + After calling this API, the other users in the room will receive [RTCRoomEventHandler.onUserUnpublishStream].
  Future<void> unpublishStream(MediaStreamType type);

  /// Publishes local screen sharing streams in the current room.
  ///
  /// [type]: Media stream type.
  ///
  /// Notes:
  /// + You don't need to call this API if you have selected Auto-publish when calling [RTCRoom.joinRoom].
  /// + An invisible user cannot publish screen sharing streams. Call [RTCRoom.setUserVisibility] to change your visibility.
  /// + After you called this API, the other users in the room will receive  [RTCRoomEventHandler.onUserPublishScreen]. Those who successfully received your streams will receive [RTCVideoEventHandler.onFirstRemoteAudioFrame]/[RTCVideoEventHandler.onFirstRemoteVideoFrameDecoded].
  /// + Call [RTCRoom.startForwardStreamToRooms] to relay the published streams to the other rooms.
  /// + Call [RTCRoom.unpublishScreen] to stop publishing screen sharing streams.
  Future<void> publishScreen(MediaStreamType type);

  /// Stops publishing local screen sharing streams in the current room.
  ///
  /// [type]: Media stream type.
  ///
  /// Notes:
  /// + After calling [RTCRoom.publishScreen], call this API to stop publishing streams.
  /// + After calling this API, the other users in the room will receive [RTCRoomEventHandler.onUserUnpublishScreen].
  Future<void> unpublishScreen(MediaStreamType type);

  /// Subscribes to specific remote media streams captured by camera/microphone, or updates subscription options.
  ///
  /// [uid]: ID of the remote user who published the target audio/video stream.
  ///
  /// [type]: Media stream type.
  ///
  /// Notes:
  /// + This method can be used to subscribe to a remote user for the first time, or to update the media stream type of an already subscribed remote user.
  /// + You must first get the remote stream information through [RTCRoomEventHandler.onUserPublishStream] before calling this API to subscribe to streams as needed.
  /// + Once the local user subscribes to the remote stream, the subscription will sustain until the local user leaves the room or unsubscribe from it by calling [RTCRoom.unsubscribeStream].
  /// + Exceptions during the call are notified through [RTCRoomEventHandler.onStreamStateChanged], see [ErrorCode] for specific reasons.
  Future<void> subscribeStream(
      {required String uid, required MediaStreamType type});

  /// Unsubscribes from specific remote media streams captured by camera/microphone.<br>
  /// You can call this API in both automatic subscription mode and manual subscription mode.
  ///
  /// [uid]: ID of the remote user of which you have subscribed to the stream.
  ///
  /// [type]: Media stream type.
  ///
  /// Exceptions during the call are notified through [RTCRoomEventHandler.onStreamStateChanged], see [ErrorCode] for specific reasons.
  Future<void> unsubscribeStream(
      {required String uid, required MediaStreamType type});

  /// Subscribes to specific screen sharing media stream, or update subscription options.
  ///
  /// [uid]: ID of the remote user who published the target screen audio/video stream.
  ///
  /// [type]: Media stream type.
  ///
  /// Notes:
  /// + This method can be used to subscribe to a remote user for the first time, or to update the media stream type of an already subscribed remote user.
  /// + You must first get the remote stream information through [RTCRoomEventHandler.onUserPublishScreen] before calling this API to subscribe to streams as needed.
  /// + Once the local user subscribes to the remote stream, the subscription will sustain until the local user leaves the room or unsubscribe from it by calling [RTCRoom.unsubscribeScreen].
  /// +  Exceptions during the call are notified through [RTCRoomEventHandler.onStreamStateChanged], see [ErrorCode] for specific reasons.
  Future<void> subscribeScreen(
      {required String uid, required MediaStreamType type});

  /// Unsubscribes from specific screen sharing media stream.<br>
  /// You can call this API in both automatic subscription mode and manual subscription mode.
  ///
  /// [uid]: ID of the remote user of which you have subscribed to the stream.
  ///
  /// [type]: Media stream type.
  ///
  /// Exceptions during the call are notified through [RTCRoomEventHandler.onStreamStateChanged], see [ErrorCode] for specific reasons.
  Future<void> unsubscribeScreen(
      {required String uid, required MediaStreamType type});

  /// Pauses receiving remote media streams.
  ///
  /// Notes:
  /// + Calling this API does not change the capture and transmission state of the remote stream.
  /// + Calling this API does not cancel the subscription or change any subscription configuration.
  /// + To resume receiving streams, call [RTCRoom.resumeAllSubscribedStream].
  /// + In a multi-room scenario, this API only pauses the reception of streams published in the current room.
  Future<void> pauseAllSubscribedStream(PauseResumeControlMediaType mediaType);

  /// Resumes receiving remote media streams
  ///
  /// Notes:
  /// + Calling this API does not change the capture and transmission state of the remote stream.
  /// + Calling this API does not change any subscription configuration.
  Future<void> resumeAllSubscribedStream(PauseResumeControlMediaType mediaType);

  /// Sends a text message (P2P) to the specified user in the room.
  ///
  /// The [message] length must not exceed 64 KB.
  ///
  /// [config]: Reliability of the message.
  ///
  /// Notes:
  /// + Before sending an in-room text message, you must call [RTCRoom.joinRoom] to join the target room.
  /// + After calling this API, the local user will receive [RTCRoomEventHandler.onUserMessageSendResult].
  /// + If the text message is sent successfully, the user specified by [uid] will receive [RTCRoomEventHandler.onUserMessageReceived].
  Future<int?> sendUserMessage(
      {required String uid,
      required String message,
      required MessageConfig config});

  /// Sends a binary message (P2P) to the specified user in the room.
  ///
  /// The [message] length must not exceed 46 KB.
  ///
  /// [config]: Reliability of the message.
  ///
  /// Notes:
  /// + Before sending an in-room binary message, you must call [RTCRoom.joinRoom] to join the target room.
  /// + After calling this API, the local user will receive [RTCRoomEventHandler.onUserMessageSendResult].
  /// + If the binary message is sent successfully, the user specified by [uid] will receive [RTCRoomEventHandler.onUserBinaryMessageReceived].
  Future<int?> sendUserBinaryMessage(
      {required String uid,
      required Uint8List message,
      required MessageConfig config});

  /// Sends text messages to all other users in the room.
  ///
  /// The [message] length must not exceed 64 KB.
  ///
  /// Notes:
  /// + Before sending an in-room broadcasting text message, you must call [RTCRoom.joinRoom] to join the target room.
  /// + After calling this API, the local user will receive [RTCRoomEventHandler.onRoomMessageSendResult].
  /// + If the text message is sent successfully, all the other users in the room will receive [RTCRoomEventHandler.onRoomMessageReceived].
  Future<int?> sendRoomMessage(String message);

  /// Sends binary messages to all other users in the room.
  ///
  /// The [message] length must not exceed 46 KB.
  ///
  /// Notes:
  /// + Before sending an in-room broadcasting binary message, you must call [RTCRoom.joinRoom] to join the target room.
  /// + After calling this API, the local user will receive [RTCRoomEventHandler.onRoomMessageSendResult].
  /// + If the binary message is sent successfully, all the other users in the room will receive [RTCRoomEventHandler.onRoomBinaryMessageReceived].
  Future<int?> sendRoomBinaryMessage(Uint8List message);

  /// Starts relaying media stream across rooms.
  ///
  /// Call this API after the user enters the room to achieve the relaying of media streams to multiple rooms, which applies to cross-room concatenation and other scenarios.
  ///
  /// [forwardStreamInfos]: Information of the rooms where you want to relay the media stream to.
  ///
  /// Notes:
  /// + Calling this method will trigger [RTCRoomEventHandler.onForwardStreamStateChanged] locally.
  /// + After calling this method, listen the events from each room during the relaying by registering [RTCRoomEventHandler.onForwardStreamEvent].
  /// + Once the relaying begins, the other users in the target room will receive [RTCRoomEventHandler.onUserJoined] and [RTCRoomEventHandler.onUserPublishStream]/[RTCRoomEventHandler.onUserPublishScreen].
  /// + Call [RTCRoom.updateForwardStreamToRooms] o add or remove target room(s) after calling this API.
  /// + Call [RTCRoom.stopForwardStreamToRooms] to stop relaying to all rooms after calling this API.
  /// + Call [RTCRoom.pauseForwardStreamToAllRooms] to pause relaying to all rooms after calling this API.
  Future<int?> startForwardStreamToRooms(
      List<ForwardStreamInfo> forwardStreamInfos);

  /// Updates information of the rooms where you want to relay the media stream to after calling [RTCRoom.startForwardStreamToRooms].
  ///
  /// Adding and removing rooms by calling this method will trigger [RTCRoomEventHandler.onForwarStreamStateChanged] locally.
  ///
  /// [forwardStreamInfos]: Information of the rooms where you want to relay the media stream to.
  ///
  /// Notes:
  /// + Users in the room which is added by calling this method will receive [RTCRoomEventHandler.onUserJoined] and [RTCRoomEventHandler.onUserPublishStream]/[RTCRoomEventHandler.onUserPublishScreen].
  /// + Users in the room which is removed by calling this method will receive [RTCRoomEventHandler.onUserUnpublishStream]/[RTCRoomEventHandler.onUserUnpublishScreen] and [RTCRoomEventHandler.onUserLeave].
  Future<int?> updateForwardStreamToRooms(
      List<ForwardStreamInfo> forwardStreamInfos);

  /// Stops relaying media streams to all rooms after calling [RTCRoom.startForwardStreamToRooms]
  ///
  /// Notes:
  /// + Calling this method will trigger [RTCRoomEventHandler.onForwardStreamStateChanged] locally.
  /// + The other users in the room will receive [RTCRoomEventHandler.onUserUnpublishStream]/[RTCRoomEventHandler.onUserUnpublishScreen] and [RTCRoomEventHandler.onUserLeave].
  /// + If you need to change the target room, call [RTCRoom.updateForwardStreamToRooms] to update room information.
  /// + To pause the relaying for a short time, call [RTCRoom.pauseForwardStreamToAllRooms], then call [RTCRoom.resumeForwardStreamToAllRooms] to resume.
  Future<void> stopForwardStreamToRooms();

  /// Pauses relaying media stream to all rooms after calling [RTCRoom.startForwardStreamToRooms].
  ///
  /// Notes:
  /// + You can call [RTCRoom.resumeForwardStreamToAllRooms] at any time to quickly resume relaying.
  /// + After calling this API, users in all rooms will receive [RTCRoomEventHandler.onUserUnpublishStream]/[RTCRoomEventHandler.onUserUnpublishScreen] and [RTCRoomEventHandler.onUserLeave].
  Future<void> pauseForwardStreamToAllRooms();

  /// Resumes relaying media streams paused by calling [RTCRoom.pauseForwardStreamToAllRooms] to all rooms.
  ///
  /// Users in all rooms will receive [RTCRoomEventHandler.onUserJoined] and [RTCRoomEventHandler.onUserPublishStream]/[RTCRoomEventHandler.onUserPublishScreen].
  Future<void> resumeForwardStreamToAllRooms();

  /// Gets range audio instance.
  ///
  /// API call result:
  /// + RTCRangeAudio: Success. You will get an [RTCRangeAudio] returned from the SDK.
  /// + null: Failure. The current SDK does not provide range audio function.
  ///
  /// The first time this API is called must be between creating a room and actually entering the room.
  RTCRangeAudio get rangeAudio;

  /// Gets spatial audio instance.
  ///
  /// See [Spatial Audio](https://docs.byteplus.com/byteplus-rtc/docs/93903) for spatial audio-related APIs and call order.
  ///
  /// API call result:
  /// + RTCSpatialAudio: Success. You will get an [RTCSpatialAudio] returned from the SDK.
  /// + null: Failure. The current SDK does not provide spatial audio function.
  ///
  /// Notes:
  /// + Only using real dual-channel playback device can users actually enjoy spatial audio effect.
  /// + In case of poor network conditions, there is no spatial audio effect even if this function is enabled.
  /// + Low-end device may have audio stalling issues due to its poor performance, so it is not recommended to enable spatial audio function on such kind of devices.
  /// + No spatial audio effect in audio routing of Conference mode.
  RTCSpatialAudio get spatialAudio;
}
