// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'package:flutter/foundation.dart';

import '../src/bytertc_room_event_impl.dart';
import 'bytertc_event_define.dart';

/// Room event callbacks.
class RTCRoomEventHandler extends RTCRoomEventValue {
  /// Callback about room state changes.
  ///
  /// The user will receive this callback when they join the room, leave the room, or receive room relating warnings and errors.
  OnRoomStateChangedType? onRoomStateChanged;

  /// Callback about stream state changes.
  ///
  /// The user will receive this callback when they receive stream relating warnings and errors.
  OnRoomStateChangedType? onStreamStateChanged;

  /// Callback about successfully leaving the room.
  ///
  /// After the user calls [RTCRoom.leaveRoom], the SDK will stop publishing and subscribing to all streams and will notify the user of the success of leaving the room through this callback after releasing all audio and video resources associated with the call.
  ///
  /// Notes:
  /// + After leaving the room, if the App needs to use the system audio/video device, it is recommended to initialize the audio/video device after receiving this callback, otherwise the initialization may fail due to the SDK occupying the audio/video device.
  /// + If the [RTCEngine.destroyEngine] is called immediately after the user leaves the room by calling the [RTCRoom.leaveRoom] method, this callback event will not be received if the RTC engine is destroyed.
  OnLeaveRoomType? onLeaveRoom;

  /// Stream publisher will receive this callback when the A/V synchronization state changes after [RTCRoom.setMultiDeviceAVSync] is called.
  OnAVSyncStateChangeType? onAVSyncStateChange;

  /// In-room call statistics callback.
  ///
  /// After the user enters the room and starts the call, they will receive this callback every 2s.
  OnRoomStatsType? get onRoomStats;
  set onRoomStats(OnRoomStatsType? onRoomStats);

  /// Callback about the statistics of audio/video streams sent by the local device during the call and the network status, which is triggered every 2s.
  OnLocalStreamStatsType? get onLocalStreamStats;
  set onLocalStreamStats(OnLocalStreamStatsType? onLocalStreamStats);

  /// Callback about the statistics of the local device receiving the subscribed remote audio/video streams during the call and the network status, which is triggered every 2s.
  OnRemoteStreamStatsType? get onRemoteStreamStats;
  set onRemoteStreamStats(OnRemoteStreamStatsType? onRemoteStreamStats);

  /// Callback about remote users joining the room.
  ///
  /// This callback is triggered in the following scenarios:
  /// + When the remote visible user calls [RTCRoom.joinRoom] to join the room, other users in the room will receive the callback.
  /// + when the remote visible user reconnects to the room after disconnecting from the network, other users in the room will receive the callback.
  /// + When the remote invisible user calls [RTCRoom.setUserVisibility] to switch themselves to visible, other users in the room will receive the callback.
  /// + New commers will receive this callback of exsiting visible users entering the room.
  OnUserJoinedType? onUserJoined;

  /// Callback about remote users leaving the room.
  ///
  /// Other users in the room will receive the callback in the following scenarios:
  /// + When the remote visible user calls [RTCRoom.leaveRoom] to leave the room.
  /// + When the remote visible user calls [RTCRoom.setUserVisibility] to switch themselves to invisible
  /// + When the remote user is disconnected and never restored.
  OnUserLeaveType? onUserLeave;

  /// The callback will be triggered 30 seconds before the Token expires.
  ///
  /// In this case, you should call [RTCRoom.updateToken] to update the Token, otherwise the user would be removed from the room and not be able to continue the call.
  EmptyCallbackType? onTokenWillExpire;

  /// Callback about new media streams captured by camera/microphone in the room.
  ///
  /// The local user will receive this callback after a remote user successfully published media streams captured by camera/microphone in the room with [RTCRoom.publishStream].<br>
  /// Then the local user can choose whether or not to call [RTCRoom.subscribeStream] to subscribe to this stream.
  OnUserPublishStreamType? onUserPublishStream;

  /// Callback about the removal of remote media stream captured by camera/microphone.
  ///
  /// After receiving this callback, the local user can choose whether to call [RTCRoom.unsubscribeStream] to unsubscribe from the streams or not.
  OnUserUnpublishStreamType? onUserUnpublishStream;

  /// Callback about new screen sharing media streams from remote users in the room.
  ///
  /// The local user will receive this callback after a remote user successfully publishes screen sharing streams in the room with [RTCRoom.publishScreen].
  /// Then the local user can choose whether to call [RTCRoom.subscribeScreen] to subscribe to the streams or not.
  OnUserPublishStreamType? onUserPublishScreen;

  /// Callback about the removal of screen sharing media streams from remote users in the room.
  ///
  /// After receiving this callback, the local can choose whether to call [RTCRoom.unsubscribeScreen] to unsubscribe from the streams or not.
  OnUserUnpublishStreamType? onUserUnpublishScreen;

  /// @nodoc
  @protected
  OnStreamSubscribedType? onStreamSubscribed;

  /// This callback is triggered after the local user receives a in-room broadcast text message.
  OnMessageReceivedType? onRoomMessageReceived;

  /// This callback is triggered after the local user receives a in-room broadcast binary message.
  OnBinaryMessageReceivedType? onRoomBinaryMessageReceived;

  /// This callback is triggered after the local user receives a text message from the other users in the room.
  OnMessageReceivedType? onUserMessageReceived;

  /// This callback is triggered after the local user receives a binary message from the other users in the room.
  OnBinaryMessageReceivedType? onUserBinaryMessageReceived;

  /// This callback is triggered after the local user sends a message to another user in the room.
  OnMessageSendResultType? onUserMessageSendResult;

  /// This callback is triggered after the local user sends a message to all the other users in the room.
  OnRoomMessageSendResultType? onRoomMessageSendResult;

  /// This callback is triggered by calling the OpenAPI BanUserStream/UnbanUserStream to disable/enable the specified user publishing video streams in the specified room.
  ///
  /// Notes:
  /// + When the specified user in the room is banned/unbanned from publishing video streams, all users in the room will receive the callback.
  /// + The banned/unbanned status will not change until the room is destroyed.
  /// + Only users being banned/unbanned will receive this callback after the conference mode is turned on in the console.
  OnVideoStreamBannedType? onVideoStreamBanned;

  /// This callback is triggered by calling the OpenAPI BanUserStream/UnbanUserStream to disable/enable the specified user publishing audio streams in the specified room.
  ///
  /// Notes:
  /// + When the specified user in the room is banned/unbanned from publishing audio streams, all users in the room will receive the callback.
  /// + The banned/unbanned status will not change until the room is destroyed.
  /// + Only users being banned/unbanned will receive this callback after the conference mode is turned on in the console.
  OnAudioStreamBannedType? onAudioStreamBanned;

  /// Callback about the state and errors during relaying media streams across rooms.
  OnForwardStreamStateChangedType? onForwardStreamStateChanged;

  /// Callback about events during relaying media streams across rooms.
  OnForwardStreamEventType? onForwardStreamEvent;

  /// After the user joins the room, this callback will report the user's network quality information every 2 seconds.
  ///
  /// See [In-call Stats](https://docs.byteplus.com/byteplus-rtc/docs/106866) for more details.
  OnNetworkQualityType? get onNetworkQuality;
  set onNetworkQuality(OnNetworkQualityType? onNetworkQuality);

  RTCRoomEventHandler({
    this.onRoomStateChanged,
    this.onStreamStateChanged,
    this.onLeaveRoom,
    this.onAVSyncStateChange,
    OnRoomStatsType? onRoomStats,
    OnLocalStreamStatsType? onLocalStreamStats,
    OnRemoteStreamStatsType? onRemoteStreamStats,
    this.onUserJoined,
    this.onUserLeave,
    this.onTokenWillExpire,
    this.onUserPublishStream,
    this.onUserUnpublishStream,
    this.onUserPublishScreen,
    this.onUserUnpublishScreen,
    this.onStreamSubscribed,
    this.onRoomMessageReceived,
    this.onRoomBinaryMessageReceived,
    this.onUserMessageReceived,
    this.onUserBinaryMessageReceived,
    this.onUserMessageSendResult,
    this.onRoomMessageSendResult,
    this.onVideoStreamBanned,
    this.onAudioStreamBanned,
    this.onForwardStreamStateChanged,
    this.onForwardStreamEvent,
    OnNetworkQualityType? onNetworkQuality,
  }) {
    this.onRoomStats = onRoomStats;
    this.onLocalStreamStats = onLocalStreamStats;
    this.onRemoteStreamStats = onRemoteStreamStats;
    this.onNetworkQuality = onNetworkQuality;
  }
}
