// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:typed_data';
import 'dart:ui';

import '../src/bytertc_video_impl.dart';
import 'bytertc_asr_engine_event_handler.dart';
import 'bytertc_audio_defines.dart';
import 'bytertc_audio_mixing_api.dart';
import 'bytertc_common_defines.dart';
import 'bytertc_face_detection_observer.dart';
import 'bytertc_live_transcoding_observer.dart';
import 'bytertc_push_single_stream_to_cdn_observer.dart';
import 'bytertc_room_api.dart';
import 'bytertc_video_defines.dart';
import 'bytertc_video_event_handler.dart';

/// The engine instance.
abstract class RTCVideo {
  /// Creates an engine instance.
  ///
  /// This is the very first API that you must call if you want to use all the RTC capabilities. <br>
  /// If there is no engine instance in current process, calling this API will create one. If an engine instance has been created, calling this API again will have the created engine instance returned.
  static Future<RTCVideo?> createRTCVideo(RTCVideoContext context) {
    return RTCVideoImpl.createRTCVideo(context);
  }

  /// Gets the current SDK version.
  static Future<String?> getSdkVersion() {
    return RTCVideoImpl.getSdkVersion();
  }

  /// Gets the description of error codes and warning codes in the SDK.
  static Future<String?> getErrorDescription(int code) {
    return RTCVideoImpl.getErrorDescription(code);
  }

  /// Destroys the engine instance created by [RTCVideo.createRTCVideo] and release all related resources.
  ///
  /// Notes:
  /// + Call this API after all business scenarios related to the RTCVideo instance that need to be destroyed have been completed.
  /// + Calling this API will destroy all memory associated with this RTCVideo instance and stops any interaction with the media server.
  Future<void> destroy();

  /// Sets the event callback class.
  void setRTCVideoEventHandler(RTCVideoEventHandler handler);

  /// Starts internal audio capturing. Off by default.
  ///
  /// Internal capture refers to audio capture using the built-in capture module of the SDK.<br>
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onAudioDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStartAudioCapture].
  ///
  /// Notes:
  /// + Calling this API without obtaining permission to use the microphone of the current device will trigger [RTCVideoEventHandler.onWarning].
  /// + Call [RTCVideo.stopAudioCapture] to stop the internal audio capturing, otherwise it will not stop until the engine is destroyed.
  /// + Since different hardware devices have different initialization response times, frequent calls to this method and [RTCVideo.stopAudioCapture] may cause short silence during the call. Therefore to temporarily mute or unmute microphone, we recommend you use [RTCRoom.publishStream]/[RTCRoom.unpublishStream].
  /// + You can call this API no matter you need to publish audio or not, but you can only publish audio after calling it.
  Future<void> startAudioCapture();

  /// Starts internal audio capturing.
  ///
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onAudioDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStopAudioCapture].
  ///
  /// Notes:
  /// + Call [RTCVideo.startAudioCapture] to start internal calturing.
  /// + If you don't call this API after the internal capture is started, it will not stop until the engine is destroyed.
  Future<void> stopAudioCapture();

  /// Sets the audio scenarios.
  ///
  /// You should choose the right type of audio scenario according to your need.<br>
  /// When an audio scenario is selected, the RTC automatically applies call volume/media volume based on the client audio routing and publishing subscription status.<br>
  ///
  /// Notes:
  /// + You can call this API anytime before or after entering the room.
  /// + Call volume is more suitable for calls, meetings and other scenarios that demand information accuracy. Call volume will activate the system hardware signal processor, making the sound clearer. The volume cannot be reduced to 0.
  /// + Media volume is more suitable for entertainment scenarios, which require musical expression. The volume can be reduced to 0.
  Future<void> setAudioScenario(AudioScenario audioScenario);

  /// Sets the sound quality.
  ///
  /// You can call this API when the audio parameters in the selected [ChannelProfile] do not meet the needs of your scene.
  ///
  /// Notes: You can call this API anytime before or after entering the room.
  Future<void> setAudioProfile(AudioProfileType audioProfile);

  /// Sets the type of voice change effect.
  ///
  /// + You can call this API anytime before or after entering the room.
  /// + This method only works for mono audio.
  /// + The version of the RTC SDK you are using must provide sound effect function
  /// + This method is mutually exclusive with [RTCVideo.setVoiceReverbType], the effects set later will override the effects set first.
  Future<void> setVoiceChangerType(VoiceChangerType voiceChanger);

  /// Sets the type of reverb effect.
  ///
  /// + You can call this API anytime before or after entering the room.
  /// + This method only works for mono audio.
  /// + The version of the RTC SDK you are using must provide sound effect function
  /// + This method is mutually exclusive with [RTCVideo.setVoiceChangerType], the effects set later will override the effects set first.
  Future<void> setVoiceReverbType(VoiceReverbType voiceReverb);

  /// Adjusts the audio capture volume.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of [0, 400].<br>
  /// For better call quality, it is recommended to set the volume value to [0,100].
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  ///
  /// Notes:
  /// + You can call this method to adjust both the audio stream captured by the microphone, and the audio stream from the screen sharing.
  /// + You can call this method both before and after starting audio capturing.
  Future<void> setCaptureVolume(
      {StreamIndex index = StreamIndex.main, required int volume});

  /// Adjusts the playback volume of the mixed remote audio.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of [0, 400].<br>
  /// For better call quality, it is recommended to set the volume value to [0,100].
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  ///
  /// You can call this API both before and during playing the audio.
  Future<void> setPlaybackVolume(int volume);

  /// Enables audio information prompts.
  ///
  /// With the prompt on, you can:
  /// + Get the information of the audio stream captured by the local microphone and screen audio stream through [RTCVideoEventHandler.onLocalAudioPropertiesReport].
  /// + Get the information of the subscribed remote audio streams through [RTCVideoEventHandler.onRemoteAudioPropertiesReport].
  /// + Get the information of the active speaker through [RTCVideoEventHandler.onActiveSpeaker].
  Future<void> enableAudioPropertiesReport(AudioPropertiesConfig config);

  /// Adjusts the audio playback volume from the remote user.
  ///
  /// [roomId]: ID of the room where the specified remote user is in.
  ///
  /// [uid]: ID of the user who publishes the audio.
  ///
  /// [volume]: Ratio(%) of capture volume to original volume with a range of [0, 400].<br>
  /// For better call quality, it is recommended to set the volume value to [0,100].
  /// + 0: Mute.
  /// + 100: Original volume.
  /// + 400: Four times the original volume with signal-clipping protection.
  Future<void> setRemoteAudioPlaybackVolume(
      {required String roomId, required String uid, required int volume});

  /// Turns on/off the earphone monitor function.
  ///
  /// Notes:
  /// + The user must wear headphones to use the in-ear monitoring function. To ensure the best experience of low-latency in-ear monitoring, it is recommended to wear wired headphones.
  /// + RTC SDK supports both hardware and software in-ear monitoring. Generally speaking, the hardware in-ear monitoring has low latency and good sound quality.
  /// + Whether or not your app can use hardware in-ear monitoring depends on the phone manufacturer. Bluetooth headsets do not support hardware in-ear monitoring. iOS platforms only supports software in-ear monitoring.
  Future<void> setEarMonitorMode(EarMonitorMode mode);

  /// Sets the volume of in-ear monitoring.
  ///
  /// [volume]: Ratio(%) of in-ear monitoring volume to original volume with a range of [0, 100].<br>
  ///
  /// Notes: Call [RTCVideo.setEarMonitorMode] to enable in-ear monitoring function before calling this API.
  Future<void> setEarMonitorVolume(int volume);

  /// Changes local voice to a different key, mostly used in Karaoke scenarios.
  ///
  /// You can adjust the pitch of local voice such as ascending or descending with this method.
  ///
  /// [pitch] The value that is higher or lower than the original local voice within a range of [-12, 12]. The default value is 0, i.e. no adjustment is made.  <br>
  /// The difference in pitch between two adjacent values within the value range is a semitone, with positive values indicating an ascending tone and negative values indicating a descending tone, and the larger the absolute value set, the more the pitch is raised or lowered.  <br>
  /// Out of the value range, the setting fails and triggers [RTCVideoEventHandler.onWarning].
  Future<void> setLocalVoicePitch(int pitch);

  /// Enables/disables the loudness equalization function.
  ///
  /// If you call this API with the parameter set to True, the loudness of user's voice will be adjusted to -16lufs.<br>
  /// If then you also call [RTCAudioMixingManager.setAudioMixingLoudness] and import the original loudness of the audio data used in audio mixing, the loudness will be adjusted to -20lufs when the audio data starts to play.
  ///
  /// Notes: You must call this API before calling [RTCAudioMixingManager.startAudioMixing] to start playing the audio.
  Future<void> enableVocalInstrumentBalance(bool enable);

  /// Enables/disables the playback ducking function. This function is usually used in scenarios such as "Watch Videos Together".
  ///
  ///  With the function on, if remote voice is detected, the local media volume will be lowered to ensure the clarity of the remote voice. If remote voice disappears, the local media volume restores.
  Future<void> enablePlaybackDucking(bool enable);

  /// Enables/Disables the mode of publishing multiple video streams with different encoding configuration. Off by default.
  ///
  /// Notes:
  /// + You should call this method before entering the room or after entering the room but before publishing the stream.
  /// + After setting this API to "True", you can call [RTCVideo.setVideoEncoderConfig] to set encoding configuration for each stream.
  /// + If this function is off, or if this function is on but you don't set the configuration of any stream, only one stream will be sent with a resolution of 640px × 360px and a frame rate of 15fps.
  Future<int?> enableSimulcastMode(bool enable);

  /// Video publisher calls this API to set the parameters of the maximum resolution video stream that is expected to be published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// [maxSolution]: The maximum video encoding parameter.
  ///
  /// Notes:
  /// + If you call this API after enabling the mode of publishing multiple streams with [RTCVideo.enableSimulcastMode], SDK will automatically adjust the number of streams published and the parameters of each published stream according to the settings of subscribers. Up to 4 streams will be published, and the resolution you set in this API will be considered as the largest resolution among these 4 streams.
  /// + Without calling this API, SDK will only publish one stream for you with a resolution of 640px × 360px and a frame rate of 15fps.
  /// + This API applies to the video stream captured by the camera, see [RTCVideo.setScreenVideoEncoderConfig] for setting parameters for screen sharing video stream.
  /// + See [Simulcasting](https://docs.byteplus.com/byteplus-rtc/docs/70139) for more details.
  Future<int?> setMaxVideoEncoderConfig(VideoEncoderConfig maxSolution);

  /// Video publisher calls this API to set the configurations of each stream published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// [channelSolutions]: List of configurations for multiple streams. You can set parameters for up to 3 streams, SDK will always take the value of the first 3 streams when parameters for more streams are set.<br>
  /// The resolution you set is the maximum resolution of each stream, and must be arranged in descending order. The frame rates must be arranged in descending order.
  ///
  /// Notes:
  /// + If you call this API after enabling the mode of publishing multiple streams with [RTCVideo.enableSimulcastMode], SDK will publish streams as you set, otherwise only the stream with the largest resolution you set will be published.
  /// + Without calling this API to set parameters for multiple video streams, the SDK only publishes one video stream with a resolution of 640px × 360px and a frame rate of 15fps.
  /// + After setting parameters for multiple video streams, SDK will automatically match the streams to be published based on the desired subscription parameters set by subscribers.
  /// + This API applies the video stream captured by the camera, see [RTCVideo.setScreenVideoEncoderConfig] for setting parameters for screen sharing video stream.
  /// + See [Simulcasting](https://docs.byteplus.com/byteplus-rtc/docs/70139) for more details.
  Future<int?> setVideoEncoderConfig(List<VideoEncoderConfig> channelSolutions);

  /// Video publisher calls this API to set the expected configurations for the screen sharing video stream, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
  ///
  /// Without calling this API, the default encoding parameters for screen sharing video streams are: resolution 1920px × 1080px, frame rate 15fps.
  Future<int?> setScreenVideoEncoderConfig(VideoEncoderConfig screenSolution);

  /// Sets the video capture parameters for internal capturing of the RTC SDK.
  ///
  /// Notes:
  /// + This API can be called after the engine is created and takes effect immediately after the call. It is recommended to call this API before calling [RTCVideo.startVideoCapture].
  /// + It is recommended to use the same video capture parameters for different engines on the same device.
  /// + Internal capturing defaults to Auto mode.
  Future<int?> setVideoCaptureConfig(VideoCaptureConfig config);

  /// Remove the local video canvas.
  Future<int?> removeLocalVideo({StreamIndex streamType = StreamIndex.main});

  /// Remove the remote video canvas.
  Future<int?> removeRemoteVideo(
      {required String roomId,
      required String uid,
      StreamIndex streamType = StreamIndex.main});

  /// Enables internal video capturing immediately. Off by default.
  ///
  /// Internal capture refers to video capture using the built-in capture module of the SDK.<br>
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStartVideoCapture].
  ///
  /// Notes:
  /// + Calling this API without obtaining permission to use the camera of the current device will trigger [RTCVideoEventHandler.onVideoDeviceStateChanged].
  /// + Call [RTCVideo.stopVideoCapture] to stop the internal video capturing, otherwise it will not stop until the engine is destroyed.
  /// + You can call this API no matter you need to publish audio or not, but you can only publish audio after calling it.
  /// + Android needs to introduce Kotlin in Gradle. iOS needs to request camera permission from the user in the app before it can start capturing.
  Future<void> startVideoCapture();

  /// Stops internal video capturing immediately.
  ///
  /// After this API is called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged].<br>
  /// If this API is called by a visible user in the current room, the other users in the room will receive [RTCVideoEventHandler.onUserStopVideoCapture].
  ///
  /// Notes:
  /// + Call [RTCVideo.startVideoCapture] to start internal video capturing.
  /// + If you don't call this API after the internal capture is started, it will not stop until the engine is destroyed.
  Future<void> stopVideoCapture();

  /// Sets the mirror mode for the captured video stream.
  ///
  /// Notes:
  /// + Switching video streams does not affect the settings of the mirror type.
  /// + This API is not applicable to screen-sharing streams.
  /// + Before you call this API, the initial states of each video stream are as follows:
  /// <table border>
  /// <tr><th></th><th>Front-facing camer</th><th>Rear-facing camera</th><th>Built-in camera</th>
  /// <tr><td>Mobile device</td><td>The preview is mirrored. The published video stream is not mirrored.</td><td>The preview and the published video stream are not mirrored.</td><td>/</td>
  /// <tr><td>PC</td><td>/</td><td>/</td><td>The preview is mirrored. The published video stream is not mirrored.</td>
  /// </table>
  Future<int?> setLocalVideoMirrorType(MirrorType mirrorType);

  /// Sets the orientation of the video capture. By default, the App direction is used as the orientation reference.
  ///
  /// During rendering, the receiving client rotates the video in the same way as the sending client does.
  ///
  /// [rotationMode]: Rotation reference can be the orientation of the App or gravity.
  ///
  /// Notes:
  /// + The orientation setting is not effective to screen-sharing stream.
  /// + If the video capture is on, the setting will be effective once you call this API. If the video capture is off, the setting will be effective on when capture starts.
  /// + See [Video Rotation](https://docs.byteplus.com/byteplus-rtc/docs/106458) for more details.
  Future<int?> setVideoRotationMode(VideoRotationMode rotationMode);

  /// Sets the orientation of the video frame before custom video processing and encoding. The default value is `Adaptive`.
  ///
  /// You should set the orientation to `Portrait` when using video effects or custom processing.<br>
  /// You should set the orientation to `Portrait` or `Landscape` when pushing a single stream to the CDN.<br>
  /// For details of different modes, see [Video Orientation](https://docs.byteplus.com/byteplus-rtc/docs/128787).
  ///
  /// Notes: We recommend you set the orientation before joining room. The updates of encoding configurations and the orientation are asynchronous, therefore can cause a brief malfunction in preview if you change the orientation after joining room.
  Future<void> setVideoOrientation(VideoOrientation orientation);

  /// Switches mobile front/rear camera.
  ///
  /// After the API is called, the local user will receive [RTCVideo.onVideoDeviceStateChanged].
  ///
  /// Notes:
  /// + The front camera is used by default before calling this method.
  /// + If you are using the camera for video capture, the switching operation takes effect immediately. If the camera is not off, the camera you set will be opened when internal capturing is enabled.
  Future<void> switchCamera(CameraId cameraId);

  /// Checks video effect license.
  ///
  /// [licenseFile]: Absolute path of the license file.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  Future<int?> checkVideoEffectLicense(String licenseFile);

  /// Creates/destroys video effects engine.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes:
  /// + Call this API after calling [RTCVideo.checkVideoEffectLicense] and [RTCVideo.setVideoEffectAlgoModelPath].
  /// + After calling this method, you must call [RTCVideo.setVideoEffectNodes] to actually turn on video effects.
  /// + In a common scenario, the special effects engine will be destroyed with the destruction of the RTC engine. When you have high performance requirements, you can call false in this method to destroy the special effects engine separately when the special effects related functions are not used.
  Future<int?> enableVideoEffect(bool enable);

  /// Sets the video effects algorithm model path.
  Future<void> setVideoEffectAlgoModelPath(String modelPath);

  /// Sets video effects resource package.
  ///
  /// [effectNodes]: An array of package paths.<br>
  /// Set this parameter to null to remove the current video effect.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes: Call this API after calling [RTCVideo.enableVideoEffect].
  Future<int?> setVideoEffectNodes(List<String>? effectNodes);

  /// Sets effect intensity.
  ///
  /// [effectNode]: Package path.
  ///
  /// [key]: Name of the resource key to be set, see [Functions of Resource Keys](https://docs.byteplus.com/effects/docs/functions-of-resource-keys-v421-and-later) for details.
  ///
  /// [value]: The intensity value with a range of [0,1]. Setting beyond this range is invalid.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// This API is only applicable to special effects resources that contain the above three parameters at the same time. For special effects that do not have strength parameters, such as most stickers, this interface call is invalid.
  Future<int?> updateVideoEffectNode(
      {required String effectNode, required String key, required double value});

  /// Sets color filter.
  ///
  /// [resFile]: Absolute path of the color filter resource package.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  Future<int?> setVideoEffectColorFilter(String? resFile);

  /// Sets the color filter intensity.
  ///
  /// [intensity]: Filter intensity with a range of [0,1].
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  Future<int?> setVideoEffectColorFilterIntensity(double intensity);

  /// Sets the original background to a specified image or a solid color.
  ///
  ///  To disable this effect, set the modelPath parameter to null.
  ///
  /// [modelPath]: The path of virtual background effects.
  ///
  /// [source]: The local path of background images.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Call the following APIs in sequence before calling this API:
  /// 1. [RTCVideo.checkVideoEffectLicense]
  /// 2. [RTCVideo.setVideoEffectAlgoModelPath]
  /// 3. [RTCVideo.enableVideoEffect]
  Future<int?> setBackgroundSticker(
      {String? modelPath, VirtualBackgroundSource? source});

  /// Enables/Disables basic beauty effects.
  ///
  /// Returned value:
  /// + 0: Success
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This function is not provided in your RTC SDK version.
  /// + 1002: This API is not available for your current Effect SDK. You can upgrade your Effect SDK to V4.3.1+.
  /// + 1003: Contact technical support team for further instructions.
  /// + 1004:  Downloading related resources. The beauty effects will take effect after downloading.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes:
  /// + You cannot use the basic beauty effects and the advanced effect features at the same time. Call [RTCVideo.enableVideoEffect]to use advanced effect features.
  /// + You need to integrate Effect SDK before calling this API. Effect SDK V4.3.1+ is recommended.
  /// + Call [RTCVideo.setBeautyIntensity] to set the beauty effect intensity. If you do not set the intensity, the brightness, smoothness, and sharpness intensity will default to 0.5.
  /// + This API is not applicable to screen capturing.
  Future<int?> enableEffectBeauty(bool enable);

  /// Sets the beauty effect intensity.
  ///
  /// The [intensity] must be within the range of [0,1].<br>
  /// When you set it to 0, the beauty effect will be turned off. The default value is 0.5.
  ///
  /// Returned value:
  /// + 0: Success.
  /// + 1000: The Effect SDK is not integrated.
  /// + 1001: This API is not available in your Effect SDK.
  /// + <0: Other errors. See [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table).
  ///
  /// Notes
  /// + If you call this API before calling [RTCVideo.senableEffectBeauty], the default settings of beauty effect intensity will adjust accordingly.
  /// + After the engine is destroyed, the beauty effect settings will be invalid.
  Future<int?> setBeautyIntensity(
      {required EffectBeautyMode beautyMode, required double intensity});

  /// Registers the observer for the result of face detection.
  ///
  /// After the observer is registered, the local user will receive [RTCFaceDetectionObserver.onFaceDetectResult] periodically.
  ///
  /// The [interval] (in ms) must be greater than 0. The actual time interval of receiving callbacks is between `interval` and the sum of `interval` and the time slot of a captured video frame.
  Future<int?> registerFaceDetectionObserver(
      {RTCFaceDetectionObserver? observer, int interval = 0});

  /// Sets the zoom magnification of the currently used camera (front/rear).
  ///
  /// [zoom]: Camera zoom parameters. 1.0 means scaling to the original image, the maximum value that can be set is obtained through [RTCVideo.getCameraZoomMaxRatio].
  ///
  /// The result of the setting will expire after calling [RTCVideo.stopVideoCapture] to turn off the internal capturing.
  Future<int?> setCameraZoomRatio(double zoom);

  /// Gets the maximum zoom magnification of the currently used camera (front/rear).
  Future<double?> getCameraZoomMaxRatio();

  /// Detects whether the currently used camera supports zoom (digital/optical zoom).
  Future<bool?> isCameraZoomSupported();

  /// Detects whether the currently used camera supports flash.
  Future<bool?> isCameraTorchSupported();

  /// Turns on/off the flash of the currently used camera.
  ///
  /// The result of the setting will expire after calling [RTCVideo.stopVideoCapture] to turn off the internal capturing.
  Future<int?> setCameraTorch(TorchState torchState);

  /// Checks if manual focus is available for the currently used camera.
  Future<bool?> isCameraFocusPositionSupported();

  /// Sets the manual focus position for the currently used camera.
  ///
  /// [position]: The position of the focus point. Setting the upper-left corner of the canvas as the origin, the `x` in `position` means the x-coordinate of the focus point in range of [0, 1], and the `y` in `position` means the y-coordinate of the focus point in range of [0, 1].
  ///
  /// Notes:
  /// + When you set the focus point at the center of the canvas, the focus point setting will be canceled.
  /// + The camera focus point setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraFocusPosition(Offset position);

  /// Checks if manual exposure setting is available for the currently used camera.
  Future<bool?> isCameraExposurePositionSupported();

  /// Sets the manual exposure position for the currently used camera.
  ///
  /// [position]: The position of the exposure point. Setting the upper-left corner of the canvas as the origin, the `x` in `position` means the x-coordinate of the exposure point in range of [0, 1], and the `y` in `position` means the y-coordinate of the exposure point in range of [0, 1].
  ///
  /// Notes:
  /// + When you set the exposure point at the center of the canvas, the exposure point setting will be canceled.
  /// + The camera exposure point setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraExposurePosition(Offset position);

  /// Sets the exposure compensation for the currently used camera.
  ///
  /// [val]: Exposure compensation in range of [-1, 1]. Default to 0, which means no exposure compensation.
  ///
  /// The camera exposure compensation setting will be invalid after calling [RTCVideo.stopVideoCapture] to stop internal capturing.
  Future<int?> setCameraExposureCompensation(double val);

  /// Sends SEI data.
  ///
  /// In a video call scenario, SEI is sent with the video frame, while in a voice call scenario, SDK will automatically publish a black frame with a resolution of 16px × 16px to carry SEI data.
  ///
  /// [streamIndex] specifys the type of media stream that carries SEI data.<br>
  /// In a voice call, you should set this parameter to `main`, otherwise the SEI data is discarded and cannot be sent to the remote user.
  ///
  /// The length of [message] must not exceed 4KB.
  ///
  /// [repeatCount]: Number of times a message is sent repeatedly. The value range is [0,30].<br>
  /// This value specifies how many consecutive video frames, starting with the current video frame, the SEI data will be added to after calling this API.
  ///
  /// Returned value:
  /// + `>=0`: The number of SEIs to be added to the video frame.
  /// + `<0`: Failure.
  ///
  /// Notes:
  /// + In a voice call scenario, this API can be called to send SEI data only in internal capture mode and with a frequency of 15/repeatCount FPS.
  /// + Each video frame carrys only the SEI data received within 2s before and after. In a voice call scenario, if no SEI data is sent within 1min after calling this API, SDK will automatically cancel publishing black frames.
  /// + After the message is sent successfully, the remote user who subscribed the video stream will receive [RTCVideoEventHandler.onSEIMessageReceived].
  /// + When you switch from a voice call to a video call, SEI data transmission will stop and you will need to call this API again to resume the transmission.
  Future<int?> sendSEIMessage(
      {StreamIndex streamIndex = StreamIndex.main,
      required Uint8List message,
      required int repeatCount});

  /// Designate the current audio playback device.
  ///
  /// The default device is set through [RTCVideo.setDefaultAudioRoute].<br>
  /// When the audio playback route changes, the local user will receive [RTCVideoEventHandler.onAudioRouteChanged].
  ///
  /// You can not set [audioRoute] to `Unknown`.<br>
  /// You can not choose `earpiece` in media volume mode, or `headsetBluetooth` and `headsetUSB` in call volume mode.
  ///
  /// Notes:
  /// + The audio route automatically switches to the wired or Bluetooth audio playback device once it is connected.
  /// + Disconnecting the current audio device will have the audio route switched to the previous device. If all the peripheral devices are removed, the audio route will switch to the default route audio.
  /// + For the volume type in different audio scenarios, see [AudioScenarioType].
  Future<int?> setAudioRoute(AudioRoute audioRoute);

  /// Sets the speaker or earpiece as the default audio playback device.
  ///
  /// Returned value:
  /// + 0: Success. The setting takes effect immediately. However, the audio route will not switch to the default device until all the audio peripheral devices are disconnected.
  /// + < 0: Failure. It fails when the device designated is neither a speaker nor an earpiece.
  ///
  /// Notes:
  /// + You can call this API no matter the user is in the room or not.
  /// + See [Set the Audio Route](https://docs.byteplus.com/byteplus-rtc/docs/117836) for more details.
  Future<int?> setDefaultAudioRoute(AudioRoute audioRoute);

  /// Gets the information of currently used playback route.
  Future<AudioRoute> getAudioRoute();

  /// Enables the audio process mode for external sound card.
  ///
  /// Notes:
  /// + When you use external sound card for audio capture, enable this mode for better audio quality.
  /// + When using the mode, you can only use earphones. If you need to use internal or external speaker, disable this mode.
  Future<void> enableExternalSoundCard(bool enable);

  /// Creates a new task of pushing media streams to CDN and sets the relevant configurations.
  ///
  /// When pushing more than one live streams in the same task, SDK will first mix those streams into one single stream and then push it to CDN.
  ///
  /// [taskId]: Task ID.<br>
  /// You may want to push more than one mixed stream to CDN from the same room. When you do that, use different ID for corresponding tasks; if you will start only one task, use an empty string.
  ///
  /// [transcoding]: Configurations to be set when pushing streams to CDN.
  ///
  /// [observer]: Register this observer to receive callbacks from the SDK.
  ///
  /// Notes:
  /// + After this API is called, the local user will receive [RTCLiveTranscodingObserver.onStreamMixingEvent] about the result and errors during the pushing process.
  /// + Call [RTCVideo.stopLiveTranscoding]  to stop pushing streams to CDN.
  Future<void> startLiveTranscoding(
      {required String taskId,
      required LiveTranscoding transcoding,
      required RTCLiveTranscodingObserver observer});

  /// Stops pushing streams to CDN.
  Future<void> stopLiveTranscoding(String taskId);

  /// Updates parameters needed when pushing media streams to CDN.
  Future<void> updateLiveTranscoding(
      {required String taskId, required LiveTranscoding transcoding});

  /// This function is currently not available. Creates a new task of pushing a single media stream to CDN.
  ///
  /// [taskId]: Task ID.<br>
  /// You may want to start more than one task to push streams to CDN. When you do that, use different IDs for corresponding tasks; if you will start only one task, use an empty string.
  ///
  /// [param]: Configurations for pushing a single stream to CDN.
  ///
  /// [observer]: Register this observer to receive callbacks from the SDK.
  ///
  /// Notes:
  /// + After this API is called, the local user will receive [RTCPushSingleStreamToCDNObserver.onStreamPushEvent] about the result and errors during the pushing process.
  /// + Call [RTCVideo.stopPushStreamToCDN] to stop the task.
  Future<void> startPushSingleStreamToCDN(
      {required String taskId,
      required PushSingleStreamParam param,
      required RTCPushSingleStreamToCDNObserver observer});

  /// Stops the task of pushing a single media stream to CDN.
  Future<void> stopPushStreamToCDN(String taskId);

  /// Starts publishing a public media stream.
  ///
  /// A public stream refers to a media stream that does not belong to any room or any user. Users within the same `appID` can call [RTCVideo.startPlayPublicStream] to subscribe to the public stream regardless the user has joined which room or has not joined any room.
  ///
  /// [publicStreamId]: ID of the public stream.
  ///
  /// [publicStreamParam]: Configurations about the public stream.<br>
  /// A public stream can include a bundle of media streams and appears as the designated layout.<br>
  /// If the specified media stream has not yet been published, the public stream will be updated in real time after the specified stream starts to be published.
  ///
  /// Returned value:
  /// + 0: Success, and [RTCVideoEventHandler.onPushPublicStreamResult] will be triggered.
  /// + !0: Failure because of invalid parameter or empty parameters.
  ///
  /// Notes:
  /// + Call [RTCVideo.updatePublicStreamParam] to update the properties of the public stream which is published by the same user. Calling this API with the same stream ID repeatedly by the same user can not update the existing public stream.
  /// + If Users with different userID call this API with the same stream ID, the public stream will be updated with the parameters passed in the latest call.
  /// + To publish multiple public streams, call this API with different stream ID.
  /// + Call [RTCVideo.stopPushPublicStream] to stop publishing the public stream.
  /// + Please contact technical support to enable this function before calling the public stream related APIs.
  Future<int?> startPushPublicStream(
      {required String publicStreamId,
      required PublicStreaming publicStreamParam});

  /// Stops publishing the public stream.
  Future<int?> stopPushPublicStream(String publicStreamId);

  /// Updates the configurations of the public stream.
  Future<int?> updatePublicStreamParam(
      {required String publicStreamId,
      required PublicStreaming publicStreamParam});

  /// Subscribes to specified public stream.
  ///
  /// This API can be called to get and play the specified public stream regardless of whether the user is in the room or not.<br>
  /// If the stream has not been published then, the local user will receive the public stream once it starts publishing.
  ///
  /// Returned value:
  /// + 0: Success, and [RTCVideoEventHandler.onPlayPublicStreamResult] will be triggered.
  /// + !0: Failure because of invalid parameter or empty parameters.
  ///
  /// Notes:
  /// + We recommend you bind a view for the public stream before calling this API.
  /// + After calling this API, the local user will receive [RTCVideoEventHandler.onFirstPublicStreamVideoFrameDecoded] and [RTCVideoEventHandler.onFirstPublicStreamAudioFrame] about firt video and audio frame decoding for public streams.
  /// + If the public stream contains SEI information, the local user will be informed by [RTCVideoEventHandler.onPublicStreamSEIMessageReceived].
  /// + Call [RTCVideo.stopPlayPublicStream] to unsubscribe from the public stream.
  Future<int?> startPlayPublicStream(String publicStreamId);

  /// Stops subscribing from the public stream.
  Future<int?> stopPlayPublicStream(String publicStreamId);

  /// Unsets the public stream video canvas
  Future<int?> removePublicStreamVideo(String publicStreamId);

  /// Sets the business ID.
  ///
  /// You can distinguish different business scenarios (roles/strategies, etc.) by [businessId].<br>
  /// You can customize your businessId to serve as a sub AppId, which can share and refine the function of the AppId, but it does not need authentication.
  ///
  /// Returned value:
  /// + 0: Success
  /// + < 0: Failure
  /// + -6001: The user is already in the room.
  /// + -6002: Invalid parameter. Legal characters include all lowercase letters, uppercase letters, numbers, and four other symbols, including '.', '-','_', and '@'.
  ///
  /// Notes: Call this API before calling [RTSRoom.joinRoom].
  Future<void> setBusinessId(String businessId);

  /// Reports user feedback to RTC.
  ///
  /// [types]: A collection of pre-defined question types with multiple choices.
  /// [problemDesc]: Specific descriptions of issues other than pre-defined questions.
  ///
  /// Returned value:
  /// + 0: Success
  /// + -1: Failure, the user has not joined a room yet.
  /// + -2: Failure, data analysis error.
  /// + -3: Failure, missing fields.
  ///
  /// Notes:
  /// If the user is in the room when reporting feedback, then the problem is located in the room(s) the user is currently in.<br>
  /// If the user is not in the room, then the problem is located in the room the engine previously exited.
  Future<int?> feedback(
      {required List<ProblemFeedback> types, required String problemDesc});

  /// Sets the fallback option for published audio & video streams.
  ///
  /// You can call this API to set whether to automatically lower the resolution you set of the published streams under limited network conditions.
  ///
  /// Notes:
  /// + This API only works after you call [RTCVideo.enableSimulcastMode] to enable the mode of publishing multiple streams.
  /// + You must call this API before the user enters the room.
  /// + When the fallback option is set, the client subscribed to the audio/video stream will receive [RTCVideoEventHandler.onSimulcastSubscribeFallback] when the audio/video stream published by the local user falls back or resumes from the fallback.
  /// + You can alternatively set fallback options in th console, which is of higher priority.
  Future<int?> setPublishFallbackOption(PublishFallbackOption option);

  /// Sets the fallback option for subscribed RTC streams..
  ///
  /// You can call this API to set whether to lower the resolution of currently subscribed stream under limited network conditions.
  ///
  /// Notes:
  /// + You must call this API before the user enters the room.
  /// + When the fallback option is set, the local user will receive [RTCVideoEventHandler.onSimulcastSubscribeFallback] when the subscribed audio/video stream falls back or resumes from a fallback.
  /// + You can alternatively set fallback options in th console, which is of higher priority.
  Future<int?> setSubscribeFallbackOption(SubscribeFallbackOption option);

  /// Set the priority of the specified remote user in the fallback.
  ///
  /// If the subscribe stream fallback option is enabled, the quality of streams received from high-priority users will be prioritized under limit network conditions.
  ///
  /// Notes:
  /// + Call this API together with [RTCVideo.setSubscribeFallbackOption].
  /// + This method can be used before and after entering the room, and the priority of the remote user can be modified.
  Future<int?> setRemoteUserPriority(
      {required String roomId,
      required String uid,
      required RemoteUserPriority priority});

  /// Set the built-in encryption method.
  ///
  /// [key]: Encryption key, limited to 36 bits in length, beyond which it will be truncated.
  ///
  /// Notes: This API can be called repeatedly before or after the user enters the room with the last called parameter as the effective parameter.
  Future<void> setEncryptInfo(
      {required EncryptType aesType, required String key});

  /// Creates a room instance.
  ///
  /// This API only returns a room instance. You still need to call [RTCRoom.joinRoom] to actually create/join the room.<br>
  /// A new [RTCRoom] instance is created every time this API is called.<br>
  /// Call [RTCRoom.joinRoom] in each RTCRoom instance separately to join multiple rooms at the same time.<br>
  /// In multi-room mode, users can subscribe to the audio and video streams of each room at the same time.
  ///
  /// [roomId]: An non-empty string within 128 bytes. Each room created by calling this API requires a unique roomid, so that the room can be distinguished from each other.<br>
  /// The following characters can be used:
  /// + 26 uppercase letters A~Z.
  /// + 26 lowercase letters a~z.
  /// + 10 numbers: 0~9.
  /// + "_", "@" or "-".<br>
  /// In multi-room mode, you need to use different roomId to create multiple rooms.
  ///
  /// Notes:
  /// + If the room that you wish to join already exists, you still need to call this API first to create the RTCRoom instance, and then call [RTCRoom.joinRoom].
  /// + To publish streams to the multiple rooms, call [RTCRoom.startForwardStreamToRooms] instead of enabling multi-room mode.
  Future<RTCRoom?> createRTCRoom(String roomId);

  /// Starts capturing screen sharing streams with the internal capturing modul of th RTC SDK.
  ///
  /// Notes:
  /// + After the streams are captured, you need to call [RTCRoom.publishScreen] to publish the streams.
  /// + After this API is successfully called, the local user will receive [RTCVideoEventHandler.onVideoDeviceStateChanged] and [RTCVideoEventHandler.onAudioDeviceStateChanged].
  /// + Call [RTCVideo.stopScreenCapture] to stop capturing screen sharing streams.
  Future<void> startScreenCapture(ScreenMediaType type);

  /// Updates the media type of the screen capture.
  Future<void> updateScreenCapture(ScreenMediaType type);

  /// Stops capturing screen sharing streams.
  Future<void> stopScreenCapture();

  /// Sends custom messages to screen sharing Extension. For iOS only.
  ///
  /// Call this API after calling [RTCVideo.startScreenCapture]
  Future<void> sendScreenCaptureExtensionMessage(Uint8List message);

  /// Sets runtime parameters.
  Future<void> setRuntimeParameters(Map<String, dynamic> params);

  /// Enables automatic speech recognition service.
  ///
  /// This method converts the recognized user's voice into text and calls it back to the user through [RTCASREngineEventHandler.onMessage].
  Future<void> startASR(
      {required RTCASRConfig asrConfig,
      required RTCASREngineEventHandler handler});

  /// Disables automatic speech recognition service.
  Future<void> stopASR();

  /// Records the audio & video data during the call to a local file.
  ///
  /// Returned value:
  /// + 0: Normal.
  /// + -1: Parameter exception.
  /// + -2: The current version of the SDK does not support this feature, please contact technical support.
  ///
  /// Notes:
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onRecordingStateUpdate].
  /// + If the recording is normal, the system will report the recording progress through [RTCVideoEventHandler.onRecordingProgressUpdate] every second.
  Future<int?> startFileRecording(
      {StreamIndex streamIndex = StreamIndex.main,
      required RecordingConfig config,
      required RecordingType recordingType});

  /// Stops local recording.
  ///
  /// Notes:
  /// + After calling [RTCVideo.startFileRecording], you must call this API to stop local recording.
  /// + The recording result will be called back through [RTCVideoEventHandler.onRecordingStateUpdate].
  Future<void> stopFileRecording({StreamIndex streamIndex = StreamIndex.main});

  /// Creates the audio mixing manager.
  RTCAudioMixingManager get audioMixingManager;

  /// Logs in to send out-of-room messages or send messages to the business server.
  ///
  /// The user must carry a [token] for authentication while logging in.<br>
  /// This Token is different from that required by calling [RTCRoom.joinRoom].<br>
  /// You can assign any value even null to `roomId` to generate a login token. During developing and testing, you can use temporary tokens generated on the console. Deploy the token generating application on your server.
  ///
  /// [uid] is unique within one appid.
  ///
  /// Returned value:
  /// + `0`: Success.
  /// + `-1`: Failure due to invalid parameter.
  /// + `-2`: Failure. The user is already logged in.
  /// + `-3`: Failure. There is no created engine instance. Call [RTCVideo.createRTCVideo] before calling this method.
  ///
  /// Notes:
  /// + To log out, call  [RTCVideo.logout].
  /// + After this API is successfully called, the local user will receive [RTCVideoEventHandler.onLoginResult].
  Future<int?> login({required String token, required String uid});

  /// Logs out.
  ///
  /// After calling this API to logout, it is not possible to call methods or receive callbacks related to out-of-room messages and end-to-server messages.
  ///
  /// After this API is successfully called, the local user will receive [RTCVideoEventHandler.onLogout].
  Future<void> logout();

  /// Updates the Token used by the user to log in.
  ///
  /// The Token has a certain valid period. When the Token expires, you need to call this method to update the login Token.<br>
  /// If [RTCVideo.login] is called with an expired Token, the login fails and the local user will receive [RTCVideoEventHandler.onLoginResult] about corresponding error code. You need to reacquire a Token and update it with this method.
  ///
  /// If the token is invalid and the login fails, call this method to update the token, and the SDK will automatically log in again. The user does not need to call [RTCVideo.login].
  Future<void> updateLoginToken(String token);

  /// Sets application server parameters.
  ///
  /// Before calling [RTCVideo.sendServerMessage] or [RTCVideo.sendServerBinaryMessage] to send messages to the application server, call this API to set a valid signature and application server address.
  ///
  /// [signature]: Dynamic signature which is used to authenticate the request.
  ///
  /// [url]: Address of the application server.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerParamsSetResult].
  Future<void> setServerParams(
      {required String signature, required String url});

  /// Queries the login status of the specified remote user or local user before sending the message.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onGetPeerOnlineStatus].
  /// + Before sending out-of-room messages, users can find out whether the user on the other end is logged in or not through this API, so that they can decide whether to send messages or not. You can also check your own login status through this API.
  Future<void> getPeerOnlineStatus(String peerUid);

  /// Sends a text message to a specified user outside the room.
  ///
  /// The length of [message] must not exceed 64KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onUserMessageSendResultOutsideRoom] about the sending result.
  /// + If the text message is sent successfully, the user specified by [uid] will receive [RTCVideoEventHandler.onUserMessageReceivedOutsideRoom].
  Future<int?> sendUserMessageOutsideRoom(
      {required String uid,
      required String message,
      required MessageConfig config});

  /// Sends a binary message to a specified user outside the room.
  ///
  /// The length of [message] must not exceed 46KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onUserMessageSendResultOutsideRoom] about the sending result.
  /// + If the binary message is sent successfully, the user specified by [uid] will receive [RTCVideoEventHandler.onUserBinaryMessageReceivedOutsideRoom]。
  Future<int?> sendUserBinaryMessageOutsideRoom(
      {required String uid,
      required Uint8List message,
      required MessageConfig config});

  /// Client sends a text message to the business server.
  ///
  /// The length of [message] must not exceed 64KB.
  ///
  /// + Call this API after calling [RTCVideo.login] to log in and [RTCVideo.setServerParams] to set up the business server.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerMessageSendResult].
  Future<int?> sendServerMessage(String message);

  /// Client sends a binary message to the business server.
  ///
  /// The length of [message] must not exceed 46KB.
  ///
  /// + Call this API after calling  [RTCVideo.login] to log in and [RTCVideo.setServerParams] to set up the business server.
  /// + After this API is called, the local user will receive [RTCVideoEventHandler.onServerMessageSendResult].
  Future<int?> sendServerBinaryMessage(Uint8List message);

  /// Starts pre-call network detection.
  ///
  /// [isTestUplink]: Whether to detect uplink bandwidth.
  ///
  /// [expectedUplinkBitrate]: Expected uplink bandwidth in kbps within a range of {0, [100-10000]}. `0` indicates that the maximum code rate is specified by the SDK.
  ///
  /// [isTestDownlink]: Whether to detect downlink bandwidth.
  ///
  /// [expectedDownlinkBitrate]: Expected downlink bandwidth in kbps within a range of {0, [100-10000]}. `0` indicates that the maximum code rate is specified by the SDK.
  ///
  /// Notes:
  /// + After a successful call to this API, the [RTCVideoEventHandler.onNetworkDetectionResult] callback will be received once in 3s and every 2s thereafter to notify the detection result.
  /// + If detection is stopped, the detection will be notified once by [RTCVideoEventHandler.onNetworkDetectionStopped].
  Future<NetworkDetectionStartReturn> startNetworkDetection(
      {required bool isTestUplink,
      required int expectedUplinkBitrate,
      required bool isTestDownlink,
      required int expectedDownlinkBitrate});

  /// Stops pre-call network detection.
  ///
  /// Calling this API will trigger [RTCVideoEventHandler.onNetworkDetectionStopped] notifying the stop of the detection.
  Future<void> stopNetworkDetection();

  /// Sets whether to mix screen audio stream and the audio stream collected by the microphone in screen sharing.
  ///
  /// Specify the mixing method via [index]:
  /// + `main`: Mixing the screen audio stream with the audio stream captured by the microphone.
  /// + `screen`: Default setting. Split the screen audio stream and the audio stream captured by the microphone into two audio streams.
  ///
  /// Call this API before [RTCRoom.publishScreen], otherwise [RTCVideoEventHandler.onWarning] will be triggered.
  Future<void> setScreenAudioStreamIndex(StreamIndex index);

  /// Sends audio stream synchronization information.
  ///
  /// This method sends the message to the remote end via an audio stream and synchronizes it with the audio stream.
  ///
  /// The [data] length must be `[1, 16]` bytes.
  ///
  /// Returned value:
  /// + `>=0`:  Message sent successfully. Returns the number of successful sends
  /// + `-1`: Message sending failed. The message length is greater than 255 bytes.
  /// + `-2`: Message sending failed. The message content is empty.
  /// + `-3`: Message sending failed. This screen stream was not published when the message was synchronized through the screen stream.
  /// + `-4`: Message sending failed. This audio stream is not yet published when you synchronize messages with an audio stream captured by a microphone or custom device, as described in [ErrorCode].
  ///
  /// After a successful call to this API, the remote user will receive [RTCVideoEventHandler.onStreamSyncInfoReceived].
  Future<int?> sendStreamSyncInfo(
      {required Uint8List data, required StreamSyncInfoConfig config});

  /// Controls whether to play local audio streams.
  ///
  /// This method does not affect local audio playback devices.
  Future<void> muteAudioPlayback(bool muteState);

  /// Adds watermark to designated video stream.
  ///
  /// [imagePath]: File path of the watermark image. You can use the absolute path or the Asset path(/assets/xx.png).<br>
  /// The path should be less than 512 bytes. <br>
  /// The watermark image should be in PNG or JPG format.
  ///
  /// Notes:
  /// + Call [RTCVideo.clearVideoWatermark] to remove the watermark on the designated video stream.
  /// + You can only add one watermark to one video stream. The newly added watermark replaces the previous one. You can call this API multiple times to add watermarks to different streams.
  /// + You can call this API before and after joining room.
  /// + If you mirror the preview, or the preview and the published stream, the watermark will also be mirrored locally, but the published watermark will not be mirrored.
  /// + hen you enable simulcast mode, the watermark will be added to all video streams, and it will scale down to smaller encoding configurations accordingly.
  Future<void> setVideoWatermark(
      {StreamIndex streamIndex = StreamIndex.main,
      required String imagePath,
      required WatermarkConfig watermarkConfig});

  /// Removes video watermark from designated video stream.
  Future<void> clearVideoWatermark(
      {StreamIndex streamIndex = StreamIndex.main});

  /// Starts cloud proxy.
  ///
  /// Notes:
  /// + Call this API before joining the room.
  /// + Pre-call network detection should be after the cloud proxy is turned on.
  /// + [RTCVideoEventHandler.onCloudProxyConnected] will be triggered after starting cloud proxy and successfully connecting the cloud proxy server.
  /// + Call [RTCVideo.stopCloudProxy] to stop cloud proxy.
  Future<void> startCloudProxy(List<CloudProxyInfo> cloudProxiesInfo);

  /// Stops cloud proxy.
  Future<void> stopCloudProxy();

  /// Starts a call test.
  ///
  /// Before entering the room, you can call this API to test whether your local audio/video equipment as well as the upstream and downstream networks are working correctly.  <br>
  /// Once the test starts, SDK will record your sound or video. If you receive the playback within the delay range you set, the test is considered normal.
  ///
  /// [delayTime]: Delayed audio/video playback time specifying how long you expect to receive the playback after starting the. The range of the value is [2,10] in seconds and the default value is 2.
  ///
  /// API call result:
  /// + 0: Success.
  /// + -1: Failure, testing in progress.
  /// + -2: Failure, the user us already in the room.
  /// + -3: Failure, neither video nor audio is captured.
  /// + -4: Failure, parameter exception.
  /// + -5: Failure, the roomID is already used.
  ///
  /// Notes:
  /// + Once you start the test, you can either call [RTCVideo.stopEchoTest] or wait until the test stops automatically after 60s, to start the next test or enter the room.
  /// + All APIs related to device control and stream control called before this API are invalidated during the test and are restored after the test.
  /// + All the calls to APIs related to device control, stream control, and room entry called during the test do not take effect, and will triggered [RTCVideoEventHandler.onWarning].
  /// + The result of the audio/video loop detection is notified via the [RTCVideoEventHandler.onEchoTestResult] callback.
  Future<int?> startEchoTest(EchoTestConfig config, int delayTime);

  /// Stops the current call test.
  ///
  /// Notes: After stopping the test with this API, all the system devices and streams are restored to the state they were in before the test.
  Future<int?> stopEchoTest();
}
