// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'dart:collection';
import 'dart:typed_data';

import '../src/base/bytertc_enum_convert.dart';
import 'bytertc_common_defines.dart';
import 'bytertc_video_event_handler.dart';

/// Used to initialize the RTCVideo.
class RTCVideoContext {
  /// A unique identifier for each App, randomly generated by the RTC console.
  ///
  /// Engine instances created with different AppIds cannot communicate with each other.
  String appId;

  /// Delegate sent from SDK to App.
  RTCVideoEventHandler? eventHandler;

  /// Reserved parameter.
  Map<String, dynamic>? parameters;

  RTCVideoContext(this.appId, {this.eventHandler, this.parameters});

  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap();
    dic['appId'] = appId;
    if (parameters != null) {
      dic['parameters'] = parameters;
    }
    return dic;
  }
}

/// Camera on a mobile device.
enum CameraId {
  /// Front-facing camera.
  front,

  /// Rear-facing camera (default setting).
  back,

  /// External camera.
  external
}

/// Video rendering scale mode.
enum VideoRenderMode {
  /// Fill and Crop.
  ///
  /// The video frame is scaled with fixed aspect ratio, until it completely fills the canvas. The region of the video exceeding the canvas will be cropped.
  hidden,

  /// Fit.
  ///
  /// The video frame is scaled with fixed aspect ratio, until it fits just within the canvas. Other part of the canvas is filled with the background color.
  fit,

  /// Fill the canvas.
  ///
  /// The video frame is scaled to fill the canvas. During the process, the aspect ratio may change.
  adaptive,
}

/// Mirror type.
enum MirrorType {
  /// Neither preview nor the published video stream is mirrored.
  none,

  /// The preview is mirrored. The published video stream is not mirrored.
  render,

  /// The preview and the published video stream are mirrored.
  renderAndEncoder,
}

/// Basic beauty effect.
enum EffectBeautyMode {
  /// Brightness.
  whiteMode,

  /// Smoothness.
  smoothMode,

  /// Sharpness.
  sharpenMode,
}

/// Video frame rotation angle.
enum VideoRotation {
  /// Video does not rotate (default setting).
  rotation0,

  /// Video rotates 90 degrees clockwise
  rotation90,

  /// Video rotates 180 degrees clockwise
  rotation180,

  /// Video rotates 270 degrees clockwise
  rotation270
}

/// Video frame scale mode.
enum ScaleMode {
  /// Automatic mode. The default value is `stretch`.
  auto,

  /// Stretch the video frame until the video frame and the window have the same resolution. The video frame's aspect ratio can be changed as it is automatically stretched to fill the window, but the whole image is visible.
  stretch,

  /// Fit the window with cropping。
  ///
  /// Scale the video frame uniformly until the window is filled. If the video frame's aspect ratio is different from that of the window, the extra part of the video frame will be cropped. <br>
  /// After the scaling process is completed, the width or height of the video frame will be consistent with that of the window, and the other dimension will be greater than or equal to that of the window.
  fitWithCropping,

  /// Fit the window with filling.
  ///
  /// Scale the video frame uniformly until its width or height reaches the boundary of the window. If the video frame's aspect ratio is different from that of the window, the area that is not filled will be black. <br>
  /// After the scaling process is completed, the width or height of the video frame will be consistent with that of the window, and the other dimension will be less than or equal to that of the window.
  fitWithFilling
}

/// Coding strategy preference.
enum EncoderPreference {
  /// No preference.
  disable,

  /// Frame rate first.
  maintainFrameRate,

  /// Quality first.
  maintainQuality,

  /// Balances quality and frame rate.
  balance,
}

/// Type of media stream supposed to be paused/resume.
enum PauseResumeControlMediaType {
  /// Controls audio only.
  audio,

  /// Controls video only.
  video,

  /// Controls both audio and video.
  videoAndAudio,
}

/// Stream switching information when a local user-subscribed remote stream triggers a fallback policy.
class RemoteStreamSwitch {
  /// User ID of the publisher of the subscribed media stream.
  final String uid;

  /// Whether it is a screen-sharing stream.
  final bool isScreen;

  /// Index corresponding to the resolution of the subscribed video stream before the stream switch.
  final int beforeVideoIndex;

  /// Index corresponding to the resolution of the subscribed video stream after the stream switch.
  final int afterVideoIndex;

  /// Is there a video stream before switching.
  final bool beforeEnable;

  /// Is there a video stream after switching.
  final bool afterEnable;

  /// Reason of the fallback or recover of the stream.
  final FallbackOrRecoverReason reason;

  RemoteStreamSwitch({
    this.uid = '',
    this.isScreen = false,
    this.beforeVideoIndex = 0,
    this.afterVideoIndex = 0,
    this.beforeEnable = false,
    this.afterEnable = false,
    this.reason = FallbackOrRecoverReason.unknown,
  });

  factory RemoteStreamSwitch.fromMap(Map<dynamic, dynamic> map) {
    return RemoteStreamSwitch(
        uid: map['uid'],
        isScreen: map['isScreen'],
        beforeVideoIndex: map['beforeVideoIndex'],
        afterVideoIndex: map['afterVideoIndex'],
        beforeEnable: map['beforeEnable'],
        afterEnable: map['afterEnable'],
        reason: (map['reason'] as int).fallbackOrRecoverReason);
  }
}

/// Stream mixing type.
enum StreamMixingType {
  /// Pushes streams to CDN using RTC server.
  byServer,

  /// Intelligently pushes streams to CDN.
  byClient,
}

/// Events during pushing streams to CDN.
enum StreamMixingEvent {
  /// Undefined event.
  base,

  /// Request to start the task of pushing streams to CDN.
  start,

  /// The task to push streams to CDN started.
  startSuccess,

  /// Failed to start the task to push streams to CDN.
  startFailed,

  /// Request to update the configuration for the task to push streams to CDN.
  update,

  /// Successfully update the configuration for the task to push streams to CDN.
  updateSuccess,

  /// Failed to update the configuration for the task to push streams to CDN.
  updateFailed,

  /// Request to stop the task to push streams to CDN.
  stop,

  /// The task to push streams to CDN stopped.
  stopSuccess,

  /// Failed to stop the task to push streams to CDN.
  stopFailed,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  changeMixeType,

  /// Got the first frame of the mixed audio stream by client.
  firstAudioFrameByClientMix,

  /// Got the first frame of the mixed video stream by client.
  firstVideoFrameByClientMix,

  /// Timeout for the request to update the configuration for the task to push streams to CDN.
  updateTimeout,

  /// Timeout for the request to start the task to push streams to CDN.
  startTimeout,

  /// Error in the parameters of the request for the task to push streams to CDN.
  requestParamError,

  /// Mixing image.
  mixImage,
}

/// Errors occurring during pushing streams to CDN.
enum TranscoderErrorCode {
  /// Successfully pushed streams to target CDN.
  ok,

  /// Invalid parameters detected by Client SDK.
  invalidParam,

  /// Program runs with an error, the state machine is in abnormal condition.
  invalidState,

  /// Invalid operation.
  invalidOperator,

  /// Request timed out. Please check network status and retry.
  timeOut,

  /// Invalid parameters detected by the server.
  invalidParamByServer,

  /// Your subscription to the stream has expired.
  subTimeoutByServer,

  /// Internal server error.
  invalidStateByServer,

  /// The server failed to push streams to CDN.
  authenticationByCDN,

  /// Signaling connection timeout error. Please check network status and retry.
  timeoutBySignaling,

  /// Fail to mix image.
  mixImageFail,

  /// Unknown error from server.
  unKnownErrorByServer,
}

/// Stream mixing content type.
enum TranscoderContentControlType {
  /// The mixed stream contains audio and video.
  hasAudioAndVideo,

  /// The mixed stream only contains audio.
  hasAudioOnly,

  /// The mixed stream only contains video.
  hasVideoOnly,
}

/// Stream mixing region type.
enum TranscoderLayoutRegionType {
  /// The region type is a video stream.
  videoStream,

  /// The region type is an image.
  image,
}

/// Advanced Audio Coding (AAC) profile
enum AACProfile {
  /// Low-Complexity profile (AAC-LC).
  lc,

  ///  HE-AAC profile (AAC LC with SBR)
  hev1,

  /// HE-AAC v2 profile (AAC LC with SBR and Parametric Stereo)
  hev2,
}

/// Video transcoding related configurations.
class LiveTranscodingVideoConfig {
  /// Mixed video encoder format.
  String codec = 'H264';

  /// Mixed video frame rate.
  int fps = 30;

  /// The number of frames between two I-frames.
  int gop = 60;

  /// Whether to push streams with low latency.
  bool lowLatency = true;

  /// Mixed video bitrate in kbps.
  int kBitrate = 500;

  /// Mixed video width in px.
  int width = 360;

  /// Mixed video height in px.
  int height = 640;

  LiveTranscodingVideoConfig(
      {this.codec = 'H264',
      this.fps = 30,
      this.gop = 60,
      this.lowLatency = true,
      this.kBitrate = 500,
      this.width = 360,
      this.height = 640});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'codec': codec,
      'fps': fps,
      'gop': gop,
      'lowLatency': lowLatency,
      'kBitrate': kBitrate,
      'width': width,
      'height': height,
    };
  }
}

/// Audio transcoding related configurations.
class LiveTranscodingAudioConfig {
  /// Mixed audio encoder format.
  String codec = 'AAC';

  /// Mixed audio bitrate in kbps.
  int kBitrate = 64;

  /// Audio frame sample rate in HZ。Supported sample rates: 8000HZ, 16000HZ, 32000HZ, 44100HZ, 48000HZ.
  int sampleRate = 48000;

  /// Number of channels, either 1 or 2.
  int channels = 2;

  /// Advanced Audio Coding (AAC) profile.
  AACProfile aacProfile = AACProfile.lc;

  LiveTranscodingAudioConfig(
      {this.channels = 2,
      this.codec = 'AAC',
      this.kBitrate = 64,
      this.sampleRate = 48000,
      this.aacProfile = AACProfile.lc});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'codec': codec,
      'kBitrate': kBitrate,
      'sampleRate': sampleRate,
      'channels': channels,
      'aacProfile': aacProfile.value,
    };
  }
}

/// Image parameters for stream mixing
class TranscoderLayoutRegionDataParam {
  /// Width of the original image in px.
  int imageWidth = 0;

  /// Height of the original image in px.
  int imageHeight = 0;

  TranscoderLayoutRegionDataParam(
      {required this.imageWidth, required this.imageHeight});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'imageWidth': imageWidth,
      'imageHeight': imageHeight,
    };
  }
}

/// Information about layout of a single video stream in a mixed stream.
class LiveTranscodingRegion {
  /// ID of the user who publishes the video stream.
  String uid;

  /// Room ID (required).
  String roomId;

  /// The offset of the user video layout relative to the left side of the canvas. Relative value, in the range `[0.0, 1.0]`.
  double x;

  /// The offset of the user video layout relative to the top of the canvas. Relative value, in the range `[0.0, 1.0]`.
  double y;

  /// The ratio of the user video width relative to the user's original video width. Relative value, in the range `[0.0, 1.0]`.
  double w;

  /// The ratio of the user video height relative to the user's original video height. Relative value, in the range `[0.0, 1.0]`.
  double h;

  /// The layer of the user video layout in the canvas. 0 is the bottom layer, the larger the value the higher the layer, the range is `[0, 100]`.
  int zorder;

  /// The transparency in range of [0.0, 1.0].
  double alpha;

  /// The stream mixing content type.
  TranscoderContentControlType contentControl;

  /// Render mode
  VideoRenderMode renderMode;

  /// Whether the source user of the stream is a local user.
  bool localUser;

  /// Whether the stream comes from screen sharing.
  bool isScreen;

  /// Stream mixing region type.
  TranscoderLayoutRegionType type;

  /// The RGBA data of the mixing image.
  Uint8List? data;

  /// Image parameters for stream mixing.
  TranscoderLayoutRegionDataParam? dataParam;

  LiveTranscodingRegion(
      {required this.uid,
      required this.roomId,
      required this.x,
      required this.y,
      required this.w,
      required this.h,
      this.zorder = 0,
      this.alpha = 1.0,
      this.contentControl = TranscoderContentControlType.hasAudioAndVideo,
      this.renderMode = VideoRenderMode.hidden,
      required this.localUser,
      this.isScreen = false,
      this.type = TranscoderLayoutRegionType.videoStream,
      this.data,
      this.dataParam});

  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap.of({
      'uid': uid,
      'roomId': roomId,
      'x': x,
      'y': y,
      'w': w,
      'h': h,
      'zorder': zorder,
      'alpha': alpha,
      'contentControl': contentControl.value,
      'renderMode': renderMode.value,
      'localUser': localUser,
      'isScreen': isScreen,
      'type': type.value,
    });
    if (this.data != null) {
      dic['data'] = data;
    }
    if (this.dataParam != null) {
      dic['dataParam'] = dataParam?.toMap();
    }
    return dic;
  }
}

/// Layout information about pushing streams to CDN.
class LiveTranscodingLayout {
  /// Region information.
  List<LiveTranscodingRegion> regions;

  /// SEI information with 4096 bytes in length.
  String appData = '';

  /// The background color to be set, in RGB hex value. The default value is #000000 (black).
  String backgroundColor = '#000000';

  LiveTranscodingLayout(
      {required this.regions,
      this.appData = '',
      this.backgroundColor = '#000000'});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'appData': appData,
      'backgroundColor': backgroundColor,
      'regions': regions.map((e) => e.toMap()).toList(),
    };
  }
}

/// Configurations of pushing streams to CDN.
class LiveTranscoding {
  /// Stream mixing type.
  StreamMixingType mixType;

  /// The CDN url where the mixed stream is supposed to be pushed.
  String url;

  /// Room ID
  String roomId;

  /// User ID
  String uid;

  /// Video transcoding related configurations.
  LiveTranscodingVideoConfig video;

  /// Audio transcoding related configurations.
  LiveTranscodingAudioConfig audio;

  /// Layout information.
  LiveTranscodingLayout layout;

  /// Passed-in authentication information.
  Map<String, dynamic>? authInfo;

  LiveTranscoding(
      {this.mixType = StreamMixingType.byServer,
      required this.url,
      required this.roomId,
      required this.uid,
      required this.video,
      required this.layout,
      required this.audio,
      this.authInfo});

  Map<String, dynamic> toMap() {
    HashMap<String, dynamic> dic = HashMap.of({
      'url': url,
      'roomId': roomId,
      'uid': uid,
      'mixType': mixType.value,
      'video': video.toMap(),
      'layout': layout.toMap(),
      'audio': audio.toMap(),
    });
    if (authInfo != null) {
      dic['authInfo'] = authInfo;
    }
    return dic;
  }
}

/// Status when pushing a single media stream to CDN.
enum StreamSinglePushEvent {
  /// Undefined status.
  base,

  /// Request to start the task to push a single stream to CDN.
  start,

  /// The task to a single stream to CDN started.
  success,

  /// Failed to start the task to push a single stream to CDN.
  failed,

  /// The task to a single stream to CDN stopped.
  stop,

  /// Timeout for the request to start the task to push a single stream to CDN.
  timeout,

  /// Invalid parameter.
  paramError,
}

/// Configurations for pushing a single media stream to CDN.
class PushSingleStreamParam {
  /// ID of the room from which the media stream is pushed.
  String roomId;

  /// ID of the user who publishes the stream.
  String uid;

  /// The target address of the CDN.
  String url;

  /// Whether the media stream is a screen-sharing stream.
  bool isScreen = false;

  PushSingleStreamParam(
      {required this.roomId,
      required this.uid,
      required this.url,
      this.isScreen = false});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'uid': uid,
      'url': url,
      'isScreen': isScreen,
    };
  }
}

/// Subscription related configuration.
class SubscribeConfig {
  /// Whether the stream is from screen sharing.
  bool isScreen;

  /// Whether video stream is included.
  bool subVideo;

  /// Whether audio stream is included.
  bool subAudio;

  /// The label of subscribed video stream with different resolution.
  ///
  /// Users can call [RTCVideo.setVideoEncoderConfig] to publish multiple video streams of different resolutions. Therefore, when subscribing to a remote stream, you need to specify the specific resolution of the subscription. This parameter is used to specify which resolution you want to subscribe, the default value is 0.
  int videoIndex;

  /// Width of the video stream in px.
  int subWidth;

  /// Height of the video stream in px.
  int subHeight;

  /// Not available yet.
  int subVideoIndex;

  /// Time domain hierarchy of the subscribed video stream, default value is 0.
  ///
  /// Not available for Android.
  int svcLayer;

  /// Expected maximum frame rate of the subscribed stream in px. The default value is 0, values greater than 10 are valid.
  ///
  /// If the frame rate of the stream published is lower than the value you set, or if your subscribed stream falls back under limited network conditions, the frame rate you set will drop accordingly. <br>
  /// Only valid if the stream is coded with SVC technique.
  int frameRate;

  SubscribeConfig(
      {required this.isScreen,
      required this.subVideo,
      required this.subAudio,
      required this.videoIndex,
      required this.subWidth,
      required this.subHeight,
      required this.subVideoIndex,
      required this.svcLayer,
      required this.frameRate});

  factory SubscribeConfig.fromMap(Map<dynamic, dynamic> map) {
    return SubscribeConfig(
        isScreen: map['isScreen'],
        subVideo: map['subVideo'],
        subAudio: map['subAudio'],
        videoIndex: map['videoIndex'],
        subWidth: map['subWidth'],
        subHeight: map['subHeight'],
        subVideoIndex: map['subVideoIndex'],
        svcLayer: map['svcLayer'],
        frameRate: map['frameRate']);
  }
}

/// Rectangle area.
class Rectangle {
  /// The x coordinate of the upper left corner of the rectangular area.
  int x;

  /// The y coordinate of the upper left corner of the rectangular area.
  int y;

  /// Rectangle width in px.
  int width;

  /// Rectangle height in px.
  int height;

  Rectangle(
      {required this.x,
      required this.y,
      required this.width,
      required this.height});

  factory Rectangle.fromMap(Map<dynamic, dynamic> map) {
    return Rectangle(
      x: map['x'],
      y: map['y'],
      width: map['width'],
      height: map['height'],
    );
  }
}

/// Face detection result.
class FaceDetectionResult {
  /// Face detection result:
  /// + 0: Success.
  /// + !0: Failure, see [Error Code Table](https://docs.byteplus.com/effects/docs/error-code-table) for the reason.
  int detectResult;

  /// Width of the original image (px).
  int imageWidth;

  /// Height of the original image (px).
  int imageHeight;

  /// The face recognition rectangles. The length of the array is the same as the number of detected faces.
  List<Rectangle> faces;

  FaceDetectionResult(
      {required this.detectResult,
      required this.imageWidth,
      required this.imageHeight,
      required this.faces});

  factory FaceDetectionResult.fromMap(Map<dynamic, dynamic> map) {
    return FaceDetectionResult(
      detectResult: map['detectResult'],
      imageWidth: map['imageWidth'],
      imageHeight: map['imageHeight'],
      faces: (map['faces'] as List<dynamic>)
          .map((e) => Rectangle.fromMap(e))
          .toList(),
    );
  }
}

/// Virtual background types.
enum VirtualBackgroundSourceType {
  /// Replace the background with a solid color.
  color,

  /// Replace the background with the specified image.
  image,
}

/// Virtual background source.
class VirtualBackgroundSource {
  /// Virtual background types.
  VirtualBackgroundSourceType sourceType;

  /// The solid color of the background. The format is 0xAARRGGBB.
  int sourceColor;

  /// The absolute path of the specified image.
  ///
  /// + You can use the absolute path of the local file (file://xxx) and the Asset path (asset://xxx).
  /// + You can upload a .JPG, .PNG, or .JPEG file.
  /// + The image with a resolution higher than 1080p(Full HD) will be rescaled proportionally to fit in the video.
  /// + If the image's aspect ratio matches the video's, the image will be rescaled proportionally to fit in the video.  <br>
  /// + If the image’s aspect ratio doesn't match the video's, the shortest side (either height or width) of the image will be stretched proportionally to match the video. Then the image will be cropped to fill in the video.  <br>
  /// + The transparent area in the image will be filled with white color.
  String sourcePath;

  VirtualBackgroundSource(
      {required this.sourceType, this.sourceColor = 0, this.sourcePath = ''});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'sourceType': sourceType.value,
      'sourceColor': sourceColor,
      'sourcePath': sourcePath,
    };
  }
}

/// Video frame information。
class VideoFrameInfo {
  /// Width in px.
  int? width;

  /// Height in px.
  int? height;

  /// Video frame clockwise rotation angle.
  VideoRotation? rotation;

  VideoFrameInfo({this.width, this.height, this.rotation});

  factory VideoFrameInfo.fromMap(Map<dynamic, dynamic> map) {
    return VideoFrameInfo(
        width: map['width'],
        height: map['height'],
        rotation: (map['rotation'] as int).videoRotation);
  }
}

/// Video capture preference.
enum VideoCapturePreference {
  /// Automatically set capture parameters (default setting).
  ///
  /// SDK determines the best video capture parameters referring to the camera output parameters and the encoder configuration.
  auto,

  /// Manually set capture parameters, including resolution and frame rate.
  manual,

  /// The capture parameters are consistent with the encoding parameters.
  autoPerformance,
}

/// Video capture configuration.
class VideoCaptureConfig {
  /// Video capture preference.
  VideoCapturePreference capturePreference;

  /// Width in px.
  int width = 0;

  /// Height in px.
  int height = 0;

  /// Video capture frame rate in fps.
  int frameRate = 0;

  VideoCaptureConfig({
    this.capturePreference = VideoCapturePreference.auto,
    this.width = 0,
    this.height = 0,
    this.frameRate = 0,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'capturePreference': capturePreference.value,
      'width': width,
      'height': height,
      'frameRate': frameRate,
    };
  }
}

/// Current audio device type.
enum VideoDeviceType {
  /// Unknown device type.
  unknown,

  /// Audio rendering device type.
  renderDevice,

  /// Audio capturing device type.
  captureDevice,

  /// Screen stream video device.
  screenCaptureDevice,
}

/// Video orientation.
enum VideoRotationMode {
  /// Follow the orientation of the App.
  followApp,

  /// Follow the gravity.
  followGSensor,
}

/// Orientation of the video frame.
enum VideoOrientation {
  /// The original orientation of the captured video frame. Default setting.
  adaptive,

  /// Portrait mode. Adjust the captured video frame to vertical layout in RTC SDK.
  portrait,

  /// Landscape mode. Adjust the captured video frame to horizontal layout in RTC SDK.
  landscape,
}

/// Camera flash status.
enum TorchState {
  /// Off.
  off,

  /// On.
  on,
}

/// Media stream type.
enum MediaStreamType {
  /// Controls audio only.
  audio,

  /// Controls video onlys
  video,

  /// Controls both audio and video.
  both,
}

/// Description of video stream parameters.
class VideoEncoderConfig {
  /// Video width in px.
  int width;

  /// Height width in px.
  int height;

  /// Video frame rate in fps.
  int frameRate;

  /// Maximum bit rate in kbps.
  ///
  /// If you set this value to -1, SDK will automatically calculate the applicable bit rate based on the input resolution and frame rate.<br>
  /// No stream will be encoded and published if you set this value to 0.<br>
  /// In 344 and later versions, the default value is -1 for internal capturing, but no default value in versions earlier than 344.
  int maxBitrate;

  /// Coding policy preference.
  EncoderPreference encoderPreference;

  VideoEncoderConfig(
      {required this.width,
      required this.height,
      required this.frameRate,
      this.maxBitrate = -1,
      this.encoderPreference = EncoderPreference.maintainFrameRate});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'width': width,
      'height': height,
      'frameRate': frameRate,
      'maxBitrate': maxBitrate,
      'encoderPreference': encoderPreference.value,
    };
  }
}

/// Information on remote video frame.
class RemoteVideoConfig {
  /// Expected maximum frame rate of the subscribed stream in px. The default value is 0, values greater than 10 are valid.
  ///
  /// If the frame rate of the stream published is lower than the value you set, or if your subscribed stream falls back under limited network conditions, the frame rate you set will drop accordingly.<br>
  /// Only valid if the stream is coded with SVC technique.
  int frameRate;

  /// Video width in px.
  int width;

  /// Video height in px.
  int height;

  RemoteVideoConfig({this.frameRate = 0, this.width = 0, this.height = 0});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'frameRate': frameRate,
      'width': width,
      'height': height,
    };
  }
}

/// Room configuration.
class RoomConfig {
  /// Room profile.
  ///
  /// The default is `communication`.<br>
  /// The setting cannot be changed after joining the room.
  RoomProfile profile;

  /// Whether to publish media streams automatically. The default is automatic publishing.
  ///
  /// Unvisible user cannot publish media streams regardless of the value of `isAutoPublish`.
  /// Only one of the rooms the user joined can be set to auto-publish.
  bool isAutoPublish;

  /// Whether to automatically subscribe to the audio stream. The default is automatic subscription.
  bool isAutoSubscribeAudio;

  /// Whether to automatically subscribe to the main video stream. The default is automatic subscription.
  bool isAutoSubscribeVideo;

  /// Expected configuration of remote video stream.
  RemoteVideoConfig? remoteVideoConfig;

  RoomConfig(
      {this.profile = RoomProfile.communication,
      this.isAutoPublish = true,
      this.isAutoSubscribeAudio = true,
      this.isAutoSubscribeVideo = true,
      this.remoteVideoConfig});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'profile': profile.value,
      'isAutoPublish': isAutoPublish,
      'isAutoSubscribeAudio': isAutoSubscribeAudio,
      'isAutoSubscribeVideo': isAutoSubscribeVideo,
      'remoteVideoConfig': remoteVideoConfig != null
          ? remoteVideoConfig!.toMap()
          : RemoteVideoConfig().toMap(),
    };
  }
}

/// Screen media type.
enum ScreenMediaType {
  /// Capture video only.
  videoOnly,

  /// Capture audio only.
  audioOnly,

  /// Capture both audio and video.
  videoAndAudio,
}

/// Watermark's scaled coordinates and size, relative to the video stream.
class Watermark {
  /// The watermark's horizontal offset from the upper left corner of the video stream to the video stream's width in range of [0,1).
  double x;

  /// The watermark's vertical offset from the upper left corner of the video stream to the video stream's height in range of [0,1).
  double y;

  /// The watermark's width to the video stream's width in range of [0,1).
  double width;

  /// The watermark height to the video stream's height in range of [0,1).
  double height;

  Watermark({
    required this.x,
    required this.y,
    required this.width,
    required this.height,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'x': x,
      'y': y,
      'width': width,
      'height': height,
    };
  }

  static Map<String, dynamic> defaultMap() {
    return <String, dynamic>{
      'x': 0,
      'y': 0,
      'width': 0,
      'height': 0,
    };
  }
}

/// Watermark configurations.
class WatermarkConfig {
  /// Whether the watermark is visible in preview. The default value is `true`.
  bool visibleInPreview;

  /// Watermark's coordinates and size in landscape mode.
  Watermark? positionInLandscapeMode;

  /// Watermark's coordinates and size in portrait mode.
  Watermark? positionInPortraitMode;

  WatermarkConfig({
    this.visibleInPreview = true,
    this.positionInLandscapeMode,
    this.positionInPortraitMode,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'visibleInPreview': visibleInPreview,
      'positionInLandscapeMode':
          positionInLandscapeMode?.toMap() ?? Watermark.defaultMap(),
      'positionInPortraitMode':
          positionInPortraitMode?.toMap() ?? Watermark.defaultMap(),
    };
  }
}

/// Crop configuration of the public stream.
class SourceCrop {
  /// The normalized scale of the upper-left corner of the cropped video frame relative to the overall frame before cropping, in the range [0.0, 1.0).
  double locationX;

  /// The normalized scale of the upper-left corner of the cropped video frame relative to the overall frame before cropping, in the range [0.0, 1.0).
  double locationY;

  /// The normalized ratio of the width of the video frame obtained after cropping to the overall frame before cropping, in the range (0.0, 1.0].
  double widthProportion;

  /// The normalized ratio of the height of the obtained video frame after cropping to the overall frame before cropping, in the range (0.0, 1.0].
  double heightProportion;

  SourceCrop(
      {this.locationX = 0,
      this.locationY = 0,
      this.widthProportion = 1,
      this.heightProportion = 1});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'locationX': locationX,
      'locationY': locationY,
      'widthProportion': widthProportion,
      'heightProportion': heightProportion,
    };
  }
}

/// Layout of each stream to be mixed into the public stream.
class PublicStreamingRegion {
  /// Required. ID of the user who publishes the video stream.
  String uid;

  /// Required. ID of the room from which the stream is pushed.
  String roomId;

  /// Background image address.
  String alternateImage;

  /// Required. The normalized horizontal coordinate value of the top left end vertex of the user's view, ranging within [0.0, 1.0).
  double x;

  /// Required. The normalized vertical coordinate value of the top left end vertex of the user's view, ranging within [0.0, 1.0).
  double y;

  /// Required. The normalized width of the user's view, ranging within (0.0, 1.0].
  double w;

  /// Required. The normalized height of the user's view, ranging within (0.0, 1.0].
  double h;

  /// The layer order of the video in the stacks when Regions overlap with each other.
  ///
  /// [0,100], 0 for the bottom, by default. 100 for the top.
  int zorder;

  /// Required. Transparency in a range of [0.0, 1.0].
  double alpha;

  /// Required. Public media stream type.
  StreamIndex streamType;

  /// Required. Public media stream content.
  TranscoderContentControlType mediaType;

  /// Required. Video render mode.
  VideoRenderMode renderMode;

  /// Relative position and size of the user video layout.
  SourceCrop sourceCrop;

  PublicStreamingRegion({
    required this.uid,
    required this.roomId,
    this.alternateImage = "",
    required this.x,
    required this.y,
    required this.w,
    required this.h,
    required this.zorder,
    required this.alpha,
    this.streamType = StreamIndex.main,
    this.mediaType = TranscoderContentControlType.hasAudioAndVideo,
    this.renderMode = VideoRenderMode.hidden,
    required this.sourceCrop,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'uid': uid,
      'roomId': roomId,
      'alternateImage': alternateImage,
      'x': x,
      'y': y,
      'w': w,
      'h': h,
      'zorder': zorder,
      'alpha': alpha,
      'streamType': streamType.value,
      'mediaType': mediaType.value,
      'renderMode': renderMode.value,
      'sourceCrop': sourceCrop.toMap()
    };
  }
}

/// Public stream layout information.
class PublicStreamingLayout {
  /// Layout of each stream to be mixed into the public stream.
  List<PublicStreamingRegion> regions;

  /// The interpolation mode:
  /// + `0`: Insert with the last frame.
  /// + `1`: Insert with the background image. RTC will insert frames of black if no background image has been set.
  int interpolationMode;

  /// Public stream layout mode. `2`: Custom mode.
  int layoutMode;

  /// The address of the background image of the canvas.
  String backgroundImage;

  /// Background color of the canvas in RGB hex value. For example, #FFB6C1 represents light pink. The default value is #000000 (black).
  String backgroundColor;

  PublicStreamingLayout(
      {required this.regions,
      this.interpolationMode = 0,
      this.layoutMode = 2,
      this.backgroundImage = '',
      this.backgroundColor = '#000000'});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'regions': regions.map((e) => e.toMap()).toList(),
      'interpolationMode': interpolationMode,
      'layoutMode': layoutMode,
      'backgroundImage': backgroundImage,
      'backgroundColor': backgroundColor,
    };
  }
}

/// Video parameters for public streams.
class PublicStreamingVideoConfig {
  /// Required. Frame rate in fps within a range of [1, 60].
  int fps;

  /// Required. Bitrate in bps within a range of [1,10000000].
  int kBitrate;

  /// Required. Width in px within a range of [16, 1920]. It defaults to 360px and must be even.
  int width;

  /// Required. Width in px within a range of [16, 1280]. It defaults to 640px and must be even.
  int height;

  PublicStreamingVideoConfig(
      {this.fps = 30,
      this.kBitrate = 500,
      this.width = 360,
      this.height = 640});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'fps': fps,
      'kBitrate': kBitrate,
      'width': width,
      'height': height,
    };
  }
}

/// Audio parameters for public streams.
class PublicStreamingAudioConfig {
  /// Required. Bitrate. Positive integer in kbps. Options: 16, 32, or 64.
  int kBitrate;

  /// Required. Sample rate in Hz. Options: 16000, 32000, 44100, or 48000.
  int sampleRate;

  /// Required. Number of the audio channels:
  /// + `1`: Mono channel by default.
  /// + `2`: Dual channel.
  int channels;

  PublicStreamingAudioConfig(
      {this.kBitrate = 16, this.sampleRate = 44100, this.channels = 1});

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'kBitrate': kBitrate,
      'sampleRate': sampleRate,
      'channels': channels,
    };
  }
}

/// Public stream configuration.
class PublicStreaming {
  /// ID of the room from which the public stream is pushed.
  String roomId;

  /// Required. Video encoder configuration.
  PublicStreamingVideoConfig video;

  /// Required. Audio encoder configuration.
  PublicStreamingAudioConfig audio;

  /// Required. Public stream layout.
  PublicStreamingLayout layout;

  PublicStreaming({
    required this.roomId,
    required this.video,
    required this.audio,
    required this.layout,
  });

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'roomId': roomId,
      'video': video.toMap(),
      'audio': audio.toMap(),
      'layout': layout.toMap(),
    };
  }
}
